/*Copyright 2019-2021 T-Head Semiconductor Co., Ltd.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
#************************************************************
#*            Target file generated by rangen               *
#************************************************************
#*                                                          *
#************************************************************

######################################################
##########   MACRO DEFINE   
.macro MSTATUS_SUM IMM
  #write sum
  li    x9,0x40000
  csrc  mstatus,x9
 #csrr  x9,mstatus
  li    x10, \IMM
  slli  x10,x10,18
 # or    x9,x9,x10
  csrs  mstatus,x10
.endm

.macro MMU_EN
  #backup regs
  addi  x2, x2, -16
  sd    x9, 0(x2)
  sd    x10, 8(x2)

  #write MODE=8 to SATP
  csrr  x9,satp
  li    x10,0xfffffffffffffff
  and   x9,x9,x10
  li    x10,8
  slli  x10,x10,60
  or    x9,x9,x10
  csrw  satp, x9

  #restore regs
  ld    x10, 8(x2)
  ld    x9, 0(x2)
  addi  x2, x2, 16
.endm

.macro MMU_SATP_PPN PPN
  #backup regs
  addi  x2, x2, -16
  sd    x9,  0(x2)
  sd    x10, 8(x2)

  #write PPN
  csrr  x9, satp
  li    x10, 0xfffffffff0000000
  and   x9,x9,x10
  li    x10, \PPN
  or    x9,x9,x10
  csrw  satp,x9

  #restore regs
  ld    x10, 8(x2)
  ld    x9,  0(x2)
  addi  x2, x2, 8
.endm

.macro MMU_SATP_ASID ASID
  #backup regs
  addi  x2, x2, -16
  sd    x9,  0(x2)
  sd    x10, 8(x2)

  #write ASID
  csrr  x9, satp
  li    x10, 0xf0000fffffffffff
  and   x9,x9,x10
  li    x10, \ASID
  slli  x10,x10,44

  or    x9,x9,x10
  csrw  satp,x9

  #restore regs
  ld    x10, 8(x2)
  ld    x9,  0(x2)
  addi  x2, x2, 8
.endm



.macro MXSTATUS_MAEE IMM
  #write maee
  li    x9,0x200000
  csrc  mxstatus,x9
  csrr  x9,mxstatus
  li    x10, \IMM
  slli  x10,x10,21
  or    x9,x9,x10
  csrw  mxstatus,x9
.endm

.macro MMU_PTW_4K VPN, PPN, FLAG, THEADFLAG
  #backup regs
  addi  x2, x2, -88
  sd    x9, 0(x2)
  sd    x10, 8(x2)
  sd    x11, 16(x2)
  sd    x12, 24(x2)
  sd    x13, 32(x2)
  sd    x14, 40(x2)
  sd    x15, 48(x2)
  sd    x16, 56(x2)
  sd    x17, 64(x2)
  sd    x18, 72(x2)
  sd    x19, 80(x2)

  # get PPN from satp in x9
  csrr  x9, satp
  li    x10, 0xfffffffffff
  and   x9, x9, x10

  # get VPN2 in x10
  li    x10, \VPN
  li    x11, 0x7fc0000
  and   x10,x10,x11
  srli  x10, x10, 18

  # cfig first-level page
  # level-1 page, entry addr:{ppn,VPN2,3'b0} in x15
  slli  x14, x9, 12
  slli  x11, x10, 3
  add   x15, x14, x11
  # write pte into level-1 page
  # level-2 base addr in x12
  addi  x12, x10, 1
  add   x12, x12, x9
  slli  x14, x12, 10
  li    x13, 0xc1
  or    x13, x13, x14
  sd    x13, 0(x15)

  # cfig level-2 page
  # get VPN1 in x16
  li    x11, \VPN
  li    x13, 0x3fe00
  and   x16, x11, x13
  srli  x16, x16, 9
  # level-2 page, entry addr:{pte.ppn,VPN1,3'b0} in x17
  slli  x13, x12, 12
  slli  x17, x16, 3
  add   x17, x17, x13
  # write pte into level-2 page
  # level-3 base addr in x18: PPN+2+2^9+{vpn2,vpn1}
  li    x11, 0x200
  addi  x11, x11, 2
  add   x11, x11, x9
  li    x10, \VPN
  srli  x10, x10, 9
  add   x18, x11, x10
  slli  x19, x18, 10
  li    x13, 0xc1
  or    x19, x19, x13
  sd    x19, 0(x17)

  # cfig level-3 page
  # get VPN0 in x12
  li    x11, \VPN
  li    x12, 0x1ff
  and   x12, x12, x11
  # get level-3 page addr x17
  slli  x18, x18, 12
  slli  x12, x12, 3
  add   x17, x18, x12
  # write pte into level-3 page
  li    x11, \PPN
  li    x12, \THEADFLAG
  li    x13, \FLAG
  slli  x11, x11, 10
  slli  x12, x12, 59
  or    x11, x11, x12
  or    x11, x11, x13
  sd    x11, 0(x17)

  #restore regs
  ld    x9, 0(x2)
  ld    x10, 8(x2)
  ld    x11, 16(x2)
  ld    x12, 24(x2)
  ld    x13, 32(x2)
  ld    x14, 40(x2)
  ld    x15, 48(x2)
  ld    x16, 56(x2)
  ld    x17, 64(x2)
  ld    x18, 72(x2)
  ld    x19, 80(x2)
  addi  x2, x2, 88
  # fence
  fence
.endm

.macro MMU_MMODE_SMODE_S  SROUTINE
  #write sepc
  la x1,\SROUTINE
  csrw sepc,x1
  li x1,0x100
  csrrs x0,mstatus,x1
  sret
.endm

.macro MMODE_SMODE  SROUTINE
  #disable sie/mie
  li   x29, 0x2
  csrc mstatus,x29
  li   x29, 0x8
  csrc mstatus,x29
  #write mepc
  la x1,\SROUTINE
  csrw mepc,x1
  li x1,0x800
  csrrs x3,mstatus,x1
  li x1,0x1000
  csrrc x3,mstatus,x1
  mret
.endm

.text
.align 6
.global main
main:

######################################################
##########   Control regiter define   
      li x3,0x4
      csrs mhcr,x3
      li t0,0x8000
      csrs mxstatus,t0

# cfig satp
      sfence.vma x0,x0
      MMU_EN
      MMU_SATP_PPN  0x40
      MMU_SATP_ASID 0x1

# cfig PMP
# entry0: addr 0x0_0000_0000~0x2_ffff_ffff, r=1 w=1 x=1 l=0
# cfig pmpcfg0
     li   x1, 0x0f
     csrw pmpcfg0, x1

# cfig pmpaddr0
     li   x1, 0xc0000000
     csrw pmpaddr0, x1
     
     MSTATUS_SUM 1

      MXSTATUS_MAEE 1
      MMU_PTW_4K 0x0,0x0,0xff,0xf
      MMU_PTW_4K 0x1,0x1,0xff,0xf
      MMU_PTW_4K 0x2,0x2,0xff,0xf
      MMU_PTW_4K 0x3,0x3,0xff,0xf
      MMU_PTW_4K 0x4,0x4,0xff,0xf
      MMU_PTW_4K 0x5,0x5,0xff,0xf
      MMU_PTW_4K 0x6,0x6,0xff,0xf
      MMU_PTW_4K 0x7,0x7,0xff,0xf
      MMU_PTW_4K 0x8,0x8,0xff,0xf
      MMU_PTW_4K 0x9,0x9,0xff,0xf
      MMU_PTW_4K 0xa,0xa,0xff,0xf
      MMU_PTW_4K 0xb,0xb,0xff,0xf
      MMU_PTW_4K 0xc,0xc,0xff,0xf
      MMU_PTW_4K 0xd,0xd,0xff,0xf
      MMU_PTW_4K 0xe,0xe,0xff,0xf
      MMU_PTW_4K 0xf,0xf,0xff,0xf
      MMU_PTW_4K 0x10,0x10,0xff,0xf
      MMU_PTW_4K 0x11,0x11,0xff,0xf
      MMU_PTW_4K 0x12,0x12,0xff,0xf
      MMU_PTW_4K 0x13,0x13,0xff,0xf
      MMU_PTW_4K 0x14,0x14,0xff,0xf
      MMU_PTW_4K 0x15,0x15,0xff,0xf
      MMU_PTW_4K 0x16,0x16,0xff,0xf
      MMU_PTW_4K 0x17,0x17,0xff,0xf
      MMU_PTW_4K 0x18,0x18,0xff,0xf
      MMU_PTW_4K 0x19,0x19,0xff,0xf
      MMU_PTW_4K 0x1a,0x1a,0xff,0xf
      MMU_PTW_4K 0x1b,0x1b,0xff,0xf
      MMU_PTW_4K 0x1c,0x1c,0xff,0xf
      MMU_PTW_4K 0x1d,0x1d,0xff,0xf
      MMU_PTW_4K 0x1e,0x1e,0xff,0xf
      MMU_PTW_4K 0x1f,0x1f,0xff,0xf
      MMU_PTW_4K 0x100,0x100,0xef,0xf
      MMU_PTW_4K 0x101,0x101,0xef,0xf
      MMU_PTW_4K 0x102,0x102,0xef,0x7
      MMU_PTW_4K 0x103,0x103,0xef,0x7
      MMU_PTW_4K 0x104,0x104,0xef,0x17
      MMU_PTW_4K 0x105,0x105,0xef,0x17
      MMU_PTW_4K 0x106,0x106,0xef,0xd
      MMU_PTW_4K 0x107,0x107,0xef,0xd
      and x26,x9,x8      
      fence o,iorw      
      lhu x5,0x000000030(x22)      
      ld x5,0x000000038(x22)      
      lwu x5,0x00000003C(x22)      
      lwu x5,0x000000008(x22)      
      sd x4,0x000000038(x22)      
      sh x4,0x00000000E(x22)      
      sd x4,0x000000038(x22)      
      sh x4,0x00000001C(x22)      
      li x8, 0xffffffc0
######################################################
##########  4 core  
      csrr x3,mhartid
      li x4,0
      beq x4,x3,CPU0
      li x4,1
      beq x4,x3,CPU1
      li x4,2
      beq x4,x3,CPU2
      li x4,3
      beq x4,x3,CPU3
      wfi
.global CPU0
CPU0:
      MMODE_SMODE CPU0_S
.global CPU0_S
CPU0_S:
######################################################
########## every core will execute ld/st instruction
#   address can be non-cacheable/strong order/cachable/shreable 
      li x22,0x00010604E      
      and x22,x22,x8
      dcache.civa x22      
      sync.s      
      li x22,0x0001020BD      
      and x22,x22,x8
      li x4,0xAA22D819CDCF2E8A      
      lh x5,0x00000003E(x22)      
      li x22,0x00010015C      
      and x22,x22,x8
      li x4,0xB4F5D18ACFAF1531      
      ld x5,0x000000040(x22)      
      li x22,0x000104084      
      and x22,x22,x8
      li x4,0xF2ED3B17ACAB6C88      
      sb x4,0x000000026(x22)      
      li x22,0x00010419C      
      and x22,x22,x8
      li x4,0xDCC79DFE631CBC67      
      sh x4,0x00000002A(x22)      
      li x22,0x0001000C0      
      and x22,x22,x8
      li x4,0xB37B73E27D0C3DD1      
      dcache.civa x22      
      li x22,0x000102076      
      and x22,x22,x8
      li x4,0x9D83336F72A31D0E      
      sb x4,0x00000002D(x22)      
      li x22,0x00010000C      
      and x22,x22,x8
      li x4,0xD54EEC9496D77D33      
      sb x4,0x00000002E(x22)      
      li x22,0x0001061F2      
      and x22,x22,x8
      li x4,0x904298D6C13980E9      
      lb x5,0x00000003A(x22)      
      li x22,0x00010010F      
      and x22,x22,x8
      li x4,0x2A33098201C69D97      
      lw x5,0x000000034(x22)      
      li x22,0x0001041D9      
      and x22,x22,x8
      li x4,0x898BEB524FCE44C6      
      dcache.civa x22      
      li x22,0x00010016A      
      and x22,x22,x8
      li x4,0xD6F188CCBBA61FBF      
      sb x4,0x00000001B(x22)      
      li x22,0x0001020AF      
      and x22,x22,x8
      li x4,0x401702BA660AC540      
      ld x5,0x000000040(x22)      
      li x22,0x000106048      
      and x22,x22,x8
      li x4,0xB35DD799AC82EFFF      
      lbu x5,0x000000019(x22)      
      li x22,0x0001061F6      
      and x22,x22,x8
      li x4,0x4A53B9A9C1480A88      
      dcache.cva x22      
      li x22,0x00010402A      
      and x22,x22,x8
      li x4,0x5DB6B2A18FB1D1F3      
      dcache.cva x22      
      li x22,0x0001040C0      
      and x22,x22,x8
      li x4,0xE0EC4B9B70DAEAE0      
      sb x4,0x000000011(x22)      
      li x22,0x000106016      
      and x22,x22,x8
      li x4,0x8D0D9495790A20E1      
      sw x4,0x00000001C(x22)      
      li x22,0x0001061B3      
      and x22,x22,x8
      li x4,0xDBFEB4F7BC6DC238      
      lwu x5,0x000000048(x22)      
      li x22,0x0001041AD      
      and x22,x22,x8
      li x4,0x508282DD795A64F      
      sd x4,0x000000010(x22)      
      li x22,0x000102093      
      and x22,x22,x8
      li x4,0xE03F2A2E53AA6074      
      sw x4,0x000000034(x22)      
      li x22,0x00010011C      
      and x22,x22,x8
      li x4,0x57F21123902B0A12      
      lwu x5,0x000000020(x22)      
      li x22,0x0001000AF      
      and x22,x22,x8
      li x4,0x7C0BFCC3EF649ED7      
      sd x4,0x000000028(x22)      
      li x22,0x0001061DB      
      and x22,x22,x8
      li x4,0x76F7F078E548A641      
      lw x5,0x000000018(x22)      
      li x22,0x0001021BB      
      and x22,x22,x8
      li x4,0x3DFA74CB37939769      
      lbu x5,0x000000028(x22)      
      li x22,0x0001061F5      
      and x22,x22,x8
      li x4,0x6E01757F5341E10      
      sd x4,0x000000018(x22)      
      li x22,0x000104142      
      and x22,x22,x8
      li x4,0x2AA7B367B6A727D0      
      sb x4,0x00000003A(x22)      
      li x22,0x000104080      
      and x22,x22,x8
      li x4,0xA10F3C70F88B80DA      
      dcache.civa x22      
      li x22,0x00010416F      
      and x22,x22,x8
      li x4,0x82F966E96599404A      
      dcache.civa x22      
      li x22,0x0001061C3      
      and x22,x22,x8
      li x4,0x983FD369869567AE      
      sb x4,0x000000033(x22)      
      li x22,0x0001021CD      
      and x22,x22,x8
      li x4,0xAA1022B978F32BC0      
      lwu x5,0x00000000C(x22)      
      li x22,0x00010009B      
      and x22,x22,x8
      li x4,0xCB6086D5DECDB944      
      lbu x5,0x000000040(x22)      
      li x22,0x00010405C      
      and x22,x22,x8
      li x4,0x163E10063B75E98B      
      lw x5,0x000000030(x22)      
      li x22,0x0001041EF      
      and x22,x22,x8
      li x4,0x985F8274BBD8D6EA      
      lb x5,0x000000006(x22)      
      li x22,0x0001020B5      
      and x22,x22,x8
      li x4,0xFB47403184F5973D      
      sh x4,0x00000004C(x22)      
      li x22,0x0001041CE      
      and x22,x22,x8
      li x4,0x85F72F68ACE1CC19      
      sd x4,0x000000008(x22)      
      li x22,0x0001040ED      
      and x22,x22,x8
      li x4,0xB141C866E32DF1B2      
      sd x4,0x000000000(x22)      
      li x22,0x000106100      
      and x22,x22,x8
      li x4,0xDCA5BF0786D830F3      
      sh x4,0x000000050(x22)      
      li x22,0x0001001BA      
      and x22,x22,x8
      li x4,0x4A11DC7CBD069413      
      dcache.civa x22      
      li x22,0x000102135      
      and x22,x22,x8
      li x4,0xF67C83B1A12BE1AA      
      lw x5,0x000000014(x22)      
      li x22,0x0001060F3      
      and x22,x22,x8
      li x4,0x13EE47A0DA872F77      
      dcache.cva x22      
      li x22,0x00010404E      
      and x22,x22,x8
      li x4,0x6898FB0F5EFB6D89      
      lw x5,0x000000004(x22)      
      li x22,0x000100086      
      and x22,x22,x8
      li x4,0x3362B98A781EFC0C      
      dcache.civa x22      
      li x22,0x00010209D      
      and x22,x22,x8
      li x4,0xDFE9682CF4F97DCC      
      lbu x5,0x00000000A(x22)      
      li x22,0x000106000      
      and x22,x22,x8
      li x4,0x594DE2F5EFC5F7A2      
      dcache.cva x22      
      li x22,0x0001060C0      
      and x22,x22,x8
      li x4,0x18955573366D500A      
      dcache.cva x22      
      li x22,0x00010202F      
      and x22,x22,x8
      li x4,0xBFA60A18F3CB6BF      
      dcache.cva x22      
      li x22,0x00010604B      
      and x22,x22,x8
      li x4,0x834150F0279877DD      
      dcache.cva x22      
      li x22,0x0001060BB      
      and x22,x22,x8
      li x4,0xFFD69518E608C4D9      
      sw x4,0x000000020(x22)      
      li x22,0x0001040B0      
      and x22,x22,x8
      li x4,0xB302E19D48C32B3A      
      lwu x5,0x000000034(x22)      
      li x22,0x00010014C      
      and x22,x22,x8
      li x4,0x9BB27BEFE8B8A0FE      
      dcache.civa x22      
      li x22,0x000104166      
      and x22,x22,x8
      li x4,0xDA3517D041391FBE      
      sd x4,0x000000008(x22)      
      li x22,0x0001061C2      
      and x22,x22,x8
      li x4,0xFF16224DC7655B0E      
      sh x4,0x00000003A(x22)      
      li x22,0x000102130      
      and x22,x22,x8
      li x4,0xBFE8BE887D1E9A52      
      sh x4,0x000000034(x22)      
      li x22,0x000102000      
      and x22,x22,x8
      li x4,0xAE19D62AE7CA1E36      
      dcache.cva x22      
      li x22,0x00010612A      
      and x22,x22,x8
      li x4,0x846C17D5731DF837      
      dcache.cva x22      
      li x22,0x0001000EE      
      and x22,x22,x8
      li x4,0x30148B4E5828EA4A      
      lbu x5,0x000000004(x22)      
      li x22,0x0001040F1      
      and x22,x22,x8
      li x4,0x77CAD44299FD9149      
      dcache.civa x22      
      li x22,0x0001020C4      
      and x22,x22,x8
      li x4,0x6A688E27D5554829      
      dcache.cva x22      
      li x22,0x000100164      
      and x22,x22,x8
      li x4,0xA0E47D7CEC977BD1      
      sw x4,0x00000003C(x22)      
      li x22,0x0001040E0      
      and x22,x22,x8
      li x4,0x9F2A38C4D35C890D      
      sh x4,0x000000044(x22)      
      li x22,0x00010600F      
      and x22,x22,x8
      li x4,0x31BDBDE6A4426914      
      dcache.civa x22      
      li x22,0x00010619D      
      and x22,x22,x8
      li x4,0xCA7F21FE48567CBD      
      sh x4,0x000000014(x22)      
      li x22,0x000106081      
      and x22,x22,x8
      li x4,0xFE1766A462E4CA59      
      sw x4,0x000000024(x22)      
      li x22,0x0001021A9      
      and x22,x22,x8
      li x4,0x1A9B5C3E00E68E6F      
      sd x4,0x000000020(x22)      
      li x22,0x0001001CB      
      and x22,x22,x8
      li x4,0x2E27325FC325D8FC      
      ld x5,0x000000028(x22)      
      li x22,0x00010619A      
      and x22,x22,x8
      li x4,0xF6CD82B9CB696AF8      
      dcache.civa x22      
      li x22,0x0001000BE      
      and x22,x22,x8
      li x4,0x72584BEC03CF5109      
      sd x4,0x000000048(x22)      
      li x22,0x0001060D0      
      and x22,x22,x8
      li x4,0xFE4BBD50D8E7664D      
      sd x4,0x000000038(x22)      
      li x22,0x0001040C0      
      and x22,x22,x8
      li x4,0xE2E556D296D545EF      
      lb x5,0x000000042(x22)      
      li x22,0x00010210D      
      and x22,x22,x8
      li x4,0x9613E6A4B5F07A24      
      sd x4,0x000000040(x22)      
      li x22,0x000104200      
      and x22,x22,x8
      li x4,0x459616F07EB9B1D9      
      dcache.cva x22      
      li x22,0x00010403D      
      and x22,x22,x8
      li x4,0xC252A4BF72C90868      
      lwu x5,0x000000000(x22)      
      li x22,0x000106124      
      and x22,x22,x8
      li x4,0xDCCF30CD4261E608      
      dcache.cva x22      
      li x22,0x00010418B      
      and x22,x22,x8
      li x4,0x89C19835783061E0      
      lb x5,0x000000047(x22)      
      li x22,0x0001040A6      
      and x22,x22,x8
      li x4,0xAD1E48EC8337E8C4      
      lwu x5,0x000000030(x22)      
      li x22,0x00010608E      
      and x22,x22,x8
      li x4,0x11B3C880A479863F      
      dcache.civa x22      
      li x22,0x000106102      
      and x22,x22,x8
      li x4,0x8FB60E30514228B3      
      dcache.cva x22      
      li x22,0x000102160      
      and x22,x22,x8
      li x4,0x931263798DDC60A9      
      sb x4,0x000000006(x22)      
      li x22,0x000102004      
      and x22,x22,x8
      li x4,0x279A755FF6DD364      
      lbu x5,0x00000003E(x22)      
      li x22,0x00010015E      
      and x22,x22,x8
      li x4,0xE968EC56A7DA3435      
      ld x5,0x000000040(x22)      
      li x22,0x000104153      
      and x22,x22,x8
      li x4,0x71AC9CA3534BBA8C      
      dcache.civa x22      
      li x22,0x00010006E      
      and x22,x22,x8
      li x4,0xF240E9989B642462      
      sd x4,0x000000020(x22)      
      li x22,0x000102062      
      and x22,x22,x8
      li x4,0x4292271B343890CB      
      dcache.civa x22      
      li x22,0x0001041FE      
      and x22,x22,x8
      li x4,0x887D8D44FB546DCE      
      lw x5,0x000000040(x22)      
      li x22,0x0001040EF      
      and x22,x22,x8
      li x4,0xF835DC5A09513224      
      dcache.civa x22      
      li x22,0x000100056      
      and x22,x22,x8
      li x4,0x1440FBEB00FAD65F      
      sw x4,0x000000004(x22)      
      li x22,0x00010016D      
      and x22,x22,x8
      li x4,0x59086EA32514FBD0      
      dcache.civa x22      
      li x22,0x000104183      
      and x22,x22,x8
      li x4,0x4DBF6C17A0F58E3C      
      dcache.cva x22      
      li x22,0x00010200F      
      and x22,x22,x8
      li x4,0x19A69FB4B7836EE      
      lh x5,0x00000004C(x22)      
      li x22,0x000102150      
      and x22,x22,x8
      li x4,0xE925A9F4FDA9687F      
      sb x4,0x000000014(x22)      
      li x22,0x000102076      
      and x22,x22,x8
      li x4,0xA19D6BC20656C310      
      lhu x5,0x000000014(x22)      
      li x22,0x0001020D9      
      and x22,x22,x8
      li x4,0x538C7072FAF314E0      
      lhu x5,0x000000028(x22)      
      li x22,0x00010606D      
      and x22,x22,x8
      li x4,0x4C90F945CECFBEFD      
      sd x4,0x000000010(x22)      
      li x22,0x0001060F0      
      and x22,x22,x8
      li x4,0xC3762A4A2411DE3E      
      sb x4,0x000000049(x22)      
      li x22,0x000102062      
      and x22,x22,x8
      li x4,0xB5447CF55F224C90      
      lh x5,0x000000044(x22)      
      li x22,0x0001040FC      
      and x22,x22,x8
      li x4,0xC559C19FC2BAFF1D      
      dcache.cva x22      
      li x22,0x00010017D      
      and x22,x22,x8
      li x4,0x6DABE6971E81A4A4      
      sb x4,0x000000007(x22)      
      li x22,0x000104044      
      and x22,x22,x8
      li x4,0xB662041CEF7458CC      
      sh x4,0x000000016(x22)      
      li x22,0x000100166      
      and x22,x22,x8
      li x4,0x1646E0434DE79C4      
      sb x4,0x00000003E(x22)      
      li x22,0x0001020C4      
      and x22,x22,x8
      li x4,0xF7CAC52DC9AB646      
      dcache.cva x22      
      li x22,0x0001060AF      
      and x22,x22,x8
      li x4,0xDFF1DFC55CE34226      
      sd x4,0x000000000(x22)      
      li x22,0x000102060      
      and x22,x22,x8
      li x4,0x205B4F6C82A8038D      
      dcache.civa x22      
      li x22,0x0001000DE      
      and x22,x22,x8
      li x4,0x9D6989DBD7C15FF3      
      dcache.cva x22      
      li x22,0x000104094      
      and x22,x22,x8
      li x4,0x917FAB094AA742DD      
      dcache.civa x22      
      li x22,0x0001001AF      
      and x22,x22,x8
      li x4,0x4D930B9E7133C27F      
      ld x5,0x000000008(x22)      
      li x22,0x00010402E      
      and x22,x22,x8
      li x4,0xB8E2170CB39D8C6D      
      dcache.civa x22      
      li x22,0x000104160      
      and x22,x22,x8
      li x4,0xA5F77D5B60EC075C      
      dcache.cva x22      
      li x22,0x000100168      
      and x22,x22,x8
      li x4,0xD9BEAD4130440B11      
      ld x5,0x000000028(x22)      
      li x22,0x00010006C      
      and x22,x22,x8
      li x4,0xA0F067B889200853      
      sw x4,0x000000000(x22)      
      li x22,0x000100142      
      and x22,x22,x8
      li x4,0x2B75200DAAC1852E      
      sw x4,0x000000014(x22)      
      li x22,0x0001021E8      
      and x22,x22,x8
      li x4,0x153A4C700EBE8C18      
      sd x4,0x000000008(x22)      
      li x22,0x0001061E6      
      and x22,x22,x8
      li x4,0x5ABDF2EFF2EBFFC      
      lhu x5,0x00000004C(x22)      
      li x22,0x000102199      
      and x22,x22,x8
      li x4,0x20DFE70FF45EB336      
      lwu x5,0x000000014(x22)      
      li x22,0x0001040B4      
      and x22,x22,x8
      li x4,0x7CD5650E82DA056B      
      lhu x5,0x00000004E(x22)      
      li x22,0x00010400F      
      and x22,x22,x8
      li x4,0xE2BE1A459E14BD5E      
      sh x4,0x000000018(x22)      
      li x22,0x0001041E2      
      and x22,x22,x8
      li x4,0xAE6F69F4C143F42D      
      lwu x5,0x000000024(x22)      
      li x22,0x00010618F      
      and x22,x22,x8
      li x4,0x432BD0C49A78477D      
      sw x4,0x000000008(x22)      
      li x22,0x0001001DA      
      and x22,x22,x8
      li x4,0xB4B1F759CCAF2458      
      sb x4,0x000000036(x22)      
      li x22,0x000100094      
      and x22,x22,x8
      li x4,0x5AD7EF13430DF0BF      
      dcache.civa x22      
      li x22,0x0001060C9      
      and x22,x22,x8
      li x4,0xD86260911B4A6CA3      
      dcache.cva x22      
      li x22,0x0001060BC      
      and x22,x22,x8
      li x4,0x5D2F0A989518F888      
      lbu x5,0x00000004F(x22)      
      li x22,0x00010011A      
      and x22,x22,x8
      li x4,0x4653C3FEA903156F      
      lhu x5,0x000000014(x22)      
      li x22,0x000100156      
      and x22,x22,x8
      li x4,0xC4A4E0415411C968      
      sb x4,0x00000002F(x22)      
      li x22,0x00010614F      
      and x22,x22,x8
      li x4,0x554E5607D156047F      
      dcache.civa x22      
      li x22,0x00010400C      
      and x22,x22,x8
      li x4,0x2D1223C212AF0FC8      
      sd x4,0x000000018(x22)      
      li x22,0x0001041C6      
      and x22,x22,x8
      li x4,0x2549AD682F37E551      
      sb x4,0x00000003B(x22)      
      li x22,0x0001021AB      
      and x22,x22,x8
      li x4,0x58172B7C6C392DEA      
      lbu x5,0x000000012(x22)      
      li x22,0x000106101      
      and x22,x22,x8
      li x4,0xD0AAF519A698476      
      lw x5,0x000000048(x22)      
      li x22,0x0001060D5      
      and x22,x22,x8
      li x4,0xCD2926D4D2EC84C3      
      dcache.cva x22      
      li x22,0x0001020AC      
      and x22,x22,x8
      li x4,0x802C924E3347C3FA      
      sh x4,0x000000048(x22)      
      li x22,0x0001041BD      
      and x22,x22,x8
      li x4,0x4FF51C1962456C5B      
      sh x4,0x000000026(x22)      
      li x22,0x00010007C      
      and x22,x22,x8
      li x4,0x71D9C78B3CC48CDB      
      sh x4,0x00000003C(x22)      
      li x22,0x0001001C2      
      and x22,x22,x8
      li x4,0x31F949144BE30B76      
      sw x4,0x000000038(x22)      
      li x22,0x000100075      
      and x22,x22,x8
      li x4,0x20C426AF4435AFA4      
      lb x5,0x00000002D(x22)      
      li x22,0x000106185      
      and x22,x22,x8
      li x4,0x97B8826611F51B2A      
      lb x5,0x00000002F(x22)      
      li x22,0x0001061FA      
      and x22,x22,x8
      li x4,0xFCD080DA37D9914F      
      sb x4,0x00000001A(x22)      
      li x22,0x0001060EA      
      and x22,x22,x8
      li x4,0xEA2FA5133CB3B73B      
      sd x4,0x000000028(x22)      
      li x22,0x00010208A      
      and x22,x22,x8
      li x4,0xE27FCC5BE76AFE94      
      dcache.civa x22      
      li x22,0x000106128      
      and x22,x22,x8
      li x4,0xB0A19A055EB4B0F0      
      lbu x5,0x000000013(x22)      
      li x22,0x0001041CC      
      and x22,x22,x8
      li x4,0x67022BFEAADD11F2      
      dcache.civa x22      
      li x22,0x00010207F      
      and x22,x22,x8
      li x4,0x428118AE8686A51      
      dcache.civa x22      
      li x22,0x00010615E      
      and x22,x22,x8
      li x4,0x503A6D89972E4CAD      
      ld x5,0x000000010(x22)      
      li x22,0x00010216C      
      and x22,x22,x8
      li x4,0x440C94424938FBCE      
      dcache.cva x22      
      li x22,0x00010218D      
      and x22,x22,x8
      li x4,0x8C4F4EC192E3718C      
      dcache.cva x22      
      li x22,0x000102104      
      and x22,x22,x8
      li x4,0xA30C97FB88EAE273      
      dcache.civa x22      
      li x22,0x0001001F8      
      and x22,x22,x8
      li x4,0x5371E9EF84FAFD55      
      dcache.civa x22      
      li x22,0x0001060C5      
      and x22,x22,x8
      li x4,0xC3FB120785C714A      
      dcache.civa x22      
      li x22,0x000104044      
      and x22,x22,x8
      li x4,0x6A9C47C97A0AAE8A      
      sw x4,0x000000000(x22)      
      li x22,0x0001061BA      
      and x22,x22,x8
      li x4,0x86D5B0F8C911B999      
      dcache.civa x22      
      li x22,0x00010411D      
      and x22,x22,x8
      li x4,0x3BC6815AED4649A4      
      sh x4,0x00000000A(x22)      
      li x22,0x000100163      
      and x22,x22,x8
      li x4,0x72F5738D665C72A2      
      sw x4,0x000000048(x22)      
      li x22,0x000106113      
      and x22,x22,x8
      li x4,0x79DF676FBD4F80E4      
      sb x4,0x000000007(x22)      
      li x22,0x00010600C      
      and x22,x22,x8
      li x4,0xC30B491A0FE623A9      
      sd x4,0x000000038(x22)      
      li x22,0x0001040BD      
      and x22,x22,x8
      li x4,0xADD97CE8EA1C911A      
      sd x4,0x000000038(x22)      
      li x22,0x00010004D      
      and x22,x22,x8
      li x4,0x77710947DA6E2E91      
      lwu x5,0x000000018(x22)      
      li x22,0x0001060F3      
      and x22,x22,x8
      li x4,0x78E115937DC57E6A      
      sb x4,0x00000000A(x22)      
      li x22,0x00010400F      
      and x22,x22,x8
      li x4,0x9EF62B912E89B9AA      
      sw x4,0x000000020(x22)      
      li x22,0x000102122      
      and x22,x22,x8
      li x4,0x60CB0BB768BD6731      
      ld x5,0x000000020(x22)      
      li x22,0x000104035      
      and x22,x22,x8
      li x4,0xF15411B65D052869      
      sw x4,0x000000024(x22)      
      li x22,0x0001040D8      
      and x22,x22,x8
      li x4,0x34A6D658EF63E03D      
      sw x4,0x000000040(x22)      
      li x22,0x0001060DF      
      and x22,x22,x8
      li x4,0xAD10E0A7883A352C      
      sh x4,0x000000004(x22)      
      li x22,0x0001001E5      
      and x22,x22,x8
      li x4,0xF88E1C4785D8DF17      
      lbu x5,0x000000030(x22)      
      li x22,0x00010215A      
      and x22,x22,x8
      li x4,0x6D0C8285587D5C01      
      dcache.cva x22      
      li x22,0x000106017      
      and x22,x22,x8
      li x4,0xD3DA7D8D503342B8      
      sd x4,0x000000000(x22)      
      li x22,0x00010400A      
      and x22,x22,x8
      li x4,0x70C2DB58037A8B3B      
      sw x4,0x000000000(x22)      
      li x22,0x00010011E      
      and x22,x22,x8
      li x4,0x800F249DC77ED8CA      
      dcache.cva x22      
      li x22,0x000102089      
      and x22,x22,x8
      li x4,0x769F939B7EFA424E      
      lhu x5,0x000000026(x22)      
      li x22,0x000102170      
      and x22,x22,x8
      li x4,0x5218034902BF5FD0      
      sd x4,0x000000008(x22)      
      li x22,0x00010600A      
      and x22,x22,x8
      li x4,0x7C3423EF1C38D568      
      lhu x5,0x000000004(x22)      
      li x22,0x0001001DF      
      and x22,x22,x8
      li x4,0x595A4C3CD2FE91DE      
      dcache.civa x22      
      li x22,0x0001000CA      
      and x22,x22,x8
      li x4,0x929659F8F4324694      
      lw x5,0x000000014(x22)      
      li x22,0x00010400C      
      and x22,x22,x8
      li x4,0x18C10187DF6086BF      
      dcache.cva x22      
      li x22,0x0001000E8      
      and x22,x22,x8
      li x4,0x38A235F3E2C08FD9      
      sh x4,0x00000003E(x22)      
      li x22,0x0001020BF      
      and x22,x22,x8
      li x4,0x42EC870C1302B150      
      sh x4,0x00000003C(x22)      
      li x22,0x00010206C      
      and x22,x22,x8
      li x4,0x98A8DF0029B360AD      
      lhu x5,0x00000004A(x22)      
      li x22,0x000100094      
      and x22,x22,x8
      li x4,0x1E92CFEAE41B57EB      
      sd x4,0x000000030(x22)      
      li x22,0x0001000B9      
      and x22,x22,x8
      li x4,0xBDC5CF82061F4985      
      dcache.cva x22      
      li x22,0x00010009C      
      and x22,x22,x8
      li x4,0x85650BDCB85FC1B2      
      lwu x5,0x000000044(x22)      
      li x22,0x000100001      
      and x22,x22,x8
      li x4,0xB40D93486A68C3CE      
      dcache.cva x22      
      li x22,0x00010211A      
      and x22,x22,x8
      li x4,0xB8EEA40C8104DD84      
      sb x4,0x00000002D(x22)      
      li x22,0x000106022      
      and x22,x22,x8
      li x4,0x3017F2F6928BC2CA      
      dcache.civa x22      
      li x22,0x00010204F      
      and x22,x22,x8
      li x4,0x6CDC9FBCD77E9C86      
      sh x4,0x000000042(x22)      
      li x22,0x0001040F2      
      and x22,x22,x8
      li x4,0x6E96AC630D3ED9D      
      dcache.civa x22      
      li x22,0x0001041B9      
      and x22,x22,x8
      li x4,0xBBB9E83322D8B92C      
      sd x4,0x000000038(x22)      
      li x22,0x0001021AA      
      and x22,x22,x8
      li x4,0x3395E2BFDA1AFBBF      
      sb x4,0x000000041(x22)      
      li x22,0x0001041FF      
      and x22,x22,x8
      li x4,0x83A942F2C3BE28DA      
      dcache.civa x22      
      li x22,0x0001021FA      
      and x22,x22,x8
      li x4,0xE7303239912FDB3F      
      dcache.civa x22      
      li x22,0x00010016C      
      and x22,x22,x8
      li x4,0x28F72F7FF1326347      
      lbu x5,0x000000026(x22)      
      li x22,0x000106110      
      and x22,x22,x8
      li x4,0xBA95FD10F36DFCC2      
      dcache.cva x22      
      li x22,0x0001021DA      
      and x22,x22,x8
      li x4,0xEB20EEFAC035E9AC      
      sh x4,0x00000004C(x22)      
      li x22,0x0001000E4      
      and x22,x22,x8
      li x4,0xE93499AD8BDF961      
      sd x4,0x000000008(x22)      
      li x22,0x00010200B      
      and x22,x22,x8
      li x4,0x33A52F3DC5BEB551      
      ld x5,0x000000028(x22)      
      li x22,0x0001041F1      
      and x22,x22,x8
      li x4,0x2BF8346B1E58F47D      
      ld x5,0x000000018(x22)      
      li x22,0x000106166      
      and x22,x22,x8
      li x4,0x5A2E38FA3E537C87      
      lh x5,0x000000024(x22)      
      li x22,0x00010613D      
      and x22,x22,x8
      li x4,0xBC2FD60A7B27C036      
      dcache.cva x22      
      li x22,0x0001061EA      
      and x22,x22,x8
      li x4,0xA8562F3C3B182863      
      dcache.civa x22      
      li x22,0x000106067      
      and x22,x22,x8
      li x4,0x9CFB3B5339D022B9      
      lhu x5,0x000000012(x22)      
      li x22,0x0001000EF      
      and x22,x22,x8
      li x4,0xB59177E003C4BB6A      
      sw x4,0x000000048(x22)      
      li x22,0x000102102      
      and x22,x22,x8
      li x4,0x7E9B77B45C21A853      
      lwu x5,0x000000020(x22)      
      li x22,0x00010606D      
      and x22,x22,x8
      li x4,0x3E6C34C10502F560      
      sh x4,0x000000030(x22)      
      li x22,0x0001020A8      
      and x22,x22,x8
      li x4,0x4ADA2D676C169D22      
      sd x4,0x000000038(x22)      
      li x22,0x0001000C0      
      and x22,x22,x8
      li x4,0x244094FE5DDFD30A      
      dcache.cva x22      
      li x22,0x000102128      
      and x22,x22,x8
      li x4,0x86208B8CA7B51634      
      sb x4,0x000000000(x22)      
      li x22,0x0001041EB      
      and x22,x22,x8
      li x4,0x867D93F8B4CE47B4      
      sd x4,0x000000010(x22)      
      li x22,0x00010604A      
      and x22,x22,x8
      li x4,0x262A18B3A1EA6479      
      sw x4,0x000000018(x22)      
      li x22,0x00010219F      
      and x22,x22,x8
      li x4,0x37B0CEFC3FC16769      
      lb x5,0x000000034(x22)      
      li x22,0x0001021F7      
      and x22,x22,x8
      li x4,0x260B58DB2B422DF3      
      dcache.civa x22      
      li x22,0x00010601B      
      and x22,x22,x8
      li x4,0x49398AF6817EE30C      
      dcache.cva x22      
      li x22,0x0001061EC      
      and x22,x22,x8
      li x4,0x9A1C4554916071D6      
      lbu x5,0x00000003B(x22)      
      li x22,0x000100102      
      and x22,x22,x8
      li x4,0xA2AEE20251ED51AA      
      dcache.civa x22      
      li x22,0x00010207A      
      and x22,x22,x8
      li x4,0xCEB2C2050161FF82      
      dcache.cva x22      
      li x22,0x0001021A7      
      and x22,x22,x8
      li x4,0x425DD2696C3C024E      
      dcache.cva x22      
      li x22,0x000102069      
      and x22,x22,x8
      li x4,0x6F8BFF8D80D04837      
      dcache.civa x22      
      li x22,0x000104143      
      and x22,x22,x8
      li x4,0x49CDB5F026AADA46      
      lwu x5,0x000000044(x22)      
      li x22,0x0001001C9      
      and x22,x22,x8
      li x4,0xA7BDAE13AD734C8E      
      dcache.cva x22      
      li x22,0x0001040AC      
      and x22,x22,x8
      li x4,0x2355158A3068D60A      
      sd x4,0x000000020(x22)      
      li x22,0x000106000      
      and x22,x22,x8
      li x4,0x9CE85E06667F1E3B      
      dcache.cva x22      
      li x22,0x000104144      
      and x22,x22,x8
      li x4,0xB488C96EE211721F      
      sh x4,0x000000002(x22)      
      li x22,0x000104054      
      and x22,x22,x8
      li x4,0x54A94BD5A6590FB0      
      sb x4,0x000000013(x22)      
      li x22,0x00010003D      
      and x22,x22,x8
      li x4,0x99C59AE66CE01553      
      ld x5,0x000000028(x22)      
      li x22,0x0001020AE      
      and x22,x22,x8
      li x4,0x6C307C49D49D0E8E      
      lwu x5,0x000000008(x22)      
      li x22,0x000102078      
      and x22,x22,x8
      li x4,0x78BEC09DED4B13D3      
      lb x5,0x00000001E(x22)      
      li x22,0x0001000ED      
      and x22,x22,x8
      li x4,0x4BD5B98BB4020D98      
      dcache.cva x22      
      li x22,0x00010618E      
      and x22,x22,x8
      li x4,0xDD7B4E554CD2D128      
      lb x5,0x000000020(x22)      
      li x22,0x0001041A1      
      and x22,x22,x8
      li x4,0xA5BA8C05F923F521      
      sw x4,0x000000004(x22)      
      li x22,0x000102132      
      and x22,x22,x8
      li x4,0x37C049C1595ACB81      
      sw x4,0x000000040(x22)      
      li x22,0x000104188      
      and x22,x22,x8
      li x4,0x8541DA4285A470A6      
      sd x4,0x000000048(x22)      
      li x22,0x00010608A      
      and x22,x22,x8
      li x4,0x7E7ECEDD1B012FB6      
      sd x4,0x000000018(x22)      
      li x22,0x00010214E      
      and x22,x22,x8
      li x4,0x706F961B9E0BCF53      
      dcache.cva x22      
      li x22,0x0001061AF      
      and x22,x22,x8
      li x4,0x68A550ECBFD5D725      
      lb x5,0x000000029(x22)      
      li x22,0x000100069      
      and x22,x22,x8
      li x4,0x5CA2419939561279      
      sh x4,0x000000010(x22)      
      li x22,0x00010600D      
      and x22,x22,x8
      li x4,0xDB55BDA8A5411BA0      
      dcache.cva x22      
      li x22,0x000104039      
      and x22,x22,x8
      li x4,0xCA2812313D1133EF      
      dcache.cva x22      
      li x22,0x000100042      
      and x22,x22,x8
      li x4,0xCE20F5D4F1A8179F      
      sb x4,0x000000037(x22)      
      li x22,0x00010603B      
      and x22,x22,x8
      li x4,0x420BF57DEA3C464F      
      lwu x5,0x000000030(x22)      
      li x22,0x000106137      
      and x22,x22,x8
      li x4,0xB87B430B783602C2      
      lhu x5,0x000000034(x22)      
      li x22,0x00010418C      
      and x22,x22,x8
      li x4,0xD80F1CD8781D1672      
      lhu x5,0x000000048(x22)      
      li x22,0x00010604A      
      and x22,x22,x8
      li x4,0x51407515947F61A0      
      dcache.cva x22      
      li x22,0x00010017A      
      and x22,x22,x8
      li x4,0x67E1DA1E5051DA4      
      dcache.cva x22      
      li x22,0x00010619F      
      and x22,x22,x8
      li x4,0x81C3A9D5D3F0C678      
      lhu x5,0x000000030(x22)      
      li x22,0x0001060F4      
      and x22,x22,x8
      li x4,0xC2CCE4D7F4CC3507      
      dcache.civa x22      
      li x22,0x00010204E      
      and x22,x22,x8
      li x4,0x7A8890BD5F74210C      
      dcache.cva x22      
      li x22,0x0001020BB      
      and x22,x22,x8
      li x4,0xE335AC74F355FEC      
      sw x4,0x000000028(x22)      
      li x22,0x0001061EC      
      and x22,x22,x8
      li x4,0x199A36FBDA84EF9F      
      lbu x5,0x000000040(x22)      
      li x22,0x0001061B2      
      and x22,x22,x8
      li x4,0x1BA03900C1951E69      
      dcache.civa x22      
      li x22,0x0001001F5      
      and x22,x22,x8
      li x4,0xA1DBC5B2B13573D      
      dcache.cva x22      
      li x22,0x0001061DB      
      and x22,x22,x8
      li x4,0x9847B4D914A50EC8      
      sh x4,0x000000014(x22)      
      li x22,0x000106098      
      and x22,x22,x8
      li x4,0x589D2E2A52048B73      
      lhu x5,0x000000018(x22)      
      li x22,0x0001040EB      
      and x22,x22,x8
      li x4,0x699921AF394CF986      
      lhu x5,0x00000004E(x22)      
      li x22,0x000102078      
      and x22,x22,x8
      li x4,0x8C05D6A70F1E397      
      sw x4,0x000000040(x22)      
      li x22,0x00010206F      
      and x22,x22,x8
      li x4,0x52069BACA00D310B      
      sb x4,0x000000029(x22)      
      li x22,0x000100154      
      and x22,x22,x8
      li x4,0xD8F6FA7E5925C9E2      
      sb x4,0x00000000D(x22)      
      li x22,0x0001021FF      
      and x22,x22,x8
      li x4,0x52B2B4741E7CA4B6      
      lb x5,0x000000036(x22)      
      li x22,0x00010017E      
      and x22,x22,x8
      li x4,0xD8C96BBC4CF5868F      
      lb x5,0x000000030(x22)      
      li x22,0x00010011D      
      and x22,x22,x8
      li x4,0xC4F4D65AD7457759      
      dcache.civa x22      
      li x22,0x0001001C8      
      and x22,x22,x8
      li x4,0xAB3F63F98E20BB11      
      lw x5,0x000000040(x22)      
      li x22,0x00010418A      
      and x22,x22,x8
      li x4,0xC2ADD4A3C82FCA96      
      sb x4,0x000000040(x22)      
      li x22,0x00010019F      
      and x22,x22,x8
      li x4,0xDBC98AAA90D5DD50      
      lb x5,0x00000002F(x22)      
      li x22,0x0001020E3      
      and x22,x22,x8
      li x4,0xFE2D65B12EBAE171      
      lhu x5,0x000000036(x22)      
      li x22,0x0001060BF      
      and x22,x22,x8
      li x4,0x379F6B9199335FFA      
      dcache.cva x22      
      li x22,0x00010019B      
      and x22,x22,x8
      li x4,0x3AF6E0BD28B07CF3      
      sw x4,0x000000038(x22)      
      li x22,0x0001040EF      
      and x22,x22,x8
      li x4,0xBCC82B163D42ED40      
      dcache.cva x22      
      li x22,0x0001061A9      
      and x22,x22,x8
      li x4,0x6F2C7C1296E94AA5      
      lwu x5,0x000000048(x22)      
      li x22,0x000100133      
      and x22,x22,x8
      li x4,0x73949015A2139293      
      dcache.civa x22      
      li x22,0x000100119      
      and x22,x22,x8
      li x4,0x9D9A99C0047C1B64      
      sb x4,0x000000038(x22)      
      li x22,0x0001020EC      
      and x22,x22,x8
      li x4,0xBF0EC2D816B820A4      
      lbu x5,0x000000015(x22)      
      li x22,0x000100021      
      and x22,x22,x8
      li x4,0xCAD4C90032C18910      
      sd x4,0x000000040(x22)      
      li x22,0x000104157      
      and x22,x22,x8
      li x4,0xF5887833644BFE88      
      dcache.cva x22      
      li x22,0x00010607C      
      and x22,x22,x8
      li x4,0xC0906637311F8CA6      
      sd x4,0x000000030(x22)      
      li x22,0x000100027      
      and x22,x22,x8
      li x4,0xCD4C5F1D3308EFC1      
      dcache.cva x22      
      li x22,0x00010614A      
      and x22,x22,x8
      li x4,0x695A92E8BDDC8DE1      
      sw x4,0x000000004(x22)      
      li x22,0x000106179      
      and x22,x22,x8
      li x4,0xFF66AAB0CF47B836      
      dcache.cva x22      
      li x22,0x0001061BA      
      and x22,x22,x8
      li x4,0x4FEF664D7D1F9C38      
      lw x5,0x000000018(x22)      
      li x22,0x0001040DB      
      and x22,x22,x8
      li x4,0x8A41D1CE0B5E08E6      
      lb x5,0x000000012(x22)      
      li x22,0x0001001F9      
      and x22,x22,x8
      li x4,0xA1C5DD954A3EF088      
      sw x4,0x000000030(x22)      
      li x22,0x000102170      
      and x22,x22,x8
      li x4,0xAEB448C1B300FFB4      
      sb x4,0x000000004(x22)      
      li x22,0x0001061F8      
      and x22,x22,x8
      li x4,0x670ABBADBD727303      
      ld x5,0x000000010(x22)      
      li x22,0x00010001A      
      and x22,x22,x8
      li x4,0x214EB09E40C2A61A      
      dcache.cva x22      
      li x22,0x0001021F7      
      and x22,x22,x8
      li x4,0xFE63C16EE74C2E46      
      dcache.civa x22      
      li x22,0x00010214A      
      and x22,x22,x8
      li x4,0x2FEB3B0D2DAD4FD7      
      sd x4,0x000000038(x22)      
      li x22,0x000106130      
      and x22,x22,x8
      li x4,0xE354BACCD4DFD6D7      
      sb x4,0x00000001A(x22)      
      li x22,0x0001020C5      
      and x22,x22,x8
      li x4,0x266B1A6D5519675D      
      lbu x5,0x00000004B(x22)      
      li x22,0x00010219C      
      and x22,x22,x8
      li x4,0xA95B30106F828102      
      sb x4,0x00000000B(x22)      
      li x22,0x000100045      
      and x22,x22,x8
      li x4,0xC0D3D81B31D06DF0      
      lbu x5,0x000000018(x22)      
      li x22,0x00010418A      
      and x22,x22,x8
      li x4,0xAFEB47AE7707DC99      
      ld x5,0x000000048(x22)      
      li x22,0x00010214D      
      and x22,x22,x8
      li x4,0x6EB6ADDB5BE2E4F8      
      dcache.civa x22      
      li x22,0x000100046      
      and x22,x22,x8
      li x4,0x9CCE6399050175EA      
      dcache.civa x22      
      li x22,0x00010418C      
      and x22,x22,x8
      li x4,0xA2C2ECD40D0A5375      
      lw x5,0x00000003C(x22)      
      li x22,0x000106098      
      and x22,x22,x8
      li x4,0x74AF6058B71EB0C1      
      sb x4,0x000000015(x22)      
      li x22,0x00010017F      
      and x22,x22,x8
      li x4,0x5CE786613B349C9F      
      ld x5,0x000000020(x22)      
      li x22,0x00010006A      
      and x22,x22,x8
      li x4,0x7C87FA5A0F413A21      
      dcache.civa x22      
      li x22,0x000102018      
      and x22,x22,x8
      li x4,0xFADD8C168E8703CB      
      sb x4,0x00000001B(x22)      
      li x22,0x0001060B4      
      and x22,x22,x8
      li x4,0x46BED8C11CACCAE0      
      lhu x5,0x000000044(x22)      
      li x22,0x000104038      
      and x22,x22,x8
      li x4,0x13F82D35DF963693      
      lbu x5,0x000000048(x22)      
      li x22,0x000102132      
      and x22,x22,x8
      li x4,0xB086DC73FDA054EA      
      lhu x5,0x00000004A(x22)      
      li x22,0x000104004      
      and x22,x22,x8
      li x4,0xF1A23D76E3B6D81C      
      dcache.civa x22      
      li x22,0x000104097      
      and x22,x22,x8
      li x4,0x838D6D17D2C5DB18      
      dcache.cva x22      
      li x22,0x00010602F      
      and x22,x22,x8
      li x4,0x118F74BBF5BDF53C      
      lw x5,0x00000002C(x22)      
      li x22,0x000102167      
      and x22,x22,x8
      li x4,0x1BC9B64442FE6C05      
      lw x5,0x000000010(x22)      
      li x22,0x000100008      
      and x22,x22,x8
      li x4,0x9617C8D90F551FCA      
      lb x5,0x00000000E(x22)      
      li x22,0x0001021EB      
      and x22,x22,x8
      li x4,0x80BE4239A105127B      
      dcache.civa x22      
      li x22,0x0001001A2      
      and x22,x22,x8
      li x4,0xECBD342130B2B4B6      
      dcache.cva x22      
      li x22,0x0001040F2      
      and x22,x22,x8
      li x4,0x3D4F9437D1053878      
      lhu x5,0x000000034(x22)      
      li x22,0x0001001BF      
      and x22,x22,x8
      li x4,0x2DEC623DFEE1FCF5      
      sb x4,0x000000000(x22)      
      li x22,0x0001060D3      
      and x22,x22,x8
      li x4,0x10580B8013385FD3      
      dcache.cva x22      
      li x22,0x000104020      
      and x22,x22,x8
      li x4,0x9F75EE22B7D570B3      
      lw x5,0x000000008(x22)      
      li x22,0x000106143      
      and x22,x22,x8
      li x4,0x671DB42E5DE336BB      
      dcache.civa x22      
      li x22,0x0001001B0      
      and x22,x22,x8
      li x4,0x989AF69B146C3786      
      sd x4,0x000000018(x22)      
      li x22,0x0001001B1      
      and x22,x22,x8
      li x4,0x8B8A61FB381CC2F3      
      sh x4,0x00000002E(x22)      
      li x22,0x000102177      
      and x22,x22,x8
      li x4,0x4E9AA15E2E17AE85      
      sh x4,0x00000002E(x22)      
      li x22,0x000100058      
      and x22,x22,x8
      li x4,0xF5C43257F9838239      
      sw x4,0x000000038(x22)      
      li x22,0x0001020FB      
      and x22,x22,x8
      li x4,0x6AD1485383CC1D3F      
      lh x5,0x000000002(x22)      
      li x22,0x000100199      
      and x22,x22,x8
      li x4,0xF5C940A707D0070C      
      sh x4,0x00000002A(x22)      
      li x22,0x000100190      
      and x22,x22,x8
      li x4,0x7874CDF60CBF5079      
      sb x4,0x00000001F(x22)      
      li x22,0x000104032      
      and x22,x22,x8
      li x4,0x59D894ED7784216E      
      lw x5,0x000000030(x22)      
      li x22,0x0001040E0      
      and x22,x22,x8
      li x4,0x511726BA08DF6C9E      
      dcache.cva x22      
      li x22,0x000102168      
      and x22,x22,x8
      li x4,0xC9CCB13256F29F17      
      lw x5,0x000000040(x22)      
      li x22,0x000102019      
      and x22,x22,x8
      li x4,0x7A6EF00CCA5F6F7A      
      dcache.civa x22      
      li x22,0x00010002A      
      and x22,x22,x8
      li x4,0x2F535F5B9D35A0D1      
      dcache.civa x22      
      li x22,0x00010417D      
      and x22,x22,x8
      li x4,0xB730ACBDDD6F5358      
      lbu x5,0x000000046(x22)      
      li x22,0x0001061EA      
      and x22,x22,x8
      li x4,0x32724B6EFF698FE2      
      lhu x5,0x000000004(x22)      
      li x22,0x000104119      
      and x22,x22,x8
      li x4,0xC449379871FB6620      
      lh x5,0x000000026(x22)      
      li x22,0x0001000ED      
      and x22,x22,x8
      li x4,0x92470D90BE66D4AD      
      lhu x5,0x00000004E(x22)      
      li x22,0x000102184      
      and x22,x22,x8
      li x4,0x5528C69A2467611B      
      sd x4,0x000000010(x22)      
      li x22,0x00010614C      
      and x22,x22,x8
      li x4,0x760ECDFC5503DD01      
      dcache.civa x22      
      li x22,0x000106151      
      and x22,x22,x8
      li x4,0x2866704B11975000      
      dcache.civa x22      
      li x22,0x000102094      
      and x22,x22,x8
      li x4,0x16B4B1226BBD9244      
      sb x4,0x00000003F(x22)      
      li x22,0x000104043      
      and x22,x22,x8
      li x4,0x5CE5C0338D906E07      
      lwu x5,0x000000024(x22)      
      li x22,0x00010609D      
      and x22,x22,x8
      li x4,0x91BC2167FF912784      
      sw x4,0x000000038(x22)      
      li x22,0x00010214F      
      and x22,x22,x8
      li x4,0xDDF9623485B4B94D      
      ld x5,0x000000038(x22)      
      li x22,0x000102069      
      and x22,x22,x8
      li x4,0x6521F6DB7BE3993A      
      sd x4,0x000000038(x22)      
      li x22,0x000102049      
      and x22,x22,x8
      li x4,0x98A164A3691C0076      
      lbu x5,0x000000001(x22)      
      li x22,0x000102152      
      and x22,x22,x8
      li x4,0x91EE5D3EB9949A85      
      sh x4,0x000000038(x22)      
      li x22,0x000104049      
      and x22,x22,x8
      li x4,0x6E839CB34CB83BC3      
      dcache.civa x22      
      li x22,0x0001060AE      
      and x22,x22,x8
      li x4,0x9DDAC4B23502F37B      
      lwu x5,0x000000010(x22)      
      li x22,0x0001041F7      
      and x22,x22,x8
      li x4,0xB092278F50EF65B2      
      dcache.cva x22      
      li x22,0x00010615D      
      and x22,x22,x8
      li x4,0x26269D571AFA1D63      
      lwu x5,0x000000004(x22)      
      li x22,0x0001020BD      
      and x22,x22,x8
      li x4,0x17988CADAD48AD53      
      lw x5,0x000000034(x22)      
      li x22,0x0001061A2      
      and x22,x22,x8
      li x4,0x4A729CF38B09DCA9      
      lw x5,0x000000048(x22)      
      li x22,0x00010007D      
      and x22,x22,x8
      li x4,0xB9129AA17586A98C      
      lbu x5,0x000000013(x22)      
      li x22,0x0001000CA      
      and x22,x22,x8
      li x4,0xDEA68AEEF946246      
      sb x4,0x000000040(x22)      
      li x22,0x000104072      
      and x22,x22,x8
      li x4,0xF6F43719C5F64F53      
      dcache.civa x22      
      li x22,0x0001020AB      
      and x22,x22,x8
      li x4,0x681A125AB3ABB4F6      
      sb x4,0x000000044(x22)      
      li x22,0x0001060E6      
      and x22,x22,x8
      li x4,0x597E82D60E85F6EF      
      dcache.cva x22      
      li x22,0x000104028      
      and x22,x22,x8
      li x4,0x5461FA9B68226FB      
      dcache.civa x22      
      li x22,0x00010211D      
      and x22,x22,x8
      li x4,0x6E64861240C4FBBC      
      dcache.cva x22      
      li x22,0x0001001A3      
      and x22,x22,x8
      li x4,0x4A4D972925E1018E      
      sb x4,0x000000035(x22)      
      li x22,0x0001040D3      
      and x22,x22,x8
      li x4,0xA3C353EAB824BEA2      
      ld x5,0x000000020(x22)      
      li x22,0x00010418C      
      and x22,x22,x8
      li x4,0xEB48DCE568838E29      
      lhu x5,0x00000001E(x22)      
      li x22,0x00010401E      
      and x22,x22,x8
      li x4,0x26444761B4E2E11      
      dcache.cva x22      
      li x22,0x000106072      
      and x22,x22,x8
      li x4,0x8F9DF1283DC3A333      
      lbu x5,0x00000004F(x22)      
      li x22,0x0001041BF      
      and x22,x22,x8
      li x4,0x2E75168AEAB9681E      
      sh x4,0x000000038(x22)      
      li x22,0x0001060B9      
      and x22,x22,x8
      li x4,0xD503527F7F0B0E1C      
      lhu x5,0x000000010(x22)      
      li x22,0x00010400E      
      and x22,x22,x8
      li x4,0x421A5547EB2DE40B      
      lhu x5,0x000000004(x22)      
      li x22,0x00010010E      
      and x22,x22,x8
      li x4,0x9BCEB8EF41DD5B41      
      sd x4,0x000000010(x22)      
      li x22,0x0001040AC      
      and x22,x22,x8
      li x4,0x46CFFBFCF9879AE5      
      dcache.civa x22      
      li x22,0x000106110      
      and x22,x22,x8
      li x4,0xE671EE0FFD337AB4      
      sw x4,0x00000000C(x22)      
      li x22,0x000102045      
      and x22,x22,x8
      li x4,0xCDFEA8FA7C6FD47A      
      lhu x5,0x00000000A(x22)      
      li x22,0x000104108      
      and x22,x22,x8
      li x4,0xEE3C71162E13166E      
      sd x4,0x000000008(x22)      
      li x22,0x00010417D      
      and x22,x22,x8
      li x4,0x334F21CEB8629F8      
      lwu x5,0x000000018(x22)      
      li x22,0x0001040FA      
      and x22,x22,x8
      li x4,0x5E9AF51DBC9375CB      
      ld x5,0x000000000(x22)      
      li x22,0x0001001EA      
      and x22,x22,x8
      li x4,0xD31945771FEAF1B2      
      sw x4,0x000000010(x22)      
      li x22,0x00010603A      
      and x22,x22,x8
      li x4,0x735AE874B29AA0A8      
      sw x4,0x000000030(x22)      
      li x22,0x00010018D      
      and x22,x22,x8
      li x4,0xCF445D43ABECF55C      
      sb x4,0x000000036(x22)      
      li x22,0x00010402B      
      and x22,x22,x8
      li x4,0xB2CC43ADA5062ED6      
      dcache.civa x22      
      li x22,0x0001020E9      
      and x22,x22,x8
      li x4,0xC7103CFA8855AECC      
      lbu x5,0x00000003C(x22)      
      li x22,0x0001000E6      
      and x22,x22,x8
      li x4,0x2AE33202D58C18B7      
      sw x4,0x000000038(x22)      
      li x22,0x000100095      
      and x22,x22,x8
      li x4,0xD9D365A0D79D0B65      
      sh x4,0x000000046(x22)      
      li x22,0x00010610B      
      and x22,x22,x8
      li x4,0x97F98ED9B2B9CF26      
      dcache.cva x22      
      li x22,0x00010200D      
      and x22,x22,x8
      li x4,0xC7343284366FEAF0      
      dcache.cva x22      
      li x22,0x00010206B      
      and x22,x22,x8
      li x4,0x99446C12FD32109C      
      lhu x5,0x000000046(x22)      
      li x22,0x0001000CB      
      and x22,x22,x8
      li x4,0xEEC7BAE9766C6D72      
      ld x5,0x000000000(x22)      
      li x22,0x00010015F      
      and x22,x22,x8
      li x4,0xF200A415149499DC      
      dcache.cva x22      
      li x22,0x0001040EC      
      and x22,x22,x8
      li x4,0x31DE23F181218809      
      dcache.civa x22      
      li x22,0x000102092      
      and x22,x22,x8
      li x4,0x3ACA8079E8CA2576      
      sh x4,0x00000000E(x22)      
      li x22,0x000104048      
      and x22,x22,x8
      li x4,0xEEE52813018E06EF      
      dcache.civa x22      
      li x22,0x000106118      
      and x22,x22,x8
      li x4,0x7A70094A75C05B05      
      dcache.cva x22      
      li x22,0x0001000CD      
      and x22,x22,x8
      li x4,0x84FAE080F94D60AF      
      dcache.cva x22      
      li x22,0x0001001FF      
      and x22,x22,x8
      li x4,0x4A437D4436B3976E      
      dcache.civa x22      
      li x22,0x0001020C9      
      and x22,x22,x8
      li x4,0xB1C87BC4DAFDF8C2      
      sw x4,0x000000014(x22)      
      li x22,0x00010609D      
      and x22,x22,x8
      li x4,0x7533EB80AF609864      
      lbu x5,0x00000000B(x22)      
      li x22,0x000106175      
      and x22,x22,x8
      li x4,0x528F8EE90334D3FA      
      dcache.civa x22      
      li x22,0x00010602D      
      and x22,x22,x8
      li x4,0x77BC814F11A34D9      
      lbu x5,0x00000000D(x22)      
      li x22,0x0001001DB      
      and x22,x22,x8
      li x4,0xC67839FC19271327      
      sb x4,0x000000044(x22)      
      li x22,0x0001001C0      
      and x22,x22,x8
      li x4,0x15278966DF046EAD      
      sd x4,0x000000008(x22)      
      li x22,0x000106191      
      and x22,x22,x8
      li x4,0xFFD80F6F0FF84CAD      
      lhu x5,0x00000003E(x22)      
      li x22,0x00010202A      
      and x22,x22,x8
      li x4,0xDE7D85C45D8FBFBE      
      lb x5,0x00000004E(x22)      
      li x22,0x0001001DD      
      and x22,x22,x8
      li x4,0x2F0F523E48D5213F      
      lb x5,0x000000023(x22)      
      li x22,0x0001041CC      
      and x22,x22,x8
      li x4,0x7489F697ACA1DCB2      
      dcache.civa x22      
      li x22,0x00010008C      
      and x22,x22,x8
      li x4,0x50F4C8586637128A      
      lhu x5,0x000000012(x22)      
      li x22,0x000104052      
      and x22,x22,x8
      li x4,0xE9E67EACA52D173A      
      sb x4,0x00000000A(x22)      
      li x22,0x00010010A      
      and x22,x22,x8
      li x4,0x71AC43494048904D      
      dcache.civa x22      
      li x22,0x0001001C6      
      and x22,x22,x8
      li x4,0xF68972901C571F2      
      dcache.cva x22      
      li x22,0x0001020BE      
      and x22,x22,x8
      li x4,0x2B2626E3DEA5D369      
      lhu x5,0x000000044(x22)      
      li x22,0x000106001      
      and x22,x22,x8
      li x4,0xD5106D7996BB8B25      
      lh x5,0x00000004E(x22)      
      li x22,0x000102139      
      and x22,x22,x8
      li x4,0x5F320FD45CA6E48E      
      ld x5,0x000000028(x22)      
      li x22,0x000102032      
      and x22,x22,x8
      li x4,0xEE9B183346DF00DB      
      lwu x5,0x000000038(x22)      
      li x22,0x00010011C      
      and x22,x22,x8
      li x4,0x73944BDC07E78A32      
      ld x5,0x000000008(x22)      
      li x22,0x00010206C      
      and x22,x22,x8
      li x4,0x904D59C37DEC3BBC      
      sb x4,0x00000004B(x22)      
      li x22,0x00010611D      
      and x22,x22,x8
      li x4,0xA89EB8A2DB5612CF      
      dcache.civa x22      
      li x22,0x00010216B      
      and x22,x22,x8
      li x4,0xCC4C695059B7B317      
      lhu x5,0x00000003E(x22)      
      li x22,0x0001021D2      
      and x22,x22,x8
      li x4,0xDD202702D50DBC18      
      dcache.civa x22      
      li x22,0x000102065      
      and x22,x22,x8
      li x4,0x19F33AC7F22461D1      
      lwu x5,0x000000028(x22)      
      li x22,0x000100198      
      and x22,x22,x8
      li x4,0x2DE66D54AF33068A      
      lw x5,0x00000004C(x22)      
      li x22,0x0001000AA      
      and x22,x22,x8
      li x4,0x7C08C72ACD272470      
      sw x4,0x000000020(x22)      
      li x22,0x000100115      
      and x22,x22,x8
      li x4,0x95859B2609772479      
      dcache.cva x22      
      li x22,0x000102002      
      and x22,x22,x8
      li x4,0xDC58739DD9430F15      
      lh x5,0x000000038(x22)      
      li x22,0x0001061BB      
      and x22,x22,x8
      li x4,0x176161E65EFEFB1A      
      lbu x5,0x00000003D(x22)      
      li x22,0x00010603C      
      and x22,x22,x8
      li x4,0x809DA26A8212AC9C      
      sw x4,0x000000018(x22)      
      li x22,0x0001001CA      
      and x22,x22,x8
      li x4,0x9EB01CBBE8AC0E41      
      sb x4,0x00000002B(x22)      
      li x22,0x0001021CE      
      and x22,x22,x8
      li x4,0x261E948FC905E118      
      sh x4,0x00000002A(x22)      
      li x22,0x00010215C      
      and x22,x22,x8
      li x4,0xC007C051D04E7477      
      sb x4,0x000000010(x22)      
      li x22,0x0001000CB      
      and x22,x22,x8
      li x4,0x896D145151CDD94C      
      sw x4,0x000000034(x22)      
      li x22,0x0001041D7      
      and x22,x22,x8
      li x4,0x32FB77B7C7A0F2D9      
      sw x4,0x00000000C(x22)      
      li x22,0x000100048      
      and x22,x22,x8
      li x4,0x1622CF3E29C2DD48      
      dcache.civa x22      
      li x22,0x00010408D      
      and x22,x22,x8
      li x4,0xF241E378269F1677      
      sw x4,0x00000000C(x22)      
      li x22,0x0001060AA      
      and x22,x22,x8
      li x4,0x924AF1621C650668      
      lbu x5,0x000000006(x22)      
      li x22,0x0001021D3      
      and x22,x22,x8
      li x4,0xD894A1E95A88A58D      
      lwu x5,0x000000030(x22)      
      li x22,0x000100198      
      and x22,x22,x8
      li x4,0xC29CEEF7D947FEC5      
      lw x5,0x00000001C(x22)      
      li x22,0x00010004B      
      and x22,x22,x8
      li x4,0x94051866E859090B      
      sw x4,0x00000001C(x22)      
      li x22,0x000102055      
      and x22,x22,x8
      li x4,0xF64443F511FE7C90      
      dcache.cva x22      
      li x22,0x000104189      
      and x22,x22,x8
      li x4,0x34A2B3C0BA15EB44      
      dcache.civa x22      
      li x22,0x00010019D      
      and x22,x22,x8
      li x4,0xCBC2B6B9B5B935D9      
      sd x4,0x000000048(x22)      
      li x22,0x00010013A      
      and x22,x22,x8
      li x4,0x8EB4AFB7D04B5504      
      dcache.civa x22      
      li x22,0x0001060F4      
      and x22,x22,x8
      li x4,0xEF599183B39353F7      
      sh x4,0x000000028(x22)      
      li x22,0x000102027      
      and x22,x22,x8
      li x4,0x8A49443A51ED6910      
      sb x4,0x000000022(x22)      
      li x22,0x0001060E1      
      and x22,x22,x8
      li x4,0x16FF184E73C062A4      
      sb x4,0x000000033(x22)      
      li x22,0x000100047      
      and x22,x22,x8
      li x4,0xB698E2378DDC78B4      
      lh x5,0x000000022(x22)      
      li x22,0x000100016      
      and x22,x22,x8
      li x4,0xF6309DEC077716A      
      sb x4,0x000000036(x22)      
      li x22,0x000104011      
      and x22,x22,x8
      li x4,0x554EFC4CAF8F5565      
      dcache.civa x22      
      li x22,0x000100188      
      and x22,x22,x8
      li x4,0xBA5C822F42AA5B0E      
      lb x5,0x00000002B(x22)      
      li x22,0x00010417D      
      and x22,x22,x8
      li x4,0xC31A8B8AD5794986      
      sw x4,0x000000008(x22)      
      li x22,0x000102128      
      and x22,x22,x8
      li x4,0x80B0E242CEDC3DC3      
      dcache.cva x22      
      li x22,0x000104066      
      and x22,x22,x8
      li x4,0xE0B8870BCE587334      
      dcache.civa x22      
      li x22,0x0001060B4      
      and x22,x22,x8
      li x4,0xD31AB6206CA13691      
      dcache.civa x22      
      li x22,0x0001060F6      
      and x22,x22,x8
      li x4,0xE8B395E4ADEA482D      
      lb x5,0x000000005(x22)      
      li x22,0x0001040FF      
      and x22,x22,x8
      li x4,0xBD667969EFD1F32E      
      dcache.civa x22      
      li x22,0x000106036      
      and x22,x22,x8
      li x4,0x61750B242602E84      
      lw x5,0x000000048(x22)      
      li x22,0x000104023      
      and x22,x22,x8
      li x4,0x6D86316FB21C1F76      
      sd x4,0x000000018(x22)      
      li x22,0x000104192      
      and x22,x22,x8
      li x4,0x438E83A50C075565      
      lw x5,0x000000038(x22)      
      li x22,0x000104039      
      and x22,x22,x8
      li x4,0x7AF945BEEC58011C      
      lw x5,0x000000028(x22)      
      li x22,0x000104170      
      and x22,x22,x8
      li x4,0x10E41A2BD3AF8B6F      
      dcache.cva x22      
      li x22,0x000106107      
      and x22,x22,x8
      li x4,0x654EB669A2E59D49      
      lw x5,0x00000001C(x22)      
      li x22,0x0001020AD      
      and x22,x22,x8
      li x4,0xE59A93B7F319FCDF      
      sb x4,0x000000005(x22)      
      li x22,0x00010603F      
      and x22,x22,x8
      li x4,0xEAC06885E4058A4D      
      dcache.cva x22      
      li x22,0x0001020E4      
      and x22,x22,x8
      li x4,0xB5C0932187E4480F      
      sw x4,0x000000034(x22)      
      li x22,0x0001060D7      
      and x22,x22,x8
      li x4,0x2A906F7968647D55      
      dcache.cva x22      
      li x22,0x000106133      
      and x22,x22,x8
      li x4,0xBF26C9A610E66AD2      
      lw x5,0x000000018(x22)      
      li x22,0x00010601C      
      and x22,x22,x8
      li x4,0x8F2CAABE804C70E3      
      sw x4,0x000000010(x22)      
      li x22,0x0001001EE      
      and x22,x22,x8
      li x4,0xA403843EFFBD652F      
      dcache.cva x22      
      li x22,0x0001020AC      
      and x22,x22,x8
      li x4,0xC74CDDB1A9467935      
      dcache.cva x22      
      li x22,0x00010008A      
      and x22,x22,x8
      li x4,0x5B838EA110CC0106      
      dcache.civa x22      
      li x22,0x000102176      
      and x22,x22,x8
      li x4,0x51924128B6AEB4E8      
      dcache.civa x22      
      li x22,0x0001041A8      
      and x22,x22,x8
      li x4,0x226360F9F7BF5DC5      
      sb x4,0x000000011(x22)      
      li x22,0x0001021B0      
      and x22,x22,x8
      li x4,0xA948814F0A472B80      
      dcache.cva x22      
      li x22,0x000104075      
      and x22,x22,x8
      li x4,0x495A6746506497A2      
      lw x5,0x000000038(x22)      
      li x22,0x00010400C      
      and x22,x22,x8
      li x4,0x2EFCFEB8F3810969      
      sw x4,0x000000018(x22)      
      li x22,0x0001001D3      
      and x22,x22,x8
      li x4,0x52A286B36273FD56      
      lb x5,0x00000000F(x22)      
      li x22,0x000102145      
      and x22,x22,x8
      li x4,0xB15CB979A24A4B03      
      sw x4,0x000000044(x22)      
      li x22,0x000104054      
      and x22,x22,x8
      li x4,0x47D05FBE961EC6E4      
      sd x4,0x000000030(x22)      
      li x22,0x000102093      
      and x22,x22,x8
      li x4,0xA6B5A08DFD9ABC80      
      lbu x5,0x000000026(x22)      
      li x22,0x000104132      
      and x22,x22,x8
      li x4,0x9D1BDE58BB8F02A1      
      sh x4,0x000000048(x22)      
      li x22,0x00010401A      
      and x22,x22,x8
      li x4,0x7B1C8AB1EF6375E5      
      sw x4,0x000000040(x22)      
      li x22,0x0001001A6      
      and x22,x22,x8
      li x4,0x68C0ECDC480BCB6E      
      dcache.cva x22      
      li x22,0x000100036      
      and x22,x22,x8
      li x4,0xB6B94C2B48068E35      
      dcache.cva x22      
      li x22,0x000100191      
      and x22,x22,x8
      li x4,0x3F20C27FA4A1DE9C      
      lhu x5,0x000000006(x22)      
      li x22,0x000100138      
      and x22,x22,x8
      li x4,0x3D7E5BFFCFDCCBF8      
      ld x5,0x000000020(x22)      
      li x22,0x0001021E9      
      and x22,x22,x8
      li x4,0x92D8B830CC4B0C46      
      lhu x5,0x00000003E(x22)      
      li x22,0x000104070      
      and x22,x22,x8
      li x4,0xCD324E92E28F679A      
      sw x4,0x000000050(x22)      
      li x22,0x0001060B1      
      and x22,x22,x8
      li x4,0xFB8E994D3F8BBFA1      
      lhu x5,0x000000044(x22)      
      li x22,0x000104057      
      and x22,x22,x8
      li x4,0x64C01BA2D5BE28D1      
      dcache.cva x22      
      li x22,0x000100062      
      and x22,x22,x8
      li x4,0x168FB176FCC5DA66      
      ld x5,0x000000008(x22)      
      li x22,0x0001040C4      
      and x22,x22,x8
      li x4,0xF7E858BA4316E195      
      lbu x5,0x000000023(x22)      
      li x22,0x00010205A      
      and x22,x22,x8
      li x4,0xF0F397BE8C6C195      
      sb x4,0x00000003E(x22)      
      li x22,0x00010201C      
      and x22,x22,x8
      li x4,0x946FD84594A2A323      
      lb x5,0x000000050(x22)      
      li x22,0x0001021A9      
      and x22,x22,x8
      li x4,0xF4853586CD2443FF      
      dcache.cva x22      
      li x22,0x000106045      
      and x22,x22,x8
      li x4,0xFCC9ACCCD8D00AE2      
      sw x4,0x00000004C(x22)      
      li x22,0x00010004B      
      and x22,x22,x8
      li x4,0xA16CF8DEBAFD55A      
      dcache.cva x22      
      li x22,0x0001061F5      
      and x22,x22,x8
      li x4,0xE762D0A7606AFE90      
      lwu x5,0x000000048(x22)      
      li x22,0x0001021A0      
      and x22,x22,x8
      li x4,0x20514A45A84DABC8      
      lh x5,0x00000004A(x22)      
      li x22,0x0001061C3      
      and x22,x22,x8
      li x4,0x65DB356E719E3530      
      dcache.cva x22      
      li x22,0x0001001C2      
      and x22,x22,x8
      li x4,0xB542FC91EF9BBECB      
      lbu x5,0x00000002B(x22)      
      li x22,0x00010001B      
      and x22,x22,x8
      li x4,0xC6E9DCFF2BF7957C      
      ld x5,0x000000048(x22)      
      li x22,0x0001061EA      
      and x22,x22,x8
      li x4,0x2D84C8C4838BE234      
      lwu x5,0x000000018(x22)      
      li x22,0x000104047      
      and x22,x22,x8
      li x4,0x8CB34991B8F09447      
      lhu x5,0x00000001C(x22)      
      li x22,0x0001020E7      
      and x22,x22,x8
      li x4,0x81A86A49F4A4CE4C      
      lbu x5,0x00000000F(x22)      
      li x22,0x000100058      
      and x22,x22,x8
      li x4,0x62D32616A40589DD      
      lwu x5,0x000000004(x22)      
      li x22,0x000106192      
      and x22,x22,x8
      li x4,0xEFB08F0FF5367455      
      sw x4,0x000000010(x22)      
      li x22,0x000100016      
      and x22,x22,x8
      li x4,0x94A51488F183AADA      
      lhu x5,0x00000001E(x22)      
      li x22,0x0001020E9      
      and x22,x22,x8
      li x4,0x2EF5995CF72CFC64      
      sw x4,0x00000004C(x22)      
      li x22,0x000100194      
      and x22,x22,x8
      li x4,0xAD9803986E26217      
      lb x5,0x00000002B(x22)      
      li x22,0x000102049      
      and x22,x22,x8
      li x4,0x47D363FFB66C39EE      
      lh x5,0x000000000(x22)      
      li x22,0x0001060B9      
      and x22,x22,x8
      li x4,0xC59F62C8158A11EB      
      dcache.cva x22      
      li x22,0x000106033      
      and x22,x22,x8
      li x4,0xCAF47C02A9DEB1D      
      dcache.civa x22      
      li x22,0x0001020AA      
      and x22,x22,x8
      li x4,0xC73A2EF966BEF679      
      sh x4,0x00000004C(x22)      
      li x22,0x0001040C4      
      and x22,x22,x8
      li x4,0xFB01ED75EFAFFB0A      
      lhu x5,0x00000004C(x22)      
      li x22,0x00010406D      
      and x22,x22,x8
      li x4,0x6AF39BC04A6056AF      
      ld x5,0x000000040(x22)      
      li x22,0x000102005      
      and x22,x22,x8
      li x4,0xCAB36BE24C33DB6E      
      sd x4,0x000000048(x22)      
      li x22,0x00010210B      
      and x22,x22,x8
      li x4,0x6F7973F59EA8C4CA      
      lb x5,0x000000030(x22)      
      li x22,0x000100127      
      and x22,x22,x8
      li x4,0x11AA53BDD6F23067      
      lb x5,0x000000007(x22)      
  la   x1, __exit
  jr   x1

.global CPU1
CPU1:
      MMODE_SMODE CPU1_S
.global CPU1_S
CPU1_S:
      li x22,0x000102079      
      and x22,x22,x8
      li x22,0x00010403A      
      and x22,x22,x8
      li x4,0xF428178136A29315      
      sh x4,0x000000002(x22)      
      li x22,0x00010407E      
      and x22,x22,x8
      li x4,0x44554A914BC46B4D      
      ld x5,0x000000040(x22)      
      li x22,0x000100068      
      and x22,x22,x8
      li x4,0x250E4593483D8231      
      dcache.cva x22      
      li x22,0x0001020E9      
      and x22,x22,x8
      li x4,0x3128CA3BE079A23B      
      sd x4,0x000000048(x22)      
      li x22,0x000106C07      
      and x22,x22,x8
      li x4,0x6AEC585BD334CC4B      
      dcache.civa x22      
      li x22,0x00010683F      
      and x22,x22,x8
      li x4,0xD7E1B938DF440DC3      
      sb x4,0x000000018(x22)      
      li x22,0x000102012      
      and x22,x22,x8
      li x4,0xA8E3E6AE9BB00F45      
      lb x5,0x000000033(x22)      
      li x22,0x000102101      
      and x22,x22,x8
      li x4,0x6C83B96EE5DF4C29      
      dcache.cva x22      
      li x22,0x0001001AD      
      and x22,x22,x8
      li x4,0xB3BD4B11952C755A      
      sd x4,0x000000038(x22)      
      li x22,0x000104015      
      and x22,x22,x8
      li x4,0x66B8D27032180BCC      
      dcache.civa x22      
      li x22,0x000106A27      
      and x22,x22,x8
      li x4,0xDF3F9132D9FE4039      
      sd x4,0x000000040(x22)      
      li x22,0x00010685F      
      and x22,x22,x8
      li x4,0xA971E8F3AEB373C8      
      lw x5,0x000000038(x22)      
      li x22,0x0001021C0      
      and x22,x22,x8
      li x4,0x3BB1415C396BD963      
      dcache.civa x22      
      li x22,0x000106A68      
      and x22,x22,x8
      li x4,0xEB343BF370B79BF8      
      sb x4,0x000000050(x22)      
      li x22,0x0001001E2      
      and x22,x22,x8
      li x4,0xADB00B18D2DFFFC7      
      lhu x5,0x00000004E(x22)      
      li x22,0x0001021F2      
      and x22,x22,x8
      li x4,0xD1EDF5CD577A964D      
      lb x5,0x000000041(x22)      
      li x22,0x000106BE5      
      and x22,x22,x8
      li x4,0x5BAFE37C472843C5      
      dcache.civa x22      
      li x22,0x0001001D0      
      and x22,x22,x8
      li x4,0x93C06F7F8B581517      
      sh x4,0x00000002E(x22)      
      li x22,0x000106A77      
      and x22,x22,x8
      li x4,0xAC27A031FB62B21      
      dcache.civa x22      
      li x22,0x000106ACD      
      and x22,x22,x8
      li x4,0xE2912D877A75DAA6      
      sh x4,0x00000002E(x22)      
      li x22,0x000106BAC      
      and x22,x22,x8
      li x4,0x5508C79564D0E031      
      sb x4,0x000000041(x22)      
      li x22,0x0001069B6      
      and x22,x22,x8
      li x4,0x715C5309FD14D485      
      dcache.cva x22      
      li x22,0x000102135      
      and x22,x22,x8
      li x4,0x258A01EC7AFFD065      
      ld x5,0x000000040(x22)      
      li x22,0x0001000C4      
      and x22,x22,x8
      li x4,0xC743C77A90936FCE      
      sw x4,0x000000040(x22)      
      li x22,0x0001068B2      
      and x22,x22,x8
      li x4,0x82193D882507D943      
      ld x5,0x000000020(x22)      
      li x22,0x000106B5C      
      and x22,x22,x8
      li x4,0xEF2BD595D92580BF      
      dcache.civa x22      
      li x22,0x0001001C7      
      and x22,x22,x8
      li x4,0xC4FE6583073892C6      
      sd x4,0x000000018(x22)      
      li x22,0x000106ADF      
      and x22,x22,x8
      li x4,0xB2749FB69C28666C      
      dcache.cva x22      
      li x22,0x00010672E      
      and x22,x22,x8
      li x4,0x6D1E9F5239ECCA14      
      sw x4,0x000000000(x22)      
      li x22,0x000106ACE      
      and x22,x22,x8
      li x4,0xC21D44E1D317AD      
      ld x5,0x000000010(x22)      
      li x22,0x0001021E1      
      and x22,x22,x8
      li x4,0x1CD95AB7B51C9E45      
      sh x4,0x000000036(x22)      
      li x22,0x000106769      
      and x22,x22,x8
      li x4,0xBFBAC92E40EB5655      
      sw x4,0x000000024(x22)      
      li x22,0x0001020BC      
      and x22,x22,x8
      li x4,0xE71AE9D4F8095639      
      dcache.cva x22      
      li x22,0x0001068B3      
      and x22,x22,x8
      li x4,0x8C774140C48D0149      
      sd x4,0x000000048(x22)      
      li x22,0x0001001D6      
      and x22,x22,x8
      li x4,0xE166951CEF1576BA      
      dcache.civa x22      
      li x22,0x000100096      
      and x22,x22,x8
      li x4,0xE2AC4AC468B0F22C      
      dcache.cva x22      
      li x22,0x000100145      
      and x22,x22,x8
      li x4,0xFB21CB86C192063C      
      lbu x5,0x000000014(x22)      
      li x22,0x000106C74      
      and x22,x22,x8
      li x4,0x89003090E529E139      
      dcache.cva x22      
      li x22,0x000100094      
      and x22,x22,x8
      li x4,0xD099548B444298CC      
      lh x5,0x00000000A(x22)      
      li x22,0x000102004      
      and x22,x22,x8
      li x4,0xA2645101212B4180      
      lwu x5,0x000000034(x22)      
      li x22,0x0001068E6      
      and x22,x22,x8
      li x4,0xBEB83CF473CC293      
      dcache.cva x22      
      li x22,0x000104150      
      and x22,x22,x8
      li x4,0x7D6F9F7FED204E3C      
      ld x5,0x000000038(x22)      
      li x22,0x000100139      
      and x22,x22,x8
      li x4,0x3DC54531568FD6DD      
      dcache.cva x22      
      li x22,0x0001040D5      
      and x22,x22,x8
      li x4,0x6C143E358908D4AE      
      sw x4,0x000000004(x22)      
      li x22,0x0001067BD      
      and x22,x22,x8
      li x4,0x633B7C0DBFCF7B10      
      dcache.civa x22      
      li x22,0x0001067FE      
      and x22,x22,x8
      li x4,0xC3E144AE170D27F      
      ld x5,0x000000040(x22)      
      li x22,0x0001001A4      
      and x22,x22,x8
      li x4,0xB6785E6BC1BFE754      
      lb x5,0x00000000A(x22)      
      li x22,0x000104009      
      and x22,x22,x8
      li x4,0xA3993047E83EF6D6      
      lbu x5,0x00000000F(x22)      
      li x22,0x0001020C5      
      and x22,x22,x8
      li x4,0x292EFF136EA75DC2      
      sd x4,0x000000010(x22)      
      li x22,0x000106BBA      
      and x22,x22,x8
      li x4,0xCCC06BEFEC9286B0      
      sh x4,0x000000004(x22)      
      li x22,0x000106AE3      
      and x22,x22,x8
      li x4,0x203C5DC01FA9D21E      
      dcache.cva x22      
      li x22,0x000106919      
      and x22,x22,x8
      li x4,0xF97CBFDDC82C4799      
      sw x4,0x000000034(x22)      
      li x22,0x0001068FD      
      and x22,x22,x8
      li x4,0xFAB1FFE0580FB16C      
      lh x5,0x000000012(x22)      
      li x22,0x000106BE7      
      and x22,x22,x8
      li x4,0xDCAA18CC35AE3DA5      
      sb x4,0x000000018(x22)      
      li x22,0x0001000E5      
      and x22,x22,x8
      li x4,0x91AC6982084B6586      
      dcache.civa x22      
      li x22,0x0001020E6      
      and x22,x22,x8
      li x4,0xA68E9B9E951C604E      
      sw x4,0x000000030(x22)      
      li x22,0x00010203F      
      and x22,x22,x8
      li x4,0xFE68F4900B80061B      
      sh x4,0x00000001A(x22)      
      li x22,0x000102166      
      and x22,x22,x8
      li x4,0x4025A59EDD28F78B      
      sb x4,0x000000037(x22)      
      li x22,0x000102180      
      and x22,x22,x8
      li x4,0xEC47BFCA2B467BB4      
      dcache.cva x22      
      li x22,0x000106C4C      
      and x22,x22,x8
      li x4,0xF2471785DFDBD0CA      
      ld x5,0x000000040(x22)      
      li x22,0x000102167      
      and x22,x22,x8
      li x4,0xC87C4339325138CE      
      lw x5,0x00000004C(x22)      
      li x22,0x000102144      
      and x22,x22,x8
      li x4,0x27ED15A7875D453C      
      lh x5,0x000000030(x22)      
      li x22,0x000106766      
      and x22,x22,x8
      li x4,0xACEC9534CD0B2293      
      sh x4,0x00000002A(x22)      
      li x22,0x000106A69      
      and x22,x22,x8
      li x4,0xC63FC598EF5DFBB      
      dcache.civa x22      
      li x22,0x0001000D0      
      and x22,x22,x8
      li x4,0x73EB5F735B65425      
      lh x5,0x00000002C(x22)      
      li x22,0x0001000C5      
      and x22,x22,x8
      li x4,0xF94F9BB870202C39      
      lbu x5,0x00000001B(x22)      
      li x22,0x000104131      
      and x22,x22,x8
      li x4,0xA7AF027DEDA8861B      
      lbu x5,0x000000015(x22)      
      li x22,0x000104184      
      and x22,x22,x8
      li x4,0x222213DD5BF496B8      
      lhu x5,0x00000004A(x22)      
      li x22,0x0001041C3      
      and x22,x22,x8
      li x4,0x95DD05112264B05E      
      lbu x5,0x000000000(x22)      
      li x22,0x0001040AB      
      and x22,x22,x8
      li x4,0xA356F31FE53E194F      
      dcache.civa x22      
      li x22,0x000104158      
      and x22,x22,x8
      li x4,0x945A8BEE55118481      
      dcache.cva x22      
      li x22,0x000102028      
      and x22,x22,x8
      li x4,0xD06B6A11DFEA298      
      lb x5,0x000000018(x22)      
      li x22,0x000106982      
      and x22,x22,x8
      li x4,0x4E177E29BA1B569B      
      dcache.cva x22      
      li x22,0x000100078      
      and x22,x22,x8
      li x4,0x17B6122062D0EFC      
      dcache.cva x22      
      li x22,0x0001041CA      
      and x22,x22,x8
      li x4,0xFCB5D407D3F62A47      
      sh x4,0x00000003E(x22)      
      li x22,0x0001020C1      
      and x22,x22,x8
      li x4,0x3B93C7D4EBA4EF60      
      lhu x5,0x000000038(x22)      
      li x22,0x000106768      
      and x22,x22,x8
      li x4,0x80824888889F6C08      
      sh x4,0x000000000(x22)      
      li x22,0x0001068E2      
      and x22,x22,x8
      li x4,0xF75FEFD67FD58D67      
      dcache.cva x22      
      li x22,0x000106A40      
      and x22,x22,x8
      li x4,0x7EBA4A7C2F6524A3      
      lb x5,0x000000046(x22)      
      li x22,0x0001001A0      
      and x22,x22,x8
      li x4,0x5863141DAECB700      
      dcache.civa x22      
      li x22,0x000102064      
      and x22,x22,x8
      li x4,0x3E8C72A5F6F059E3      
      dcache.civa x22      
      li x22,0x0001041CC      
      and x22,x22,x8
      li x4,0x76276785BE1A0248      
      lw x5,0x000000024(x22)      
      li x22,0x00010677D      
      and x22,x22,x8
      li x4,0x523616808B811E02      
      lbu x5,0x00000000C(x22)      
      li x22,0x0001000C6      
      and x22,x22,x8
      li x4,0x2FD3E516E93BC624      
      lbu x5,0x00000002F(x22)      
      li x22,0x000106C49      
      and x22,x22,x8
      li x4,0x94017F641494F79D      
      dcache.cva x22      
      li x22,0x000106798      
      and x22,x22,x8
      li x4,0x1A96157E14712B5D      
      sd x4,0x000000018(x22)      
      li x22,0x0001040A0      
      and x22,x22,x8
      li x4,0x430A738976441A06      
      lbu x5,0x00000001A(x22)      
      li x22,0x00010215F      
      and x22,x22,x8
      li x4,0x3D29411C7F2BAE5F      
      lb x5,0x00000000E(x22)      
      li x22,0x000102007      
      and x22,x22,x8
      li x4,0xEF43A59D6422B4A8      
      sb x4,0x000000010(x22)      
      li x22,0x0001021BF      
      and x22,x22,x8
      li x4,0xA330789AFC90824D      
      sd x4,0x000000008(x22)      
      li x22,0x0001069C0      
      and x22,x22,x8
      li x4,0x39DB4ED359D1AEDD      
      lwu x5,0x000000000(x22)      
      li x22,0x000106BD4      
      and x22,x22,x8
      li x4,0xFEBC079F56E908AC      
      dcache.cva x22      
      li x22,0x00010694A      
      and x22,x22,x8
      li x4,0x6F7AF80DE4984FE8      
      sh x4,0x00000003C(x22)      
      li x22,0x0001069A7      
      and x22,x22,x8
      li x4,0x8D06B1288D4F8F14      
      sh x4,0x00000000A(x22)      
      li x22,0x000104054      
      and x22,x22,x8
      li x4,0x455FD614DFEAAFBF      
      lb x5,0x000000022(x22)      
      li x22,0x000106BD9      
      and x22,x22,x8
      li x4,0xD6AC684BD513AAB      
      sd x4,0x000000038(x22)      
      li x22,0x0001001C5      
      and x22,x22,x8
      li x4,0x16B75D46806C9FE3      
      sd x4,0x000000040(x22)      
      li x22,0x00010407F      
      and x22,x22,x8
      li x4,0x5237284BD1026E4F      
      sd x4,0x000000010(x22)      
      li x22,0x000106822      
      and x22,x22,x8
      li x4,0x62E72F77DF3D3AE      
      dcache.civa x22      
      li x22,0x000104008      
      and x22,x22,x8
      li x4,0x9BFB994C2E8338F      
      sh x4,0x000000028(x22)      
      li x22,0x000106749      
      and x22,x22,x8
      li x4,0xD3FFAB12B5CA484      
      lb x5,0x000000016(x22)      
      li x22,0x000104027      
      and x22,x22,x8
      li x4,0x404B2446E7999232      
      lbu x5,0x00000004D(x22)      
      li x22,0x000104020      
      and x22,x22,x8
      li x4,0x6C1DDF3536D9CB38      
      ld x5,0x000000018(x22)      
      li x22,0x00010212F      
      and x22,x22,x8
      li x4,0xA0CA67AA7E6EFA5F      
      sw x4,0x000000004(x22)      
      li x22,0x000106C9B      
      and x22,x22,x8
      li x4,0xFB3F8053DD43FC55      
      dcache.cva x22      
      li x22,0x0001021BC      
      and x22,x22,x8
      li x4,0xF277CD57FB5B9F6E      
      dcache.civa x22      
      li x22,0x000106C33      
      and x22,x22,x8
      li x4,0xFEC663FAE0171745      
      sh x4,0x000000018(x22)      
      li x22,0x0001067D8      
      and x22,x22,x8
      li x4,0xB19A1D5E8C29A0AD      
      sd x4,0x000000038(x22)      
      li x22,0x000106C98      
      and x22,x22,x8
      li x4,0x841B6CA979A510CC      
      sb x4,0x000000008(x22)      
      li x22,0x000106A90      
      and x22,x22,x8
      li x4,0x5798477D9D107B4E      
      dcache.civa x22      
      li x22,0x000106ACB      
      and x22,x22,x8
      li x4,0x7837BB5C4CBB75AB      
      lb x5,0x00000000B(x22)      
      li x22,0x000106902      
      and x22,x22,x8
      li x4,0x825E8932D3C692B6      
      lw x5,0x000000038(x22)      
      li x22,0x0001041F3      
      and x22,x22,x8
      li x4,0x5A77DA2BC768A7AA      
      dcache.civa x22      
      li x22,0x00010672B      
      and x22,x22,x8
      li x4,0xC0B335FA459A0966      
      sw x4,0x000000030(x22)      
      li x22,0x0001040E9      
      and x22,x22,x8
      li x4,0xDC559093F40AEAAB      
      sd x4,0x000000040(x22)      
      li x22,0x000106CAC      
      and x22,x22,x8
      li x4,0xC71BCF18C5C2F911      
      sb x4,0x000000048(x22)      
      li x22,0x0001067BC      
      and x22,x22,x8
      li x4,0xC8DF4C20F047C412      
      lwu x5,0x000000020(x22)      
      li x22,0x000106B1C      
      and x22,x22,x8
      li x4,0x13D2CA2D79FA62F9      
      dcache.civa x22      
      li x22,0x00010205D      
      and x22,x22,x8
      li x4,0xF97595687EE8A193      
      lbu x5,0x000000005(x22)      
      li x22,0x000106BD1      
      and x22,x22,x8
      li x4,0x1EACBB2F180E7FAC      
      lh x5,0x00000001E(x22)      
      li x22,0x000104044      
      and x22,x22,x8
      li x4,0xAEC8C161315EFA32      
      lw x5,0x000000014(x22)      
      li x22,0x000102104      
      and x22,x22,x8
      li x4,0xA615AE1713F46356      
      dcache.civa x22      
      li x22,0x000106753      
      and x22,x22,x8
      li x4,0x4530E1073ADC7AC      
      dcache.cva x22      
      li x22,0x000104082      
      and x22,x22,x8
      li x4,0x3DE8016C6C11BA0D      
      dcache.cva x22      
      li x22,0x00010019E      
      and x22,x22,x8
      li x4,0x7B39D23CAFDEAC3D      
      lw x5,0x000000008(x22)      
      li x22,0x000106CDB      
      and x22,x22,x8
      li x4,0x6E38A2DA88FC8B0E      
      dcache.cva x22      
      li x22,0x000104103      
      and x22,x22,x8
      li x4,0x8B9ED4CEC207DEB4      
      dcache.civa x22      
      li x22,0x0001041C1      
      and x22,x22,x8
      li x4,0x81821B2B9A38F23A      
      lwu x5,0x00000000C(x22)      
      li x22,0x0001041C4      
      and x22,x22,x8
      li x4,0x8F97BCBB0CB0E384      
      dcache.civa x22      
      li x22,0x000100165      
      and x22,x22,x8
      li x4,0x59B753AA2AFEFD9C      
      sh x4,0x000000044(x22)      
      li x22,0x000106BF4      
      and x22,x22,x8
      li x4,0x268D834BACBA99BB      
      dcache.civa x22      
      li x22,0x0001068CC      
      and x22,x22,x8
      li x4,0x776FAD6739B38257      
      lwu x5,0x00000004C(x22)      
      li x22,0x00010415C      
      and x22,x22,x8
      li x4,0xE19AAA6276F15C7C      
      sd x4,0x000000010(x22)      
      li x22,0x000106AB3      
      and x22,x22,x8
      li x4,0x852F4646F86E616      
      sb x4,0x000000047(x22)      
      li x22,0x00010407A      
      and x22,x22,x8
      li x4,0x9EEE0F57C69E79AA      
      sw x4,0x000000038(x22)      
      li x22,0x000104190      
      and x22,x22,x8
      li x4,0xB41259103E7FC6D3      
      lwu x5,0x000000028(x22)      
      li x22,0x000100038      
      and x22,x22,x8
      li x4,0xFDAFFBE667BBB081      
      dcache.civa x22      
      li x22,0x000106C0B      
      and x22,x22,x8
      li x4,0x6AB88082656D8ACC      
      dcache.civa x22      
      li x22,0x00010678D      
      and x22,x22,x8
      li x4,0x87A933BFAFAE92B2      
      dcache.civa x22      
      li x22,0x0001068E7      
      and x22,x22,x8
      li x4,0xB908CFD1CCA2DE53      
      dcache.civa x22      
      li x22,0x000104132      
      and x22,x22,x8
      li x4,0xE1D674F912D796E6      
      sd x4,0x000000000(x22)      
      li x22,0x00010687B      
      and x22,x22,x8
      li x4,0x3708066C964F9F86      
      dcache.civa x22      
      li x22,0x000106A29      
      and x22,x22,x8
      li x4,0x758657CFE28C3B41      
      lw x5,0x000000038(x22)      
      li x22,0x0001040FF      
      and x22,x22,x8
      li x4,0xA423FBF353EDA9D6      
      dcache.cva x22      
      li x22,0x0001000DA      
      and x22,x22,x8
      li x4,0x5706372748BB8B74      
      dcache.cva x22      
      li x22,0x000106B70      
      and x22,x22,x8
      li x4,0x8B0FC96F82A5C67B      
      sh x4,0x00000002E(x22)      
      li x22,0x0001041EE      
      and x22,x22,x8
      li x4,0x1EAC193B09744DF5      
      sh x4,0x00000004E(x22)      
      li x22,0x000106BD2      
      and x22,x22,x8
      li x4,0x244C86EFE32E3FBD      
      lh x5,0x000000002(x22)      
      li x22,0x000102155      
      and x22,x22,x8
      li x4,0x6AC84FFCBCCB119C      
      lb x5,0x00000000A(x22)      
      li x22,0x0001040B2      
      and x22,x22,x8
      li x4,0x47F5C9E594F10A8D      
      lwu x5,0x00000000C(x22)      
      li x22,0x000100029      
      and x22,x22,x8
      li x4,0x3AA0568603C5B869      
      dcache.cva x22      
      li x22,0x0001021D8      
      and x22,x22,x8
      li x4,0xD1A48130EA092626      
      dcache.cva x22      
      li x22,0x000106807      
      and x22,x22,x8
      li x4,0x46AFFD3112D38933      
      dcache.cva x22      
      li x22,0x0001001BE      
      and x22,x22,x8
      li x4,0x2CA992CBB340BA80      
      dcache.civa x22      
      li x22,0x0001041C0      
      and x22,x22,x8
      li x4,0xBCE67F366E434A64      
      dcache.cva x22      
      li x22,0x000100024      
      and x22,x22,x8
      li x4,0x8E50131CFD4B82BE      
      ld x5,0x000000038(x22)      
      li x22,0x000104092      
      and x22,x22,x8
      li x4,0xF994324BFC3AA845      
      sd x4,0x000000000(x22)      
      li x22,0x000100167      
      and x22,x22,x8
      li x4,0xD6AD9DAB1DC1AFE2      
      dcache.cva x22      
      li x22,0x000104148      
      and x22,x22,x8
      li x4,0xE7E3247B3084FA8A      
      lwu x5,0x000000020(x22)      
      li x22,0x00010410D      
      and x22,x22,x8
      li x4,0x94F8B9B3BB011C42      
      lh x5,0x000000014(x22)      
      li x22,0x00010019C      
      and x22,x22,x8
      li x4,0xF8B5F75421C88C4E      
      lhu x5,0x00000001C(x22)      
      li x22,0x000106874      
      and x22,x22,x8
      li x4,0xD0BD45DCC2AB15EF      
      lbu x5,0x000000037(x22)      
      li x22,0x000106A70      
      and x22,x22,x8
      li x4,0x95FB2481F69042D0      
      lhu x5,0x00000004C(x22)      
      li x22,0x0001020A6      
      and x22,x22,x8
      li x4,0xBD2F400909048774      
      lh x5,0x000000042(x22)      
      li x22,0x000106C3C      
      and x22,x22,x8
      li x4,0x84C77C21CD34C48D      
      sw x4,0x000000024(x22)      
      li x22,0x00010681A      
      and x22,x22,x8
      li x4,0x58637CBEADEAF679      
      sb x4,0x000000017(x22)      
      li x22,0x000100052      
      and x22,x22,x8
      li x4,0x57F9570DB2A7E489      
      sw x4,0x000000044(x22)      
      li x22,0x0001021CD      
      and x22,x22,x8
      li x4,0x67228117E867F272      
      lwu x5,0x00000000C(x22)      
      li x22,0x000102047      
      and x22,x22,x8
      li x4,0x8DAB076D192802A5      
      dcache.civa x22      
      li x22,0x000100075      
      and x22,x22,x8
      li x4,0x9F756CE2E92A6CB0      
      sh x4,0x000000036(x22)      
      li x22,0x0001067FE      
      and x22,x22,x8
      li x4,0xC42E5453AF87342F      
      dcache.civa x22      
      li x22,0x000106CEB      
      and x22,x22,x8
      li x4,0x68AFD769A5E82297      
      sb x4,0x00000002B(x22)      
      li x22,0x000100162      
      and x22,x22,x8
      li x4,0x745C18A6CF3E98CD      
      lbu x5,0x000000024(x22)      
      li x22,0x000106C95      
      and x22,x22,x8
      li x4,0xFF539C7BE173EF77      
      sw x4,0x000000020(x22)      
      li x22,0x00010213A      
      and x22,x22,x8
      li x4,0x8AD4459E061411B6      
      sb x4,0x00000003D(x22)      
      li x22,0x000100113      
      and x22,x22,x8
      li x4,0x9F2BED560868D255      
      sb x4,0x00000003B(x22)      
      li x22,0x000104180      
      and x22,x22,x8
      li x4,0xC73B6D111D63F3E3      
      sd x4,0x000000008(x22)      
      li x22,0x0001021A6      
      and x22,x22,x8
      li x4,0xBD611C207BA2FEF5      
      lhu x5,0x00000002A(x22)      
      li x22,0x000106C95      
      and x22,x22,x8
      li x4,0x5C01905C70846D5D      
      lhu x5,0x000000008(x22)      
      li x22,0x000104060      
      and x22,x22,x8
      li x4,0x366637616F4E90D1      
      sw x4,0x000000028(x22)      
      li x22,0x000104118      
      and x22,x22,x8
      li x4,0xE2F49ADE957A4290      
      dcache.cva x22      
      li x22,0x00010001D      
      and x22,x22,x8
      li x4,0x7BE35DB6D992DEDC      
      sw x4,0x000000018(x22)      
      li x22,0x000106A1C      
      and x22,x22,x8
      li x4,0x57662BB923773E48      
      sb x4,0x000000029(x22)      
      li x22,0x000106751      
      and x22,x22,x8
      li x4,0xBCF2227BD28E2AAA      
      lb x5,0x00000001C(x22)      
      li x22,0x000102091      
      and x22,x22,x8
      li x4,0x1B0FE3367ECC853C      
      dcache.cva x22      
      li x22,0x000104117      
      and x22,x22,x8
      li x4,0xA6A16A9A583883AB      
      lh x5,0x00000002A(x22)      
      li x22,0x000106CBE      
      and x22,x22,x8
      li x4,0x4418DDEBFAE87D0C      
      sd x4,0x000000038(x22)      
      li x22,0x000104088      
      and x22,x22,x8
      li x4,0xB69D326950962522      
      lh x5,0x000000008(x22)      
      li x22,0x000106A5E      
      and x22,x22,x8
      li x4,0xCDA530490F2E9DBD      
      ld x5,0x000000030(x22)      
      li x22,0x00010402F      
      and x22,x22,x8
      li x4,0xC253E786490D500      
      lh x5,0x00000001E(x22)      
      li x22,0x0001069CF      
      and x22,x22,x8
      li x4,0xFC7E08421DC3D06      
      dcache.cva x22      
      li x22,0x000100038      
      and x22,x22,x8
      li x4,0xDF1395E36FF66793      
      lw x5,0x000000038(x22)      
      li x22,0x000104005      
      and x22,x22,x8
      li x4,0x380014371AD77D2      
      dcache.cva x22      
      li x22,0x000106A37      
      and x22,x22,x8
      li x4,0x5CB24ED7DC288AC3      
      sb x4,0x00000001E(x22)      
      li x22,0x000106AEC      
      and x22,x22,x8
      li x4,0xE939A3A40EB57B1      
      lw x5,0x000000040(x22)      
      li x22,0x0001001EF      
      and x22,x22,x8
      li x4,0x30275DE2DEA88979      
      dcache.civa x22      
      li x22,0x000106C91      
      and x22,x22,x8
      li x4,0x10E7C29F59F50090      
      dcache.cva x22      
      li x22,0x000106A27      
      and x22,x22,x8
      li x4,0x82FB1661964A9AC9      
      lh x5,0x000000048(x22)      
      li x22,0x000106B88      
      and x22,x22,x8
      li x4,0xDA0717AB4D4FE18C      
      dcache.civa x22      
      li x22,0x00010670C      
      and x22,x22,x8
      li x4,0xA135E3690B125AD5      
      sw x4,0x000000008(x22)      
      li x22,0x0001041BF      
      and x22,x22,x8
      li x4,0xE7C6204AAD9D4E60      
      dcache.cva x22      
      li x22,0x0001000B4      
      and x22,x22,x8
      li x4,0xDA024B05072DCECD      
      sw x4,0x00000003C(x22)      
      li x22,0x000106946      
      and x22,x22,x8
      li x4,0xCB90F1015B7368CA      
      sb x4,0x00000000F(x22)      
      li x22,0x000106A50      
      and x22,x22,x8
      li x4,0xEF0C1DEC07564E59      
      sw x4,0x000000010(x22)      
      li x22,0x000106A2F      
      and x22,x22,x8
      li x4,0x6E338A59573BA756      
      dcache.cva x22      
      li x22,0x000106852      
      and x22,x22,x8
      li x4,0xBC0C925B062ADC3C      
      dcache.civa x22      
      li x22,0x00010019B      
      and x22,x22,x8
      li x4,0x1C6E1F923AA3D38B      
      lh x5,0x000000026(x22)      
      li x22,0x0001067D0      
      and x22,x22,x8
      li x4,0x2270799E2A992BC2      
      dcache.civa x22      
      li x22,0x000100140      
      and x22,x22,x8
      li x4,0xDBD8C19BE29D5A7E      
      sh x4,0x00000002E(x22)      
      li x22,0x0001001C2      
      and x22,x22,x8
      li x4,0x22EFAE16EE29815E      
      sw x4,0x000000004(x22)      
      li x22,0x000106BA0      
      and x22,x22,x8
      li x4,0xAC1EFE13F8B1AC35      
      sw x4,0x000000050(x22)      
      li x22,0x000106743      
      and x22,x22,x8
      li x4,0x69D7A84DD186D31E      
      dcache.civa x22      
      li x22,0x0001021B8      
      and x22,x22,x8
      li x4,0x41B9603EF6405560      
      dcache.civa x22      
      li x22,0x000106CFE      
      and x22,x22,x8
      li x4,0x7507208D53B6C28A      
      sh x4,0x00000001E(x22)      
      li x22,0x00010412E      
      and x22,x22,x8
      li x4,0xA5514BAB037447D9      
      sb x4,0x00000002B(x22)      
      li x22,0x0001020C8      
      and x22,x22,x8
      li x4,0x583DC1E20C103EBB      
      lwu x5,0x000000028(x22)      
      li x22,0x000106CB1      
      and x22,x22,x8
      li x4,0x2E4A4E03E14AC975      
      sh x4,0x00000004C(x22)      
      li x22,0x000104118      
      and x22,x22,x8
      li x4,0x1A0FE2087FBE0901      
      lw x5,0x00000004C(x22)      
      li x22,0x000106CA1      
      and x22,x22,x8
      li x4,0xAD993C70AC053AF2      
      lbu x5,0x00000001C(x22)      
      li x22,0x00010678B      
      and x22,x22,x8
      li x4,0x8604724466C0620C      
      ld x5,0x000000018(x22)      
      li x22,0x00010412F      
      and x22,x22,x8
      li x4,0xDE3CD3A897F1168F      
      lw x5,0x00000003C(x22)      
      li x22,0x00010412D      
      and x22,x22,x8
      li x4,0xF3D445C08C8B0D48      
      sd x4,0x000000048(x22)      
      li x22,0x000106991      
      and x22,x22,x8
      li x4,0xC16A8E7F46730D57      
      lw x5,0x00000002C(x22)      
      li x22,0x0001040F2      
      and x22,x22,x8
      li x4,0x8B226DCA8CC88363      
      sd x4,0x000000028(x22)      
      li x22,0x000104194      
      and x22,x22,x8
      li x4,0xC31D858089A3371A      
      sb x4,0x000000042(x22)      
      li x22,0x000106A8E      
      and x22,x22,x8
      li x4,0xC1D030392FD66246      
      ld x5,0x000000020(x22)      
      li x22,0x000106C1B      
      and x22,x22,x8
      li x4,0x98C15633CC57421A      
      dcache.civa x22      
      li x22,0x000106C98      
      and x22,x22,x8
      li x4,0x98F502E55CA50F2F      
      sb x4,0x000000026(x22)      
      li x22,0x000100127      
      and x22,x22,x8
      li x4,0x7F3E5D5CB870C0E      
      dcache.cva x22      
      li x22,0x00010671B      
      and x22,x22,x8
      li x4,0x4D2B7936495F1F1F      
      lw x5,0x000000024(x22)      
      li x22,0x000102039      
      and x22,x22,x8
      li x4,0x83D19194661738A9      
      dcache.civa x22      
      li x22,0x000106787      
      and x22,x22,x8
      li x4,0x4E1EDEC3071328FF      
      sw x4,0x000000008(x22)      
      li x22,0x00010695E      
      and x22,x22,x8
      li x4,0xC1BFBBAC65EECDB6      
      sw x4,0x000000008(x22)      
      li x22,0x0001069F9      
      and x22,x22,x8
      li x4,0x519C5A655594CDC9      
      dcache.civa x22      
      li x22,0x0001041B1      
      and x22,x22,x8
      li x4,0x76DB70A7AC59D442      
      dcache.cva x22      
      li x22,0x000106C7A      
      and x22,x22,x8
      li x4,0x8E95A3ECB6CB7F6E      
      dcache.civa x22      
      li x22,0x0001041AD      
      and x22,x22,x8
      li x4,0x214189606FFD3260      
      lbu x5,0x000000026(x22)      
      li x22,0x000106AAD      
      and x22,x22,x8
      li x4,0xE19A3A828151C28      
      dcache.cva x22      
      li x22,0x000104150      
      and x22,x22,x8
      li x4,0xA7DDCA4707B2B502      
      dcache.cva x22      
      li x22,0x0001000B4      
      and x22,x22,x8
      li x4,0x406EDB986FCBE152      
      dcache.civa x22      
      li x22,0x0001068FB      
      and x22,x22,x8
      li x4,0x48CEED245F17A2C8      
      lhu x5,0x00000004E(x22)      
      li x22,0x0001041E1      
      and x22,x22,x8
      li x4,0xE796298586C932E2      
      dcache.civa x22      
      li x22,0x000106703      
      and x22,x22,x8
      li x4,0x24231829682E9809      
      dcache.civa x22      
      li x22,0x000100176      
      and x22,x22,x8
      li x4,0x8544AA52767D77B8      
      sb x4,0x000000038(x22)      
      li x22,0x000106875      
      and x22,x22,x8
      li x4,0x65619344388B83E5      
      sb x4,0x00000003A(x22)      
      li x22,0x0001040B4      
      and x22,x22,x8
      li x4,0xE46EF64885DD2D7A      
      lh x5,0x000000016(x22)      
      li x22,0x000102092      
      and x22,x22,x8
      li x4,0xD4BEB2862463E79      
      dcache.civa x22      
      li x22,0x00010002B      
      and x22,x22,x8
      li x4,0xABD8FE77DD7FD9FF      
      sb x4,0x00000000B(x22)      
      li x22,0x0001001B6      
      and x22,x22,x8
      li x4,0xAC6E1798B6EE3522      
      lb x5,0x00000000C(x22)      
      li x22,0x000106970      
      and x22,x22,x8
      li x4,0x2B529FAF9F5FF331      
      lb x5,0x000000019(x22)      
      li x22,0x0001068E0      
      and x22,x22,x8
      li x4,0x688F1076B5075A33      
      lw x5,0x000000010(x22)      
      li x22,0x0001067FA      
      and x22,x22,x8
      li x4,0xB8391FAE8CD1B80E      
      sw x4,0x000000030(x22)      
      li x22,0x0001000A5      
      and x22,x22,x8
      li x4,0x77E76B069B52FC7      
      lw x5,0x000000044(x22)      
      li x22,0x00010699E      
      and x22,x22,x8
      li x4,0x3DA4C729A73DDD5F      
      sb x4,0x000000047(x22)      
      li x22,0x0001040D1      
      and x22,x22,x8
      li x4,0xF17CBCD5FB67C976      
      dcache.civa x22      
      li x22,0x00010002C      
      and x22,x22,x8
      li x4,0xF69EF621C4BA62CF      
      lbu x5,0x00000004B(x22)      
      li x22,0x00010013B      
      and x22,x22,x8
      li x4,0xBE1A5AF62B610D73      
      dcache.cva x22      
      li x22,0x0001000A5      
      and x22,x22,x8
      li x4,0xE1FBCAC68D2969EB      
      lhu x5,0x000000036(x22)      
      li x22,0x0001068ED      
      and x22,x22,x8
      li x4,0x792F40EA2C8CDA10      
      lh x5,0x000000036(x22)      
      li x22,0x00010673D      
      and x22,x22,x8
      li x4,0x89F58417A3651B78      
      dcache.civa x22      
      li x22,0x00010003A      
      and x22,x22,x8
      li x4,0x19E7EC38FD719D15      
      sh x4,0x000000014(x22)      
      li x22,0x000102157      
      and x22,x22,x8
      li x4,0x6E22B34DAC40680A      
      dcache.civa x22      
      li x22,0x000104035      
      and x22,x22,x8
      li x4,0x6F6CF7C9382ED0B5      
      sb x4,0x00000002E(x22)      
      li x22,0x000104121      
      and x22,x22,x8
      li x4,0xAD14FD2778A8765F      
      sd x4,0x000000020(x22)      
      li x22,0x0001041A5      
      and x22,x22,x8
      li x4,0xE770C60C3A94441C      
      dcache.civa x22      
      li x22,0x00010414D      
      and x22,x22,x8
      li x4,0x9460C6B32E8D7580      
      lh x5,0x000000022(x22)      
      li x22,0x0001021D7      
      and x22,x22,x8
      li x4,0x4E377D79A93FE0C6      
      lh x5,0x000000022(x22)      
      li x22,0x000100056      
      and x22,x22,x8
      li x4,0xB17E1A5A003DB301      
      sw x4,0x00000002C(x22)      
      li x22,0x000100149      
      and x22,x22,x8
      li x4,0xD1B1914C3D69242D      
      dcache.civa x22      
      li x22,0x000104056      
      and x22,x22,x8
      li x4,0xFC6B63B4F4C296B1      
      lbu x5,0x00000000E(x22)      
      li x22,0x000106B8C      
      and x22,x22,x8
      li x4,0xC23FADDBB17FF4C5      
      lhu x5,0x000000036(x22)      
      li x22,0x00010418C      
      and x22,x22,x8
      li x4,0xF255CDC6496A904C      
      lb x5,0x00000002F(x22)      
      li x22,0x000106BC7      
      and x22,x22,x8
      li x4,0x405386019F1F1B69      
      sb x4,0x000000032(x22)      
      li x22,0x0001001F9      
      and x22,x22,x8
      li x4,0x7B246A8EA3FBE8F      
      sd x4,0x000000008(x22)      
      li x22,0x0001069DC      
      and x22,x22,x8
      li x4,0x6778A4FAA5A59063      
      lh x5,0x00000000A(x22)      
      li x22,0x0001021F0      
      and x22,x22,x8
      li x4,0x1459CE85F14C8E35      
      lh x5,0x000000010(x22)      
      li x22,0x000106B3C      
      and x22,x22,x8
      li x4,0x4AD1375DDC5DD641      
      lh x5,0x000000018(x22)      
      li x22,0x000104198      
      and x22,x22,x8
      li x4,0x76C21D6281F3A308      
      dcache.cva x22      
      li x22,0x000106B1F      
      and x22,x22,x8
      li x4,0x69E2C857A68EFF1A      
      dcache.civa x22      
      li x22,0x0001020C9      
      and x22,x22,x8
      li x4,0xC97399D3AAFD206E      
      sd x4,0x000000030(x22)      
      li x22,0x0001000A0      
      and x22,x22,x8
      li x4,0xB81CE41CE0192224      
      sd x4,0x000000020(x22)      
      li x22,0x00010412F      
      and x22,x22,x8
      li x4,0xF76E0F53A4496096      
      lb x5,0x000000012(x22)      
      li x22,0x000104034      
      and x22,x22,x8
      li x4,0x16ECFFC4F140679F      
      sb x4,0x000000000(x22)      
      li x22,0x00010210B      
      and x22,x22,x8
      li x4,0x62A738034267DE88      
      sd x4,0x000000018(x22)      
      li x22,0x000106720      
      and x22,x22,x8
      li x4,0x3A6D9584E2827C71      
      dcache.cva x22      
      li x22,0x000106826      
      and x22,x22,x8
      li x4,0x3BCD26551FFB7DC3      
      lwu x5,0x00000002C(x22)      
      li x22,0x000106C3E      
      and x22,x22,x8
      li x4,0xB9E90C48C244A10F      
      lb x5,0x000000033(x22)      
      li x22,0x000102127      
      and x22,x22,x8
      li x4,0xEA32B79D3B94488B      
      dcache.civa x22      
      li x22,0x00010002F      
      and x22,x22,x8
      li x4,0x20906CD0FB523D2C      
      ld x5,0x000000008(x22)      
      li x22,0x000102177      
      and x22,x22,x8
      li x4,0x2EF1E59EC30EC18F      
      lw x5,0x00000003C(x22)      
      li x22,0x000106A0A      
      and x22,x22,x8
      li x4,0x403A7B52CD681F8F      
      dcache.cva x22      
      li x22,0x00010401B      
      and x22,x22,x8
      li x4,0xB82DD624F978382F      
      sb x4,0x000000013(x22)      
      li x22,0x000104055      
      and x22,x22,x8
      li x4,0xE7D2E061D9337FEB      
      dcache.civa x22      
      li x22,0x0001069DF      
      and x22,x22,x8
      li x4,0x9D7D1F5CF45DB761      
      sb x4,0x000000027(x22)      
      li x22,0x0001001FA      
      and x22,x22,x8
      li x4,0xD74BD6C2BA9F3BEC      
      sh x4,0x000000020(x22)      
      li x22,0x0001021D0      
      and x22,x22,x8
      li x4,0x33B43423503A767F      
      lh x5,0x00000003A(x22)      
      li x22,0x0001000CB      
      and x22,x22,x8
      li x4,0x638A45726DBD769C      
      sb x4,0x000000006(x22)      
      li x22,0x000102130      
      and x22,x22,x8
      li x4,0xB4700EFA452FC76      
      sb x4,0x00000000B(x22)      
      li x22,0x000106C0F      
      and x22,x22,x8
      li x4,0x3597E01E6D9A98E2      
      lhu x5,0x000000006(x22)      
      li x22,0x00010690C      
      and x22,x22,x8
      li x4,0x2BB0DC864D6CB8E6      
      sh x4,0x000000040(x22)      
      wfi
.global CPU2
CPU2:
      MMODE_SMODE CPU2_S
.global CPU2_S
CPU2_S:
      li x22,0x00010711F      
      and x22,x22,x8
      li x22,0x000107037      
      and x22,x22,x8
      li x4,0x16D57D713A39F242      
      sb x4,0x000000036(x22)      
      li x22,0x00010706F      
      and x22,x22,x8
      li x4,0x96B87F71430EA184      
      sd x4,0x000000018(x22)      
      li x22,0x00010211D      
      and x22,x22,x8
      li x4,0xDD6812F359B6D720      
      lhu x5,0x00000001A(x22)      
      li x22,0x00010003D      
      and x22,x22,x8
      li x4,0xF3CF278BD8B54716      
      lbu x5,0x000000018(x22)      
      li x22,0x0001020FC      
      and x22,x22,x8
      li x4,0xEB163FB95311149E      
      sb x4,0x00000002C(x22)      
      li x22,0x000100170      
      and x22,x22,x8
      li x4,0x79EA3F0B8D38540D      
      lbu x5,0x000000013(x22)      
      li x22,0x0001070C5      
      and x22,x22,x8
      li x4,0x62ED9662561499E4      
      lwu x5,0x000000020(x22)      
      li x22,0x0001070F9      
      and x22,x22,x8
      li x4,0xAB262D01F812EC68      
      lwu x5,0x000000000(x22)      
      li x22,0x000102080      
      and x22,x22,x8
      li x4,0xCA4747A6195CDC90      
      lw x5,0x00000003C(x22)      
      li x22,0x000107002      
      and x22,x22,x8
      li x4,0xECA8E7F51215F12E      
      dcache.civa x22      
      li x22,0x00010201A      
      and x22,x22,x8
      li x4,0xF6D3B803032B36B      
      lhu x5,0x000000044(x22)      
      li x22,0x00010004C      
      and x22,x22,x8
      li x4,0xF812459E486D85F3      
      sd x4,0x000000038(x22)      
      li x22,0x0001000C1      
      and x22,x22,x8
      li x4,0x1D0F048356470D5C      
      dcache.cva x22      
      li x22,0x000100066      
      and x22,x22,x8
      li x4,0x40BAD8B6AC7B7245      
      sh x4,0x00000001A(x22)      
      li x22,0x000104118      
      and x22,x22,x8
      li x4,0xFA98DC1532E89833      
      dcache.cva x22      
      li x22,0x000104152      
      and x22,x22,x8
      li x4,0xE5C1C185A9460364      
      dcache.civa x22      
      li x22,0x00010211B      
      and x22,x22,x8
      li x4,0xB0106C162E1B5693      
      dcache.civa x22      
      li x22,0x0001021BC      
      and x22,x22,x8
      li x4,0x50AF4DF130DBC97C      
      sh x4,0x000000046(x22)      
      li x22,0x00010719A      
      and x22,x22,x8
      li x4,0x5C385CAA8DBD29F3      
      sw x4,0x00000003C(x22)      
      li x22,0x0001001E0      
      and x22,x22,x8
      li x4,0xCCD7EF71431E8520      
      lw x5,0x00000002C(x22)      
      li x22,0x000100047      
      and x22,x22,x8
      li x4,0xC0B821AFD9A91366      
      lw x5,0x000000018(x22)      
      li x22,0x00010718C      
      and x22,x22,x8
      li x4,0xC42139B95334F776      
      dcache.cva x22      
      li x22,0x000107012      
      and x22,x22,x8
      li x4,0xE2AEB016175EFCAF      
      sh x4,0x000000002(x22)      
      li x22,0x00010705F      
      and x22,x22,x8
      li x4,0xD75A7DBB7482EA10      
      lwu x5,0x000000034(x22)      
      li x22,0x000100165      
      and x22,x22,x8
      li x4,0x1D3C43FA6087BC13      
      dcache.cva x22      
      li x22,0x000104070      
      and x22,x22,x8
      li x4,0xB8962A6BE1D83C5D      
      lwu x5,0x000000034(x22)      
      li x22,0x000102183      
      and x22,x22,x8
      li x4,0x4C8C30409E48E615      
      sd x4,0x000000048(x22)      
      li x22,0x000104068      
      and x22,x22,x8
      li x4,0x61C5E71EC3FC03F6      
      ld x5,0x000000040(x22)      
      li x22,0x00010704E      
      and x22,x22,x8
      li x4,0x5567F1EE0F3E0B10      
      sh x4,0x00000004E(x22)      
      li x22,0x00010712E      
      and x22,x22,x8
      li x4,0xC9BEFED9C4459566      
      dcache.cva x22      
      li x22,0x0001001D6      
      and x22,x22,x8
      li x4,0x81B50EFDAF0463E9      
      sd x4,0x000000038(x22)      
      li x22,0x0001040F9      
      and x22,x22,x8
      li x4,0x7B90589AA2CAE548      
      dcache.civa x22      
      li x22,0x000102184      
      and x22,x22,x8
      li x4,0x12C32CB3B5E8B7BD      
      dcache.civa x22      
      li x22,0x0001070B1      
      and x22,x22,x8
      li x4,0xC155A130A813B5CE      
      lwu x5,0x000000044(x22)      
      li x22,0x00010200B      
      and x22,x22,x8
      li x4,0x522B765FE72449FC      
      sw x4,0x00000001C(x22)      
      li x22,0x0001021DF      
      and x22,x22,x8
      li x4,0x29647B34D5BD5B48      
      lh x5,0x000000048(x22)      
      li x22,0x0001040F7      
      and x22,x22,x8
      li x4,0x28888ACB6A04C29D      
      ld x5,0x000000048(x22)      
      li x22,0x000104189      
      and x22,x22,x8
      li x4,0xE9A1212BCB0DC0AF      
      dcache.cva x22      
      li x22,0x000107146      
      and x22,x22,x8
      li x4,0xE706287D520A51C8      
      sh x4,0x00000004A(x22)      
      li x22,0x00010013A      
      and x22,x22,x8
      li x4,0xBA92A067BEA1A6F7      
      dcache.civa x22      
      li x22,0x0001070F4      
      and x22,x22,x8
      li x4,0xFB9EE832CE8ED5BD      
      lw x5,0x000000024(x22)      
      li x22,0x000100157      
      and x22,x22,x8
      li x4,0xF5ADF6162402204B      
      dcache.cva x22      
      li x22,0x000100199      
      and x22,x22,x8
      li x4,0xFAA593FA649958FC      
      sh x4,0x000000036(x22)      
      li x22,0x00010012B      
      and x22,x22,x8
      li x4,0x2DB5073CE08B1F43      
      dcache.civa x22      
      li x22,0x0001001B8      
      and x22,x22,x8
      li x4,0xF732791B66758A8A      
      dcache.cva x22      
      li x22,0x000107061      
      and x22,x22,x8
      li x4,0xF06C4BE5538108DA      
      dcache.civa x22      
      li x22,0x0001071C1      
      and x22,x22,x8
      li x4,0x3DE525950FD15379      
      lhu x5,0x00000003C(x22)      
      li x22,0x00010403C      
      and x22,x22,x8
      li x4,0x6D17BB1F4351D7E      
      sd x4,0x000000008(x22)      
      li x22,0x0001001D6      
      and x22,x22,x8
      li x4,0x9FF534ED08FD6829      
      lw x5,0x000000040(x22)      
      li x22,0x00010405F      
      and x22,x22,x8
      li x4,0x216FF965A43ED484      
      lbu x5,0x000000027(x22)      
      li x22,0x0001020D2      
      and x22,x22,x8
      li x4,0x75D085DF6D7E1F90      
      sb x4,0x000000021(x22)      
      li x22,0x0001000C3      
      and x22,x22,x8
      li x4,0x859DF9F20E12A908      
      dcache.cva x22      
      li x22,0x0001020A3      
      and x22,x22,x8
      li x4,0x60BAF4B4A728AA1      
      sw x4,0x000000038(x22)      
      li x22,0x00010005D      
      and x22,x22,x8
      li x4,0x365D4DD3FA9889CC      
      sb x4,0x000000042(x22)      
      li x22,0x0001020F9      
      and x22,x22,x8
      li x4,0xFB299476470F0F8D      
      sd x4,0x000000030(x22)      
      li x22,0x00010709B      
      and x22,x22,x8
      li x4,0x5B55D47853A52864      
      dcache.cva x22      
      li x22,0x0001001F9      
      and x22,x22,x8
      li x4,0x8C3F55C88E1C664B      
      dcache.civa x22      
      li x22,0x000102156      
      and x22,x22,x8
      li x4,0xBB49D078605BB636      
      dcache.civa x22      
      li x22,0x000100120      
      and x22,x22,x8
      li x4,0x7DFB6DDC5289EC6A      
      sd x4,0x000000020(x22)      
      li x22,0x00010212A      
      and x22,x22,x8
      li x4,0xD5B1570366A62879      
      sh x4,0x00000000E(x22)      
      li x22,0x000102139      
      and x22,x22,x8
      li x4,0xDD518CA4F8400D2A      
      dcache.cva x22      
      li x22,0x0001021D5      
      and x22,x22,x8
      li x4,0xCC1DBB1944143F0A      
      sd x4,0x000000000(x22)      
      li x22,0x00010402F      
      and x22,x22,x8
      li x4,0xBDCA76E727B59912      
      sh x4,0x000000002(x22)      
      li x22,0x00010005B      
      and x22,x22,x8
      li x4,0xB110D1BCF35054B3      
      sw x4,0x000000040(x22)      
      li x22,0x0001001FA      
      and x22,x22,x8
      li x4,0xAED7BBC004BFF3DE      
      ld x5,0x000000048(x22)      
      li x22,0x00010203A      
      and x22,x22,x8
      li x4,0x933156AFD5AB7F5C      
      sh x4,0x00000004A(x22)      
      li x22,0x0001071D6      
      and x22,x22,x8
      li x4,0xCF1B5F1A863ADA7F      
      sh x4,0x000000022(x22)      
      li x22,0x000100127      
      and x22,x22,x8
      li x4,0xD11F560EC3AB1046      
      dcache.civa x22      
      li x22,0x00010701D      
      and x22,x22,x8
      li x4,0x89A2C7621B2EE3BD      
      sb x4,0x00000001A(x22)      
      li x22,0x00010013B      
      and x22,x22,x8
      li x4,0xAF1934260433C03D      
      lhu x5,0x000000022(x22)      
      li x22,0x000107199      
      and x22,x22,x8
      li x4,0x38301F4F4836C65A      
      sw x4,0x000000024(x22)      
      li x22,0x00010209E      
      and x22,x22,x8
      li x4,0x8FD08A4174F0FCC0      
      lbu x5,0x000000032(x22)      
      li x22,0x00010700A      
      and x22,x22,x8
      li x4,0x7D304E95B065F5E2      
      dcache.cva x22      
      li x22,0x00010007C      
      and x22,x22,x8
      li x4,0x707420FCABB81C52      
      lbu x5,0x000000013(x22)      
      li x22,0x0001070B6      
      and x22,x22,x8
      li x4,0x88E43A8395BF4290      
      lhu x5,0x000000036(x22)      
      li x22,0x0001021BD      
      and x22,x22,x8
      li x4,0x88686632C64BFD65      
      sw x4,0x000000038(x22)      
      li x22,0x000102103      
      and x22,x22,x8
      li x4,0x5207F9AB360FD895      
      dcache.civa x22      
      li x22,0x00010219E      
      and x22,x22,x8
      li x4,0x2A5600E63A4FCAA5      
      dcache.civa x22      
      li x22,0x00010200E      
      and x22,x22,x8
      li x4,0x85C074B3782AA2CE      
      dcache.cva x22      
      li x22,0x0001021D1      
      and x22,x22,x8
      li x4,0x56473DED93C18A0D      
      lw x5,0x000000018(x22)      
      li x22,0x0001040ED      
      and x22,x22,x8
      li x4,0xF4BA3BB3CEA8435B      
      sb x4,0x00000002E(x22)      
      li x22,0x0001041D5      
      and x22,x22,x8
      li x4,0xBD3524095937C8B6      
      dcache.civa x22      
      li x22,0x000104146      
      and x22,x22,x8
      li x4,0xA0E1C1681197E0D1      
      lwu x5,0x00000001C(x22)      
      li x22,0x000107189      
      and x22,x22,x8
      li x4,0xA57536F11A02052F      
      dcache.civa x22      
      li x22,0x00010006E      
      and x22,x22,x8
      li x4,0x24172FAD06FBF83A      
      sw x4,0x000000010(x22)      
      li x22,0x000102097      
      and x22,x22,x8
      li x4,0x902C5CEFD1680AB2      
      lbu x5,0x000000038(x22)      
      li x22,0x00010704A      
      and x22,x22,x8
      li x4,0xA5B669D4F3505839      
      dcache.civa x22      
      li x22,0x0001000BF      
      and x22,x22,x8
      li x4,0x9A42CEFC149965F3      
      sh x4,0x000000022(x22)      
      li x22,0x000100022      
      and x22,x22,x8
      li x4,0xA3A7EFFD400A7BE5      
      lw x5,0x000000020(x22)      
      li x22,0x000104147      
      and x22,x22,x8
      li x4,0x5E30C1FEDF76A3F7      
      lwu x5,0x000000038(x22)      
      li x22,0x0001000FB      
      and x22,x22,x8
      li x4,0x60676610292FA7B9      
      sw x4,0x000000038(x22)      
      li x22,0x000100029      
      and x22,x22,x8
      li x4,0xC7BD3C824F928282      
      sw x4,0x000000028(x22)      
      li x22,0x000100157      
      and x22,x22,x8
      li x4,0x4F5399D29F363704      
      lw x5,0x00000003C(x22)      
      li x22,0x0001021B7      
      and x22,x22,x8
      li x4,0xF4482C0C57EC6B29      
      lw x5,0x000000044(x22)      
      li x22,0x000107009      
      and x22,x22,x8
      li x4,0xF5638AF3CE8CC3F9      
      lb x5,0x000000005(x22)      
      li x22,0x0001040FB      
      and x22,x22,x8
      li x4,0xC4D0D3A91B65EADC      
      lwu x5,0x00000001C(x22)      
      li x22,0x000102199      
      and x22,x22,x8
      li x4,0xB3454D6ECF2DD3E7      
      ld x5,0x000000018(x22)      
      li x22,0x000100004      
      and x22,x22,x8
      li x4,0x6C6DB7CE79AFF161      
      sh x4,0x000000048(x22)      
      li x22,0x0001041B9      
      and x22,x22,x8
      li x4,0xB4A5D14D0FEEF5B6      
      ld x5,0x000000030(x22)      
      li x22,0x00010201D      
      and x22,x22,x8
      li x4,0x991D927520F6FE21      
      dcache.civa x22      
      li x22,0x000104063      
      and x22,x22,x8
      li x4,0x7F732CB95255F0E1      
      dcache.cva x22      
      li x22,0x000100104      
      and x22,x22,x8
      li x4,0x3A792E8C9FAD35E1      
      sh x4,0x00000002E(x22)      
      li x22,0x000107186      
      and x22,x22,x8
      li x4,0x4F019FDB729ACEDE      
      lb x5,0x000000033(x22)      
      li x22,0x000104096      
      and x22,x22,x8
      li x4,0xBEA5981C7FAC1886      
      sw x4,0x000000014(x22)      
      li x22,0x00010707C      
      and x22,x22,x8
      li x4,0xC6AADC74369B2510      
      sd x4,0x000000020(x22)      
      li x22,0x00010415F      
      and x22,x22,x8
      li x4,0x91900C551E581691      
      lhu x5,0x00000002C(x22)      
      li x22,0x0001040A7      
      and x22,x22,x8
      li x4,0x8635CFC5F6A03595      
      lwu x5,0x00000002C(x22)      
      li x22,0x00010411D      
      and x22,x22,x8
      li x4,0x7438E85F0E5058D7      
      sh x4,0x000000010(x22)      
      li x22,0x0001040C9      
      and x22,x22,x8
      li x4,0x3732A654BE45425E      
      lwu x5,0x000000028(x22)      
      li x22,0x0001040B4      
      and x22,x22,x8
      li x4,0x9EB2F10D1E8DE439      
      lw x5,0x000000034(x22)      
      li x22,0x00010703A      
      and x22,x22,x8
      li x4,0xDC2AA5A136A0D19C      
      dcache.civa x22      
      li x22,0x000107147      
      and x22,x22,x8
      li x4,0x51B51255A764C4BD      
      sw x4,0x000000010(x22)      
      li x22,0x000102019      
      and x22,x22,x8
      li x4,0x90DDA5706393026C      
      dcache.cva x22      
      li x22,0x0001071B8      
      and x22,x22,x8
      li x4,0x23F9658987AA1496      
      ld x5,0x000000018(x22)      
      li x22,0x0001020FB      
      and x22,x22,x8
      li x4,0x91656F48F2ADE617      
      sd x4,0x000000050(x22)      
      li x22,0x0001041D8      
      and x22,x22,x8
      li x4,0xBB87E2AC184CEC81      
      lb x5,0x000000001(x22)      
      li x22,0x000104040      
      and x22,x22,x8
      li x4,0x3FABA0B6F7E5FC01      
      ld x5,0x000000020(x22)      
      li x22,0x0001000CB      
      and x22,x22,x8
      li x4,0x2D5561A16A4474F1      
      sw x4,0x000000004(x22)      
      li x22,0x000104136      
      and x22,x22,x8
      li x4,0x1369BB1C98175801      
      lw x5,0x00000000C(x22)      
      li x22,0x0001021E0      
      and x22,x22,x8
      li x4,0x62EC0D5FE729A809      
      sh x4,0x00000000A(x22)      
      li x22,0x000102137      
      and x22,x22,x8
      li x4,0x99F727C092CF9BAF      
      sw x4,0x000000004(x22)      
      li x22,0x0001071AC      
      and x22,x22,x8
      li x4,0x3DA0DB7605EF9637      
      sh x4,0x00000004A(x22)      
      li x22,0x0001021FF      
      and x22,x22,x8
      li x4,0x3F9E721471FEEE54      
      lbu x5,0x00000000B(x22)      
      li x22,0x0001021FB      
      and x22,x22,x8
      li x4,0xB424E7726CB987B2      
      lb x5,0x000000002(x22)      
      li x22,0x00010717A      
      and x22,x22,x8
      li x4,0xB5C9775420C484E2      
      sw x4,0x00000002C(x22)      
      li x22,0x0001000D5      
      and x22,x22,x8
      li x4,0x11DEC754D7767E9      
      dcache.civa x22      
      li x22,0x000104018      
      and x22,x22,x8
      li x4,0x8854D74559B8538F      
      lwu x5,0x000000004(x22)      
      li x22,0x000107012      
      and x22,x22,x8
      li x4,0xE1D8CD2BBF39436A      
      sh x4,0x000000028(x22)      
      li x22,0x0001001C8      
      and x22,x22,x8
      li x4,0x8C6125C377A3D108      
      lbu x5,0x000000041(x22)      
      li x22,0x0001000D5      
      and x22,x22,x8
      li x4,0xAD09B2D5856B4871      
      sw x4,0x000000048(x22)      
      li x22,0x00010200C      
      and x22,x22,x8
      li x4,0xFC5DE6F6481BCB24      
      dcache.cva x22      
      li x22,0x000102126      
      and x22,x22,x8
      li x4,0x8909C000087F8      
      dcache.cva x22      
      li x22,0x0001000E6      
      and x22,x22,x8
      li x4,0x72CC6A6F8C83D8A5      
      dcache.cva x22      
      li x22,0x000104097      
      and x22,x22,x8
      li x4,0xA6D3E960A622962C      
      dcache.cva x22      
      li x22,0x000107143      
      and x22,x22,x8
      li x4,0x5D9EEE29BB9308FC      
      lw x5,0x000000044(x22)      
      li x22,0x0001041CC      
      and x22,x22,x8
      li x4,0xA3C38A7DD28B96AD      
      dcache.civa x22      
      li x22,0x000100050      
      and x22,x22,x8
      li x4,0xDE2AC16CD201E74      
      sd x4,0x000000028(x22)      
      li x22,0x0001020C3      
      and x22,x22,x8
      li x4,0x7B4E2BAA3353CE9E      
      dcache.cva x22      
      li x22,0x000104103      
      and x22,x22,x8
      li x4,0xB8110589821783D5      
      dcache.civa x22      
      li x22,0x00010403F      
      and x22,x22,x8
      li x4,0x6FC838AA48CA90B8      
      dcache.cva x22      
      li x22,0x000102153      
      and x22,x22,x8
      li x4,0xB7653395636FC760      
      dcache.civa x22      
      li x22,0x000100162      
      and x22,x22,x8
      li x4,0x27DF4058F2C80DCD      
      sb x4,0x000000017(x22)      
      li x22,0x0001001A0      
      and x22,x22,x8
      li x4,0xFE7E51E6441D4777      
      lw x5,0x000000018(x22)      
      li x22,0x00010702A      
      and x22,x22,x8
      li x4,0x40164A659C082A11      
      dcache.civa x22      
      li x22,0x00010204A      
      and x22,x22,x8
      li x4,0xFF5CFB1A8224C502      
      lb x5,0x000000011(x22)      
      li x22,0x00010017E      
      and x22,x22,x8
      li x4,0x1A8582C15281B14      
      dcache.cva x22      
      li x22,0x000100138      
      and x22,x22,x8
      li x4,0xA1A274C6506A3BC8      
      sb x4,0x00000002E(x22)      
      li x22,0x000107106      
      and x22,x22,x8
      li x4,0x3BEB5A38AB49B5C7      
      dcache.civa x22      
      li x22,0x0001040CF      
      and x22,x22,x8
      li x4,0x34190B986CA8D76B      
      sw x4,0x00000001C(x22)      
      li x22,0x0001071E4      
      and x22,x22,x8
      li x4,0x1AA00A3B8A5B4D86      
      lhu x5,0x00000002C(x22)      
      li x22,0x000102144      
      and x22,x22,x8
      li x4,0x79DE19B004D48B2D      
      dcache.cva x22      
      li x22,0x00010202F      
      and x22,x22,x8
      li x4,0xAFBC213A182B105E      
      lwu x5,0x000000014(x22)      
      li x22,0x000102048      
      and x22,x22,x8
      li x4,0xB7C132F6CD6933FF      
      lhu x5,0x000000034(x22)      
      li x22,0x0001020FF      
      and x22,x22,x8
      li x4,0x9FDE79FE90C305E2      
      sd x4,0x000000010(x22)      
      li x22,0x00010716B      
      and x22,x22,x8
      li x4,0xA7B196C231AA5CBB      
      lb x5,0x000000000(x22)      
      li x22,0x000102005      
      and x22,x22,x8
      li x4,0x904ABEC91B218B88      
      sb x4,0x000000006(x22)      
      li x22,0x000100158      
      and x22,x22,x8
      li x4,0x60DDE16BD0B28E5      
      sh x4,0x00000003C(x22)      
      li x22,0x00010411E      
      and x22,x22,x8
      li x4,0xC05A0F32D40421CD      
      lw x5,0x000000014(x22)      
      li x22,0x00010218F      
      and x22,x22,x8
      li x4,0x48FA0EB573A43DCB      
      dcache.civa x22      
      li x22,0x0001071F0      
      and x22,x22,x8
      li x4,0xC32A64C159F936AD      
      dcache.civa x22      
      li x22,0x00010701A      
      and x22,x22,x8
      li x4,0xB95278E51F7E7EFF      
      lb x5,0x00000004C(x22)      
      li x22,0x0001020FB      
      and x22,x22,x8
      li x4,0x7FE0855B65541D40      
      dcache.cva x22      
      li x22,0x00010707C      
      and x22,x22,x8
      li x4,0xDD23F70A1D819E03      
      sb x4,0x00000004D(x22)      
      li x22,0x00010005F      
      and x22,x22,x8
      li x4,0x84DE0AB52FC4B5BF      
      dcache.cva x22      
      li x22,0x0001001F8      
      and x22,x22,x8
      li x4,0x824C72738B3F2014      
      lbu x5,0x000000043(x22)      
      li x22,0x000102150      
      and x22,x22,x8
      li x4,0x1BE2BA1563ACC635      
      dcache.cva x22      
      li x22,0x000102033      
      and x22,x22,x8
      li x4,0x35FE6F9BFC063EA4      
      dcache.civa x22      
      li x22,0x0001070E2      
      and x22,x22,x8
      li x4,0x4F36EC3865106D20      
      sw x4,0x000000014(x22)      
      li x22,0x000102086      
      and x22,x22,x8
      li x4,0x66E5A0BC2C4A7FD2      
      sw x4,0x00000001C(x22)      
      li x22,0x0001020E8      
      and x22,x22,x8
      li x4,0x8793C6F7974627C8      
      sw x4,0x00000000C(x22)      
      li x22,0x0001020BD      
      and x22,x22,x8
      li x4,0x4C8976F30600AFAD      
      dcache.civa x22      
      li x22,0x00010216D      
      and x22,x22,x8
      li x4,0x83DC0D36F4F18D49      
      dcache.civa x22      
      li x22,0x00010018F      
      and x22,x22,x8
      li x4,0xDF1A91BFD0E7855B      
      sh x4,0x00000004E(x22)      
      li x22,0x0001020DD      
      and x22,x22,x8
      li x4,0xB91B8E5FF0111E8      
      lbu x5,0x000000010(x22)      
      li x22,0x000107107      
      and x22,x22,x8
      li x4,0x78F9201A87696326      
      ld x5,0x000000008(x22)      
      li x22,0x000104078      
      and x22,x22,x8
      li x4,0x1FC45B31E8515A29      
      dcache.civa x22      
      li x22,0x0001021B0      
      and x22,x22,x8
      li x4,0xD1D61330D1B4196B      
      sd x4,0x000000010(x22)      
      li x22,0x0001001E6      
      and x22,x22,x8
      li x4,0x1DE355A704E39071      
      lb x5,0x000000020(x22)      
      li x22,0x00010403E      
      and x22,x22,x8
      li x4,0x132C75A0AA6B2AB4      
      dcache.civa x22      
      li x22,0x000104144      
      and x22,x22,x8
      li x4,0xBD488E48C92AF627      
      dcache.cva x22      
      li x22,0x0001020C5      
      and x22,x22,x8
      li x4,0xF2D9B6644857F095      
      dcache.cva x22      
      li x22,0x000100014      
      and x22,x22,x8
      li x4,0x8698DB86908E4BCA      
      dcache.cva x22      
      li x22,0x0001041B1      
      and x22,x22,x8
      li x4,0x509D696802B8D26B      
      dcache.civa x22      
      li x22,0x000107104      
      and x22,x22,x8
      li x4,0x94AB770E8CCC8E04      
      dcache.civa x22      
      li x22,0x00010207C      
      and x22,x22,x8
      li x4,0x236E945101B2B6E0      
      lb x5,0x000000037(x22)      
      li x22,0x000100169      
      and x22,x22,x8
      li x4,0x914C4709604CF6CA      
      sw x4,0x000000014(x22)      
      li x22,0x00010405A      
      and x22,x22,x8
      li x4,0x2A77139611BC2149      
      sd x4,0x000000040(x22)      
      li x22,0x0001040EA      
      and x22,x22,x8
      li x4,0x9CD27EBE72C01CC7      
      dcache.civa x22      
      li x22,0x0001021A9      
      and x22,x22,x8
      li x4,0x80E8B4168BD46C37      
      ld x5,0x000000030(x22)      
      li x22,0x0001071A9      
      and x22,x22,x8
      li x4,0x36E4E925A6A5DD2A      
      dcache.cva x22      
      li x22,0x000104050      
      and x22,x22,x8
      li x4,0xDE5E6E2202ACBB92      
      lwu x5,0x000000018(x22)      
      li x22,0x0001040C9      
      and x22,x22,x8
      li x4,0x27F00BAC4ED948A      
      dcache.civa x22      
      li x22,0x000104139      
      and x22,x22,x8
      li x4,0xF4AF08FE53BB8720      
      sw x4,0x00000000C(x22)      
      li x22,0x00010711E      
      and x22,x22,x8
      li x4,0xE99B27AF2A8487B2      
      ld x5,0x000000018(x22)      
      li x22,0x000104187      
      and x22,x22,x8
      li x4,0x9419AE609F7D4709      
      lbu x5,0x00000003A(x22)      
      li x22,0x0001040EA      
      and x22,x22,x8
      li x4,0x7D04283BE28AD734      
      ld x5,0x000000038(x22)      
      li x22,0x0001020DD      
      and x22,x22,x8
      li x4,0x68EFA7FC8FAE4765      
      lh x5,0x000000014(x22)      
      li x22,0x00010017A      
      and x22,x22,x8
      li x4,0xC6062601531CB4EE      
      lhu x5,0x000000018(x22)      
      li x22,0x00010001E      
      and x22,x22,x8
      li x4,0x2F762CDF7B14CE26      
      dcache.cva x22      
      li x22,0x000102162      
      and x22,x22,x8
      li x4,0x6FE6666C5F5B81AE      
      lwu x5,0x000000024(x22)      
      li x22,0x000104065      
      and x22,x22,x8
      li x4,0xBAD29018ECBF9AA1      
      lb x5,0x000000021(x22)      
      li x22,0x000107041      
      and x22,x22,x8
      li x4,0xB704F3EFF0782BBC      
      sb x4,0x000000027(x22)      
      li x22,0x000102173      
      and x22,x22,x8
      li x4,0x3F4B1ADE88EFFB91      
      lhu x5,0x000000020(x22)      
      li x22,0x0001040FE      
      and x22,x22,x8
      li x4,0xB218681E7AC02F17      
      dcache.cva x22      
      li x22,0x000104026      
      and x22,x22,x8
      li x4,0xA23A83B49E73151B      
      sh x4,0x000000008(x22)      
      li x22,0x0001020D1      
      and x22,x22,x8
      li x4,0x3243A80B699748F0      
      sh x4,0x00000000E(x22)      
      li x22,0x000104133      
      and x22,x22,x8
      li x4,0x1761CE135D39FCAD      
      sb x4,0x000000048(x22)      
      li x22,0x00010702C      
      and x22,x22,x8
      li x4,0xFA5632993EB492F9      
      sw x4,0x000000034(x22)      
      li x22,0x0001041C8      
      and x22,x22,x8
      li x4,0x11D16EFCD15D8049      
      lh x5,0x00000002C(x22)      
      li x22,0x000100040      
      and x22,x22,x8
      li x4,0xACB18048B8BD7173      
      dcache.cva x22      
      li x22,0x0001021AB      
      and x22,x22,x8
      li x4,0xD606C6661A23CAA1      
      dcache.cva x22      
      li x22,0x000100031      
      and x22,x22,x8
      li x4,0xFC01D49B0E8E862D      
      dcache.civa x22      
      li x22,0x000107060      
      and x22,x22,x8
      li x4,0x5E5BA76C8A7E69DD      
      sb x4,0x000000022(x22)      
      li x22,0x000102162      
      and x22,x22,x8
      li x4,0x3E8A5B427FFD31FF      
      dcache.cva x22      
      li x22,0x0001070F7      
      and x22,x22,x8
      li x4,0x4EBB6E01C8465AD6      
      dcache.civa x22      
      li x22,0x00010710B      
      and x22,x22,x8
      li x4,0x49AAC1146913EB53      
      lh x5,0x000000046(x22)      
      li x22,0x000100176      
      and x22,x22,x8
      li x4,0x1870A4B10F0D5662      
      sd x4,0x000000000(x22)      
      li x22,0x0001020C6      
      and x22,x22,x8
      li x4,0x47833462A798BD82      
      sw x4,0x000000044(x22)      
      li x22,0x000104017      
      and x22,x22,x8
      li x4,0x494C9ED8E70B49F1      
      lw x5,0x000000020(x22)      
      li x22,0x000107136      
      and x22,x22,x8
      li x4,0x6512ABDB61009B65      
      sw x4,0x00000000C(x22)      
      li x22,0x0001020CB      
      and x22,x22,x8
      li x4,0x3D49387C41A3B707      
      sd x4,0x000000010(x22)      
      li x22,0x000107152      
      and x22,x22,x8
      li x4,0xE669634FCB9A82FB      
      lbu x5,0x00000002E(x22)      
      li x22,0x00010707E      
      and x22,x22,x8
      li x4,0xCA7ED3C122A42F6F      
      dcache.civa x22      
      li x22,0x00010705B      
      and x22,x22,x8
      li x4,0xD1A08A7531D43516      
      lhu x5,0x00000000E(x22)      
      li x22,0x000102088      
      and x22,x22,x8
      li x4,0x9EE241BBE40FE38D      
      lbu x5,0x00000002D(x22)      
      li x22,0x000107167      
      and x22,x22,x8
      li x4,0x7453C391C8347A06      
      sh x4,0x000000026(x22)      
      li x22,0x000107058      
      and x22,x22,x8
      li x4,0x4A3617D526833259      
      sb x4,0x000000028(x22)      
      li x22,0x00010703A      
      and x22,x22,x8
      li x4,0x59E53F91A8E5BFAB      
      lwu x5,0x000000018(x22)      
      li x22,0x00010403F      
      and x22,x22,x8
      li x4,0xE7DE8F4C1D0E73D0      
      lwu x5,0x000000034(x22)      
      li x22,0x000104166      
      and x22,x22,x8
      li x4,0x24257CCAE368ABC1      
      dcache.cva x22      
      li x22,0x00010203A      
      and x22,x22,x8
      li x4,0xE518239EA57F2C10      
      sb x4,0x000000010(x22)      
      li x22,0x0001040A4      
      and x22,x22,x8
      li x4,0xC0F5F01960F132F1      
      dcache.civa x22      
      li x22,0x0001020BD      
      and x22,x22,x8
      li x4,0xB4CCA5BB8C82604D      
      lh x5,0x00000000E(x22)      
      li x22,0x0001021A9      
      and x22,x22,x8
      li x4,0x72848A34CE5C1043      
      dcache.civa x22      
      li x22,0x00010009C      
      and x22,x22,x8
      li x4,0xCC3270DFD0044EF9      
      lbu x5,0x00000002A(x22)      
      li x22,0x000104020      
      and x22,x22,x8
      li x4,0xB845F2056BB5A62C      
      lw x5,0x000000018(x22)      
      li x22,0x0001041C1      
      and x22,x22,x8
      li x4,0x1ADAB71607A0E48C      
      lbu x5,0x00000000B(x22)      
      li x22,0x000107138      
      and x22,x22,x8
      li x4,0x9B29AD2B86D46F18      
      sd x4,0x000000038(x22)      
      li x22,0x00010702F      
      and x22,x22,x8
      li x4,0x28B1D0404A038746      
      dcache.civa x22      
      li x22,0x0001071FA      
      and x22,x22,x8
      li x4,0x44D8A19A5E65E2D4      
      sw x4,0x00000004C(x22)      
      li x22,0x000102003      
      and x22,x22,x8
      li x4,0x1974C874B1B26232      
      dcache.cva x22      
      li x22,0x0001021CB      
      and x22,x22,x8
      li x4,0x6DB9D4C427957D62      
      lhu x5,0x00000003C(x22)      
      li x22,0x000102125      
      and x22,x22,x8
      li x4,0xEC9EA89CA39548EE      
      dcache.cva x22      
      li x22,0x00010214C      
      and x22,x22,x8
      li x4,0xABFB21E3E52014C      
      lbu x5,0x00000004D(x22)      
      li x22,0x0001020E5      
      and x22,x22,x8
      li x4,0x37D1F0FE65238419      
      dcache.civa x22      
      li x22,0x000104200      
      and x22,x22,x8
      li x4,0x4C8A5370ECE3A000      
      dcache.cva x22      
      li x22,0x000107014      
      and x22,x22,x8
      li x4,0x2F19BAD9C63C5F30      
      sb x4,0x00000001F(x22)      
      li x22,0x000104084      
      and x22,x22,x8
      li x4,0x3891B9C54E510E7A      
      ld x5,0x000000018(x22)      
      li x22,0x00010018C      
      and x22,x22,x8
      li x4,0xB39EDF7901B2EBC1      
      dcache.cva x22      
      li x22,0x00010701D      
      and x22,x22,x8
      li x4,0xBBCD1BACCE5246F2      
      sh x4,0x000000040(x22)      
      li x22,0x000104197      
      and x22,x22,x8
      li x4,0xAEFD8BB22FBD23B5      
      dcache.civa x22      
      li x22,0x0001001FE      
      and x22,x22,x8
      li x4,0x9627E4155F454250      
      sb x4,0x000000030(x22)      
      li x22,0x0001021DB      
      and x22,x22,x8
      li x4,0xBFF12588A062E13C      
      lh x5,0x000000020(x22)      
      li x22,0x000102009      
      and x22,x22,x8
      li x4,0xBB54EF2F6D83EEBC      
      dcache.civa x22      
      li x22,0x0001001DB      
      and x22,x22,x8
      li x4,0xB58474A683C777EB      
      lw x5,0x000000004(x22)      
      li x22,0x000107029      
      and x22,x22,x8
      li x4,0x9641CA349C5A4462      
      dcache.civa x22      
      li x22,0x000100003      
      and x22,x22,x8
      li x4,0x2494545C700CF6D      
      dcache.cva x22      
      li x22,0x00010000C      
      and x22,x22,x8
      li x4,0xF7D07008CF6598C5      
      dcache.civa x22      
      li x22,0x000107049      
      and x22,x22,x8
      li x4,0xA6482248B625ED0F      
      dcache.civa x22      
      li x22,0x00010406B      
      and x22,x22,x8
      li x4,0x28B78DCE47FE1C1      
      sb x4,0x000000010(x22)      
      li x22,0x000100131      
      and x22,x22,x8
      li x4,0x507D84165FD6BB69      
      dcache.civa x22      
      li x22,0x0001021D6      
      and x22,x22,x8
      li x4,0x8F5131B700319FB2      
      sb x4,0x000000011(x22)      
      li x22,0x000104026      
      and x22,x22,x8
      li x4,0x34757B339F5B4A6E      
      sw x4,0x000000038(x22)      
      li x22,0x0001071C2      
      and x22,x22,x8
      li x4,0x312131FF2865B41C      
      sd x4,0x000000028(x22)      
      li x22,0x00010012F      
      and x22,x22,x8
      li x4,0x705E14E9062522A0      
      sw x4,0x00000003C(x22)      
      li x22,0x0001021CC      
      and x22,x22,x8
      li x4,0xAD76FCCC69CB18D4      
      lwu x5,0x000000028(x22)      
      li x22,0x00010711A      
      and x22,x22,x8
      li x4,0x47B7CCF3D566A359      
      lwu x5,0x000000020(x22)      
      li x22,0x000100159      
      and x22,x22,x8
      li x4,0x140F80BD4253241A      
      dcache.cva x22      
      li x22,0x0001041F4      
      and x22,x22,x8
      li x4,0x2EA44BB2C6B6FA18      
      lhu x5,0x00000004E(x22)      
      li x22,0x00010705B      
      and x22,x22,x8
      li x4,0xE038BF16D8997E69      
      dcache.civa x22      
      li x22,0x000107091      
      and x22,x22,x8
      li x4,0xAB13E2506FA62064      
      sw x4,0x00000000C(x22)      
      li x22,0x0001020B6      
      and x22,x22,x8
      li x4,0xE73DBCFED967A73B      
      dcache.civa x22      
      li x22,0x000104066      
      and x22,x22,x8
      li x4,0x4A8757EB3FC91A03      
      dcache.cva x22      
      li x22,0x0001021D9      
      and x22,x22,x8
      li x4,0xF6B57625E939F0F2      
      lbu x5,0x000000045(x22)      
      li x22,0x000104081      
      and x22,x22,x8
      li x4,0xB6973A897172D047      
      sb x4,0x00000002A(x22)      
      li x22,0x000107067      
      and x22,x22,x8
      li x4,0x75CA910D5DB0E570      
      dcache.civa x22      
      li x22,0x000100129      
      and x22,x22,x8
      li x4,0xFE6C040E799DF353      
      lw x5,0x000000010(x22)      
      li x22,0x0001071DF      
      and x22,x22,x8
      li x4,0xB8B77542A1BF13BE      
      dcache.cva x22      
      li x22,0x0001021E2      
      and x22,x22,x8
      li x4,0xE93352B3C3449CAD      
      lh x5,0x000000004(x22)      
      li x22,0x000100069      
      and x22,x22,x8
      li x4,0xF34A77B3D378955D      
      lb x5,0x00000001C(x22)      
      li x22,0x0001040ED      
      and x22,x22,x8
      li x4,0x2A96565DBC3BF7BB      
      dcache.civa x22      
      li x22,0x000107160      
      and x22,x22,x8
      li x4,0x60CAAABBFF5E61D9      
      dcache.civa x22      
      li x22,0x0001040F6      
      and x22,x22,x8
      li x4,0xF41F782C5392893      
      sb x4,0x00000001D(x22)      
      li x22,0x0001040E5      
      and x22,x22,x8
      li x4,0xCE0E2A9D53FEFCF9      
      sh x4,0x00000003E(x22)      
      li x22,0x00010200C      
      and x22,x22,x8
      li x4,0x80C1A94AC6BB2D4D      
      sb x4,0x00000003A(x22)      
      li x22,0x00010214C      
      and x22,x22,x8
      li x4,0xE3DD31BEB347C624      
      dcache.civa x22      
      li x22,0x000100142      
      and x22,x22,x8
      li x4,0x5924AE07187E757A      
      lw x5,0x000000028(x22)      
      li x22,0x0001020FF      
      and x22,x22,x8
      li x4,0x5244054875D04790      
      lhu x5,0x000000028(x22)      
      li x22,0x000102105      
      and x22,x22,x8
      li x4,0x9D15822E7AA29CCA      
      lh x5,0x00000003A(x22)      
      li x22,0x00010202B      
      and x22,x22,x8
      li x4,0x61E52FB81F216CE2      
      lhu x5,0x00000000C(x22)      
      li x22,0x00010013E      
      and x22,x22,x8
      li x4,0xA4DC462027C75172      
      sd x4,0x000000030(x22)      
      li x22,0x00010410C      
      and x22,x22,x8
      li x4,0xC9268D95385AB628      
      lb x5,0x000000004(x22)      
      li x22,0x000100170      
      and x22,x22,x8
      li x4,0x5F1DBE418B7A555D      
      lbu x5,0x000000018(x22)      
      li x22,0x0001021C2      
      and x22,x22,x8
      li x4,0x63445766E0650C40      
      dcache.civa x22      
      li x22,0x0001071A8      
      and x22,x22,x8
      li x4,0xFE84BB3CDC8D3BE      
      lhu x5,0x000000046(x22)      
      li x22,0x0001071EA      
      and x22,x22,x8
      li x4,0x216A5097AD2EFFC7      
      sd x4,0x000000048(x22)      
      li x22,0x000102091      
      and x22,x22,x8
      li x4,0x11524735D66D30F5      
      lb x5,0x00000000D(x22)      
      li x22,0x000102094      
      and x22,x22,x8
      li x4,0x8DA3FCE9C6D0C1E6      
      dcache.civa x22      
      li x22,0x0001071E6      
      and x22,x22,x8
      li x4,0xDF689EEAD061D0F8      
      dcache.cva x22      
      li x22,0x000104161      
      and x22,x22,x8
      li x4,0xDC2CE95F93E7A7EC      
      lwu x5,0x000000028(x22)      
      wfi
.global CPU3
CPU3:
      MMODE_SMODE CPU3_S
.global CPU3_S
CPU3_S:
      li x22,0x000107BED      
      and x22,x22,x8
      li x22,0x000107AF2      
      and x22,x22,x8
      li x4,0x56BCF2DDE8B1262C      
      lwu x5,0x000000020(x22)      
      lui x31,0xC5A1B      
      or x31,x13,x5      
      li x22,0x000104125      
      and x22,x22,x8
      li x4,0x449569B18E7689C4      
      dcache.civa x22      
      mulhu x9,x7,x11      
      sltu x15,x27,x18      
      xor x16,x17,x28      
      li x22,0x000107C79      
      and x22,x22,x8
      li x4,0x9C63CC3CD599486B      
      dcache.cva x22      
      divu x11,x22,x5      
      andi x30,x21,0xFFFFFFFFFFFFFAB3      
      li x22,0x000100028      
      and x22,x22,x8
      li x4,0x156D601628FF5109      
      dcache.civa x22      
      or x9,x15,x5      
      slt x29,x25,x31      
      and x29,x17,x23      
      li x22,0x00010201A      
      and x22,x22,x8
      li x4,0x62CC9D9590B745CE      
      sb x4,0x000000038(x22)      
      slti x13,x23,0x000000458      
      addi x28,x3,0xFFFFFFFFFFFFFC09      
      auipc x20,0x42C6D      
      li x22,0x0001077AF      
      and x22,x22,x8
      li x4,0xE5F8754B91371094      
      sh x4,0x000000046(x22)      
      mulh x31,x7,x29      
      ori x27,x15,0xFFFFFFFFFFFFFA35      
      li x22,0x000107A63      
      and x22,x22,x8
      li x4,0xFDF240E06B60F3D8      
      lb x5,0x000000045(x22)      
      mulhsu x11,x10,x17      
      xor x13,x7,x4      
      or x21,x1,x1      
      li x22,0x00010211D      
      and x22,x22,x8
      li x4,0xDAE14B00BCAA7AA6      
      sh x4,0x00000002C(x22)      
      auipc x11,0xC67D2      
      div x15,x7,x27      
      li x22,0x0001041B1      
      and x22,x22,x8
      li x4,0x46066DE105D7BD6      
      ld x5,0x000000040(x22)      
      remu x12,x8,x14      
      addi x17,x25,0xFFFFFFFFFFFFF825      
      li x22,0x000102071      
      and x22,x22,x8
      li x4,0x6BC46DD97A36D0EE      
      lw x5,0x000000020(x22)      
      mulhsu x16,x26,x20      
      xor x13,x31,x14      
      sltiu x13,x27,0x0000006CF      
      li x22,0x000104185      
      and x22,x22,x8
      li x4,0xFB29C15DAFB02BA1      
      lh x5,0x000000030(x22)      
      mulh x14,x23,x9      
      and x10,x29,x26      
      auipc x18,0x5AFE8      
      li x22,0x0001021A2      
      and x22,x22,x8
      li x4,0x8905420C4F79EBC8      
      sw x4,0x000000018(x22)      
      xor x15,x22,x20      
      andi x9,x19,0xFFFFFFFFFFFFFB5E      
      rem x20,x25,x14      
      li x22,0x000102173      
      and x22,x22,x8
      li x4,0x8CDC0463DDBF7832      
      sw x4,0x00000004C(x22)      
      andi x11,x0,0xFFFFFFFFFFFFFBFC      
      andi x23,x31,0xFFFFFFFFFFFFFEFC      
      mulh x25,x31,x9      
      li x22,0x000107CE3      
      and x22,x22,x8
      li x4,0x9F44602801693606      
      dcache.cva x22      
      sll x24,x6,x23      
      sra x24,x5,x16      
      mulhu x11,x20,x12      
      li x22,0x0001078BF      
      and x22,x22,x8
      li x4,0xE5FACE934DEAB8C7      
      dcache.cva x22      
      andi x13,x13,0x00000051D      
      srai x19,x21,0x08      
      remu x27,x18,x2      
      li x22,0x000107B66      
      and x22,x22,x8
      li x4,0xAD14F93844401E2D      
      dcache.civa x22      
      or x28,x25,x8      
      sltu x24,x21,x27      
      and x9,x4,x9      
      li x22,0x00010019B      
      and x22,x22,x8
      li x4,0x2E5CFC76782B0AA4      
      sb x4,0x000000036(x22)      
      srl x9,x18,x1      
      lui x9,0xD64B8      
      ori x10,x13,0xFFFFFFFFFFFFF9BA      
      li x22,0x000107A80      
      and x22,x22,x8
      li x4,0x772F74409C1217D1      
      ld x5,0x000000040(x22)      
      mulhu x23,x5,x17      
      rem x31,x30,x19      
      sll x19,x24,x5      
      li x22,0x000107B2D      
      and x22,x22,x8
      li x4,0xB81A62DA521C5FE7      
      sh x4,0x000000032(x22)      
      auipc x27,0x36E3D      
      mulhu x14,x15,x5      
      li x22,0x000107C2A      
      and x22,x22,x8
      li x4,0x86F654379324C39E      
      lw x5,0x000000038(x22)      
      div x9,x2,x8      
      remu x21,x29,x22      
      li x22,0x00010771D      
      and x22,x22,x8
      li x4,0x4890DF998A72CC46      
      lh x5,0x00000002E(x22)      
      ori x9,x18,0x00000037D      
      auipc x30,0x54199      
      li x22,0x0001077EA      
      and x22,x22,x8
      li x4,0x74AAAF863BC8361D      
      sw x4,0x000000028(x22)      
      and x26,x16,x11      
      or x27,x19,x17      
      li x22,0x000107B2D      
      and x22,x22,x8
      li x4,0xD5EFDD94866B220F      
      lhu x5,0x00000004E(x22)      
      sltiu x14,x15,0xFFFFFFFFFFFFF945      
      srl x19,x12,x28      
      srl x6,x19,x9      
      li x22,0x000107AB7      
      and x22,x22,x8
      li x4,0x942D5ADD4185E5BF      
      dcache.civa x22      
      xor x25,x14,x31      
      add x10,x26,x23      
      li x22,0x000107A46      
      and x22,x22,x8
      li x4,0xC28D4DB92D09C8DD      
      dcache.civa x22      
      sll x27,x5,x9      
      mul x15,x9,x6      
      xori x30,x9,0x0000005BA      
      li x22,0x000107707      
      and x22,x22,x8
      li x4,0xCDC75870846FB9FA      
      lb x5,0x000000006(x22)      
      slti x27,x7,0x000000550      
      srli x20,x3,0x09      
      li x22,0x000107CD3      
      and x22,x22,x8
      li x4,0xA2FD50A964E94B4B      
      dcache.cva x22      
      mulh x20,x30,x30      
      mulhu x27,x24,x19      
      andi x29,x30,0x000000092      
      li x22,0x00010790E      
      and x22,x22,x8
      li x4,0xF94D1BB2E0E8C60      
      sb x4,0x000000018(x22)      
      sll x21,x27,x24      
      sra x29,x24,x31      
      li x22,0x000100051      
      and x22,x22,x8
      li x4,0xD618AC13950B432E      
      sb x4,0x000000003(x22)      
      div x19,x25,x17      
      srl x13,x15,x30      
      slti x21,x27,0xFFFFFFFFFFFFF9E1      
      li x22,0x00010016D      
      and x22,x22,x8
      li x4,0x9116764038EE7211      
      dcache.cva x22      
      slt x24,x0,x25      
      mulhsu x14,x17,x27      
      mul x30,x29,x1      
      li x22,0x000107AAB      
      and x22,x22,x8
      li x4,0x486E11D5E23B4845      
      dcache.cva x22      
      rem x27,x30,x24      
      slt x29,x6,x4      
      auipc x26,0xA501D      
      li x22,0x000102069      
      and x22,x22,x8
      li x4,0xCE1B3AD9443A2CFD      
      sb x4,0x000000006(x22)      
      slt x29,x31,x26      
      rem x6,x23,x26      
      li x22,0x000107A02      
      and x22,x22,x8
      li x4,0xC354A347FCEA48F6      
      lwu x5,0x000000000(x22)      
      lui x16,0x8CBF0      
      mulhsu x10,x5,x3      
      li x22,0x00010004E      
      and x22,x22,x8
      li x4,0xBA908FD7090C396A      
      ld x5,0x000000008(x22)      
      divu x9,x14,x23      
      sll x16,x21,x22      
      xori x27,x12,0x0000005AF      
      li x22,0x0001079CF      
      and x22,x22,x8
      li x4,0x922A36B4BCE536A7      
      lw x5,0x00000004C(x22)      
      xor x21,x11,x6      
      lui x28,0x20BF0      
      li x22,0x00010790D      
      and x22,x22,x8
      li x4,0x182DD953BFC26D51      
      dcache.civa x22      
      remu x16,x10,x22      
      slli x11,x7,0x018      
      li x22,0x000100096      
      and x22,x22,x8
      li x4,0x8B8A0FDE02C58FB4      
      dcache.cva x22      
      addi x21,x24,0x0000004F8      
      rem x28,x7,x18      
      li x22,0x00010771B      
      and x22,x22,x8
      li x4,0x9BB4666A072825E      
      sh x4,0x00000004E(x22)      
      srl x21,x14,x9      
      ori x17,x13,0x0000001C6      
      li x22,0x000107CDC      
      and x22,x22,x8
      li x4,0x15042791EC2BB4E4      
      lhu x5,0x00000004C(x22)      
      slti x20,x28,0xFFFFFFFFFFFFFAD9      
      mulh x23,x15,x25      
      li x22,0x0001021A2      
      and x22,x22,x8
      li x4,0xA601F2DA1A81C0B      
      dcache.cva x22      
      sub x12,x4,x30      
      div x31,x9,x22      
      sll x29,x8,x31      
      li x22,0x000107BB4      
      and x22,x22,x8
      li x4,0xC0C5C384B68850C      
      dcache.civa x22      
      slli x27,x30,0x0F      
      auipc x11,0x82639      
      add x31,x19,x20      
      li x22,0x0001021EF      
      and x22,x22,x8
      li x4,0x13870DB2883B1E01      
      lh x5,0x000000048(x22)      
      add x11,x10,x11      
      addi x26,x27,0x000000714      
      ori x12,x23,0xFFFFFFFFFFFFFBE1      
      li x22,0x000100194      
      and x22,x22,x8
      li x4,0x16F6EAF17338C0F7      
      lbu x5,0x000000015(x22)      
      addi x23,x21,0x0000004E6      
      rem x9,x20,x31      
      li x22,0x000100130      
      and x22,x22,x8
      li x4,0x65233483656EA649      
      ld x5,0x000000010(x22)      
      add x13,x18,x10      
      div x16,x1,x18      
      and x19,x25,x25      
      li x22,0x0001040D2      
      and x22,x22,x8
      li x4,0xABAD7CA73850D4EA      
      sd x4,0x000000028(x22)      
      mulh x27,x20,x16      
      srl x29,x30,x3      
      li x22,0x0001078AA      
      and x22,x22,x8
      li x4,0x58DD146924985576      
      dcache.civa x22      
      auipc x10,0x89D94      
      slt x18,x8,x1      
      div x14,x26,x18      
      li x22,0x00010798E      
      and x22,x22,x8
      li x4,0x94EFED27BA98167B      
      dcache.cva x22      
      lui x12,0x3E62B      
      rem x27,x0,x16      
      li x22,0x00010212A      
      and x22,x22,x8
      li x4,0x669CB997F74CBA18      
      sh x4,0x000000036(x22)      
      divu x7,x29,x21      
      sub x25,x19,x20      
      li x22,0x00010779E      
      and x22,x22,x8
      li x4,0x1317020ECA6E18A9      
      dcache.cva x22      
      and x26,x22,x0      
      addi x18,x30,0xFFFFFFFFFFFFFEA6      
      li x22,0x00010203F      
      and x22,x22,x8
      li x4,0xAAB876623E578C84      
      dcache.cva x22      
      mulhsu x6,x13,x22      
      sltu x12,x10,x29      
      slli x15,x27,0x015      
      li x22,0x00010401F      
      and x22,x22,x8
      li x4,0x684E94DE512DE9F      
      dcache.civa x22      
      slti x20,x28,0x00000051E      
      remu x29,x20,x29      
      mulh x23,x1,x11      
      li x22,0x000102029      
      and x22,x22,x8
      li x4,0x175324EAB987FC14      
      sb x4,0x000000024(x22)      
      mulhu x21,x8,x20      
      sub x10,x2,x11      
      li x22,0x000102044      
      and x22,x22,x8
      li x4,0x7AD77CF68B59E279      
      dcache.civa x22      
      sub x15,x12,x25      
      sltiu x15,x12,0xFFFFFFFFFFFFF83D      
      li x22,0x000107CB1      
      and x22,x22,x8
      li x4,0xC7782B9F387F2676      
      sh x4,0x00000001A(x22)      
      ori x20,x23,0xFFFFFFFFFFFFF99A      
      auipc x26,0x184C5      
      srai x19,x19,0x01F      
      li x22,0x000107A73      
      and x22,x22,x8
      li x4,0xC36DE3C083D78673      
      ld x5,0x000000040(x22)      
      addi x7,x26,0xFFFFFFFFFFFFFDD4      
      sub x12,x25,x9      
      li x22,0x0001000EF      
      and x22,x22,x8
      li x4,0xD1C6FC0927B461A0      
      ld x5,0x000000038(x22)      
      sll x11,x12,x29      
      sltiu x7,x17,0x00000047B      
      or x31,x17,x13      
      li x22,0x000102169      
      and x22,x22,x8
      li x4,0x184AC488DB668073      
      dcache.cva x22      
      and x6,x16,x3      
      lui x7,0xD24BD      
      li x22,0x0001041CA      
      and x22,x22,x8
      li x4,0x1FD9A3CCA5E92C1A      
      dcache.cva x22      
      slti x9,x30,0xFFFFFFFFFFFFFE15      
      xori x31,x30,0xFFFFFFFFFFFFFF4D      
      sll x14,x6,x19      
      li x22,0x000107BFD      
      and x22,x22,x8
      li x4,0x5353E9FEC4CE372A      
      lwu x5,0x00000002C(x22)      
      divu x25,x28,x22      
      lui x7,0xE579      
      addi x26,x8,0xFFFFFFFFFFFFFB3F      
      li x22,0x000107C76      
      and x22,x22,x8
      li x4,0xF9B59A514B23603F      
      sb x4,0x000000040(x22)      
      add x16,x20,x20      
      slt x25,x19,x30      
      mulhu x17,x25,x14      
      li x22,0x000100081      
      and x22,x22,x8
      li x4,0x91DD65CBD835DB1B      
      lb x5,0x000000014(x22)      
      rem x16,x3,x27      
      auipc x21,0x9B372      
      li x22,0x000100152      
      and x22,x22,x8
      li x4,0xC149A3592ED9F222      
      sb x4,0x000000018(x22)      
      mulhsu x28,x24,x15      
      srl x30,x11,x8      
      li x22,0x000107A45      
      and x22,x22,x8
      li x4,0xB105D5C5E25C1A40      
      dcache.cva x22      
      div x31,x10,x28      
      rem x13,x25,x26      
      li x22,0x000104107      
      and x22,x22,x8
      li x4,0x32942EB050E6553B      
      dcache.civa x22      
      sra x20,x5,x25      
      ori x26,x21,0x00000059F      
      li x22,0x0001020F4      
      and x22,x22,x8
      li x4,0x1BB7260FA6B801B9      
      sh x4,0x00000002C(x22)      
      slli x31,x15,0x01F      
      sltu x15,x24,x9      
      li x22,0x000104120      
      and x22,x22,x8
      li x4,0x87D2F37E34F519E      
      lwu x5,0x000000044(x22)      
      div x10,x0,x29      
      slti x12,x15,0x000000005      
      li x22,0x000104098      
      and x22,x22,x8
      li x4,0x262EDD904FC163E0      
      lhu x5,0x000000046(x22)      
      slli x7,x10,0x012      
      addi x29,x0,0xFFFFFFFFFFFFF979      
      and x29,x18,x4      
      li x22,0x00010792E      
      and x22,x22,x8
      li x4,0x5D540770790090B1      
      dcache.civa x22      
      ori x30,x16,0x00000000B      
      ori x28,x23,0x0000001D5      
      li x22,0x0001079EF      
      and x22,x22,x8
      li x4,0x188087895ABB2BF      
      lhu x5,0x00000000A(x22)      
      sub x31,x7,x29      
      addi x9,x0,0xFFFFFFFFFFFFFE0D      
      div x14,x27,x11      
      li x22,0x000107CFD      
      and x22,x22,x8
      li x4,0x4D4D528FB5E6F419      
      sh x4,0x000000044(x22)      
      div x23,x14,x21      
      auipc x29,0x46AFF      
      li x22,0x00010003F      
      and x22,x22,x8
      li x4,0xB8DF0B4670F995FE      
      sw x4,0x00000004C(x22)      
      sltiu x14,x1,0xFFFFFFFFFFFFFE08      
      sltu x25,x0,x26      
      mul x27,x7,x14      
      li x22,0x0001079A8      
      and x22,x22,x8
      li x4,0xF6E8BF046318A0D9      
      dcache.civa x22      
      srai x28,x30,0x0B      
      sltiu x28,x31,0xFFFFFFFFFFFFF86F      
      li x22,0x0001020A3      
      and x22,x22,x8
      li x4,0x3639830D2A1FDD0      
      dcache.civa x22      
      srli x14,x8,0x01E      
      sltu x25,x4,x26      
      li x22,0x000107A63      
      and x22,x22,x8
      li x4,0x71A3DB1FCADCB7F6      
      ld x5,0x000000040(x22)      
      auipc x31,0xE51C3      
      remu x29,x10,x14      
      li x22,0x0001000E5      
      and x22,x22,x8
      li x4,0xA1203471C5C5BC98      
      lw x5,0x000000020(x22)      
      srai x31,x9,0x01C      
      and x12,x3,x24      
      li x22,0x0001021EE      
      and x22,x22,x8
      li x4,0xD673552596925C45      
      dcache.cva x22      
      sltiu x31,x27,0xFFFFFFFFFFFFFFDA      
      srli x16,x0,0x01E      
      srl x24,x10,x10      
      li x22,0x000107C2A      
      and x22,x22,x8
      li x4,0x2A2B187E758EE7E0      
      sd x4,0x000000010(x22)      
      ori x14,x24,0xFFFFFFFFFFFFFA34      
      ori x26,x3,0xFFFFFFFFFFFFFE26      
      li x22,0x00010202B      
      and x22,x22,x8
      li x4,0x2A821E756C7D92E0      
      lh x5,0x000000018(x22)      
      srl x9,x20,x23      
      slli x24,x16,0x01D      
      and x26,x31,x17      
      li x22,0x000107B14      
      and x22,x22,x8
      li x4,0xC5ABBF7445001009      
      lwu x5,0x000000024(x22)      
      mulhsu x7,x16,x13      
      sll x26,x16,x1      
      xori x20,x15,0xFFFFFFFFFFFFFBAE      
      li x22,0x00010212A      
      and x22,x22,x8
      li x4,0xB9466239A80E1283      
      lbu x5,0x00000001E(x22)      
      slti x12,x5,0x000000559      
      rem x7,x9,x17      
      li x22,0x000107951      
      and x22,x22,x8
      li x4,0x5E3869CB40AD5FDB      
      dcache.civa x22      
      slt x7,x29,x29      
      mulh x20,x1,x8      
      andi x23,x14,0x0000005A2      
      li x22,0x0001079A0      
      and x22,x22,x8
      li x4,0x54792D7F9238B8D6      
      lwu x5,0x000000040(x22)      
      or x20,x15,x9      
      ori x28,x18,0xFFFFFFFFFFFFFF54      
      ori x9,x14,0x00000070F      
      li x22,0x000100179      
      and x22,x22,x8
      li x4,0x901EABABF2A96384      
      dcache.civa x22      
      add x26,x21,x23      
      mulh x14,x19,x24      
      mulhsu x29,x28,x5      
      li x22,0x0001001C7      
      and x22,x22,x8
      li x4,0x3271770704580E8E      
      lw x5,0x000000024(x22)      
      slt x26,x12,x31      
      srai x24,x15,0x01B      
      srai x23,x22,0x014      
      li x22,0x000102034      
      and x22,x22,x8
      li x4,0xFBC7E7FEC06B355E      
      dcache.cva x22      
      sll x13,x7,x17      
      andi x11,x24,0xFFFFFFFFFFFFFE40      
      li x22,0x0001041F5      
      and x22,x22,x8
      li x4,0x19B1E1F3F17EC758      
      ld x5,0x000000030(x22)      
      auipc x24,0x43A6B      
      mulhu x24,x30,x4      
      li x22,0x000107CE4      
      and x22,x22,x8
      li x4,0x35EE47E70E62242D      
      sb x4,0x000000028(x22)      
      slt x13,x11,x7      
      add x26,x29,x1      
      li x22,0x000104098      
      and x22,x22,x8
      li x4,0x50489CD6601F6630      
      sb x4,0x000000006(x22)      
      or x31,x4,x10      
      auipc x25,0xD3D1C      
      li x22,0x00010405C      
      and x22,x22,x8
      li x4,0xCC7545A6966D4B15      
      dcache.civa x22      
      rem x10,x26,x8      
      sll x10,x9,x29      
      li x22,0x000107AEC      
      and x22,x22,x8
      li x4,0xF8DB28128D9410DC      
      dcache.civa x22      
      xori x15,x17,0x00000038D      
      auipc x28,0xF196E      
      remu x15,x3,x12      
      li x22,0x0001040C8      
      and x22,x22,x8
      li x4,0x46A39FAB7912B0A1      
      dcache.civa x22      
      andi x13,x2,0xFFFFFFFFFFFFF842      
      sll x27,x28,x13      
      mulhu x14,x10,x20      
      li x22,0x0001041F6      
      and x22,x22,x8
      li x4,0x3DD74CAFDD3BED39      
      lb x5,0x00000000A(x22)      
      sltu x14,x30,x31      
      ori x11,x11,0xFFFFFFFFFFFFFB41      
      srai x11,x8,0x0E      
      li x22,0x0001020BD      
      and x22,x22,x8
      li x4,0x504271052F544673      
      lh x5,0x000000012(x22)      
      remu x17,x19,x13      
      mulhsu x6,x26,x5      
      li x22,0x00010786C      
      and x22,x22,x8
      li x4,0x9745A307552E9D2D      
      sw x4,0x000000044(x22)      
      div x16,x14,x23      
      and x15,x12,x24      
      li x22,0x000100074      
      and x22,x22,x8
      li x4,0xF55D9EB88E2FEB5F      
      dcache.civa x22      
      addi x20,x6,0xFFFFFFFFFFFFFE52      
      sra x9,x15,x21      
      li x22,0x00010008F      
      and x22,x22,x8
      li x4,0x65066FBC5003BBED      
      dcache.cva x22      
      mulh x15,x0,x30      
      srli x30,x11,0x01E      
      li x22,0x000102096      
      and x22,x22,x8
      li x4,0xD21259E3F666DE1A      
      sb x4,0x000000000(x22)      
      mulh x10,x23,x20      
      slt x28,x9,x23      
      addi x25,x0,0x0000003AD      
      li x22,0x000107C6F      
      and x22,x22,x8
      li x4,0xC123EF3E303161C      
      ld x5,0x000000038(x22)      
      lui x10,0x5FD0      
      sra x19,x17,x25      
      li x22,0x000107B41      
      and x22,x22,x8
      li x4,0xDE28DDB37A7E9F43      
      sw x4,0x000000044(x22)      
      slt x9,x13,x28      
      xor x18,x3,x27      
      slli x17,x4,0x017      
      li x22,0x000107CC4      
      and x22,x22,x8
      li x4,0x77D04074BD0CF6FE      
      lw x5,0x00000001C(x22)      
      slt x10,x0,x7      
      divu x31,x6,x1      
      auipc x29,0xD2EE9      
      li x22,0x00010407E      
      and x22,x22,x8
      li x4,0xDC2BFAB45DFBA3C2      
      lh x5,0x000000036(x22)      
      slti x30,x26,0x000000764      
      srai x19,x4,0x016      
      li x22,0x000107B69      
      and x22,x22,x8
      li x4,0x34AE39C3002FF54F      
      dcache.cva x22      
      andi x7,x17,0xFFFFFFFFFFFFFD5F      
      sltu x10,x28,x21      
      ori x25,x27,0x000000337      
      li x22,0x00010777A      
      and x22,x22,x8
      li x4,0xAE6FE5010EB2B5A4      
      lh x5,0x000000048(x22)      
      sll x25,x0,x24      
      sltiu x15,x5,0xFFFFFFFFFFFFF980      
      li x22,0x000107BD0      
      and x22,x22,x8
      li x4,0x1DB260ADF1DCA1FA      
      dcache.civa x22      
      xori x31,x23,0xFFFFFFFFFFFFFB51      
      sll x27,x0,x16      
      srl x26,x16,x10      
      li x22,0x0001040AA      
      and x22,x22,x8
      li x4,0xEB89A68C1EC586F0      
      sh x4,0x000000042(x22)      
      mulh x7,x13,x13      
      slli x7,x5,0x01C      
      srl x25,x6,x19      
      li x22,0x000107C81      
      and x22,x22,x8
      li x4,0xC06651BC0793D089      
      lhu x5,0x000000018(x22)      
      auipc x20,0x45878      
      mulhu x26,x15,x21      
      li x22,0x0001077B1      
      and x22,x22,x8
      li x4,0xF4CB0841BD20D2A6      
      ld x5,0x000000030(x22)      
      remu x10,x0,x29      
      sltu x12,x11,x8      
      sll x28,x24,x12      
      li x22,0x000104050      
      and x22,x22,x8
      li x4,0xB1896D9B18ABE7E6      
      sb x4,0x000000049(x22)      
      mulhsu x30,x27,x2      
      andi x14,x15,0x00000079C      
      li x22,0x000100122      
      and x22,x22,x8
      li x4,0xD1C51805315C1D09      
      lwu x5,0x00000002C(x22)      
      srl x31,x13,x19      
      slli x23,x14,0x00      
      addi x21,x28,0x00000053E      
      li x22,0x000102132      
      and x22,x22,x8
      li x4,0x74F0ED3A9589C706      
      lb x5,0x000000027(x22)      
      slti x24,x1,0xFFFFFFFFFFFFFA10      
      slli x17,x10,0x018      
      li x22,0x000107826      
      and x22,x22,x8
      li x4,0xBE3C485D3BDAA744      
      sb x4,0x00000003A(x22)      
      sra x9,x11,x10      
      sra x7,x8,x14      
      li x22,0x000107947      
      and x22,x22,x8
      li x4,0xA875B9094A0D0918      
      dcache.cva x22      
      sra x30,x11,x2      
      mulhsu x13,x4,x8      
      li x22,0x000107CD6      
      and x22,x22,x8
      li x4,0x8D36BC3524DB78DF      
      lhu x5,0x000000038(x22)      
      slt x27,x8,x11      
      rem x24,x17,x10      
      slli x26,x30,0x011      
      li x22,0x00010777D      
      and x22,x22,x8
      li x4,0x2780A59176C3E9FC      
      dcache.civa x22      
      srl x20,x2,x8      
      andi x11,x23,0x00000029B      
      srai x20,x18,0x0B      
      li x22,0x000107A3C      
      and x22,x22,x8
      li x4,0xB1F9AE50A5DBD6E7      
      sd x4,0x000000018(x22)      
      slt x11,x23,x19      
      srli x26,x11,0x01E      
      li x22,0x000100059      
      and x22,x22,x8
      li x4,0x50F91654DB2AD8CA      
      sh x4,0x00000004E(x22)      
      lui x13,0xF0A7      
      rem x17,x5,x8      
      xor x21,x18,x3      
      li x22,0x000102114      
      and x22,x22,x8
      li x4,0x9E26EECCF0EA171D      
      lhu x5,0x00000004E(x22)      
      or x21,x24,x9      
      divu x19,x16,x19      
      li x22,0x000107C61      
      and x22,x22,x8
      li x4,0xF7CFA983FC00CA6C      
      lbu x5,0x000000026(x22)      
      srl x9,x23,x29      
      mulh x14,x11,x23      
      xor x21,x22,x12      
      li x22,0x00010010D      
      and x22,x22,x8
      li x4,0x99AF18443A406D83      
      sh x4,0x00000001A(x22)      
      mulhsu x28,x22,x2      
      rem x7,x16,x13      
      li x22,0x000107C4D      
      and x22,x22,x8
      li x4,0x8A735B3BE5D32646      
      sd x4,0x000000018(x22)      
      slt x24,x10,x31      
      auipc x19,0x628B6      
      mul x29,x5,x22      
      li x22,0x0001077E1      
      and x22,x22,x8
      li x4,0xAEDC51EBAC18BD0F      
      sd x4,0x000000040(x22)      
      slli x9,x0,0x0C      
      sltu x23,x28,x30      
      li x22,0x000102107      
      and x22,x22,x8
      li x4,0x9A107FD053BE07D      
      sw x4,0x000000024(x22)      
      andi x19,x12,0xFFFFFFFFFFFFFA7C      
      mulhu x24,x28,x2      
      li x22,0x000100083      
      and x22,x22,x8
      li x4,0xCE6477302DFC7A7      
      dcache.civa x22      
      mul x14,x15,x7      
      slli x27,x23,0x04      
      auipc x11,0x3BD43      
      li x22,0x000107B45      
      and x22,x22,x8
      li x4,0xE2B91F3CE9858A2E      
      lw x5,0x000000000(x22)      
      sltu x28,x30,x17      
      and x16,x0,x9      
      li x22,0x0001021A6      
      and x22,x22,x8
      li x4,0x8FE061D4A47A2A8D      
      dcache.cva x22      
      srl x10,x31,x28      
      rem x14,x25,x24      
      rem x28,x21,x14      
      li x22,0x000107931      
      and x22,x22,x8
      li x4,0xBDD6662CE8EE4699      
      sb x4,0x000000032(x22)      
      sltiu x7,x23,0x000000597      
      mulhsu x18,x17,x21      
      li x22,0x0001078BC      
      and x22,x22,x8
      li x4,0x7611CB4FCD0FFA07      
      lh x5,0x000000034(x22)      
      divu x24,x9,x1      
      sll x29,x3,x7      
      add x30,x0,x13      
      li x22,0x0001078B9      
      and x22,x22,x8
      li x4,0xD3E987C8610B8251      
      lhu x5,0x00000004C(x22)      
      auipc x13,0x2FADD      
      sra x7,x6,x7      
      lui x13,0xCDE83      
      li x22,0x00010203D      
      and x22,x22,x8
      li x4,0x20867D399BB0AAF9      
      sb x4,0x00000003A(x22)      
      xori x9,x29,0xFFFFFFFFFFFFFF9C      
      sra x9,x25,x31      
      slli x21,x8,0x0B      
      li x22,0x0001078D5      
      and x22,x22,x8
      li x4,0x96B401EFFE8C8C98      
      lwu x5,0x000000004(x22)      
      ori x23,x1,0xFFFFFFFFFFFFFC07      
      mul x14,x24,x5      
      mulh x23,x26,x24      
      li x22,0x000104095      
      and x22,x22,x8
      li x4,0xDB82B68AD85EAD56      
      lbu x5,0x000000038(x22)      
      mulh x25,x20,x7      
      or x9,x3,x8      
      xori x25,x5,0xFFFFFFFFFFFFF887      
      li x22,0x0001040F4      
      and x22,x22,x8
      li x4,0xC901BE33C03CA0E4      
      ld x5,0x000000030(x22)      
      or x25,x27,x22      
      or x7,x21,x17      
      slt x26,x29,x19      
      li x22,0x0001020E2      
      and x22,x22,x8
      li x4,0xB0E9D9F818296CA5      
      dcache.civa x22      
      sltiu x11,x5,0x00000042D      
      add x12,x27,x18      
      mul x7,x0,x18      
      li x22,0x000107779      
      and x22,x22,x8
      li x4,0x7211DFAC596F9553      
      lhu x5,0x000000020(x22)      
      slti x20,x10,0x0000001B0      
      xor x9,x15,x11      
      li x22,0x0001078DE      
      and x22,x22,x8
      li x4,0xC8D88B9DF2D5991      
      lwu x5,0x00000000C(x22)      
      slt x26,x21,x26      
      mulhu x19,x15,x1      
      xori x30,x24,0x00000045B      
      li x22,0x00010207B      
      and x22,x22,x8
      li x4,0x5616EA776A2D0AEC      
      lw x5,0x000000018(x22)      
      xor x6,x21,x15      
      remu x13,x7,x23      
      mulh x24,x14,x25      
      li x22,0x00010408F      
      and x22,x22,x8
      li x4,0x75A06AF8FD883894      
      dcache.cva x22      
      divu x11,x5,x17      
      ori x13,x17,0x000000260      
      add x9,x24,x25      
      li x22,0x000107AB2      
      and x22,x22,x8
      li x4,0x2D347D8A883EE3BE      
      dcache.civa x22      
      andi x9,x24,0x000000653      
      andi x14,x8,0x000000452      
      sub x10,x27,x1      
      li x22,0x0001079C7      
      and x22,x22,x8
      li x4,0xC5761287CE36E4DE      
      ld x5,0x000000008(x22)      
      sub x14,x19,x9      
      srai x25,x22,0x07      
      srl x7,x15,x27      
      li x22,0x0001077A6      
      and x22,x22,x8
      li x4,0xFE9199090A52FFE      
      dcache.cva x22      
      xori x31,x25,0xFFFFFFFFFFFFF95E      
      div x13,x7,x7      
      and x9,x7,x30      
      li x22,0x000107743      
      and x22,x22,x8
      li x4,0x60A23AD1701B34D0      
      sw x4,0x000000040(x22)      
      mulhsu x7,x17,x2      
      sll x14,x18,x2      
      slti x28,x14,0xFFFFFFFFFFFFFEBD      
      li x22,0x000107A78      
      and x22,x22,x8
      li x4,0x9A50B3C686B06B37      
      dcache.civa x22      
      srli x28,x29,0x08      
      rem x31,x29,x7      
      sra x29,x14,x6      
      li x22,0x000107A98      
      and x22,x22,x8
      li x4,0xE7C48746DDFB5AFC      
      dcache.civa x22      
      sll x23,x17,x18      
      and x6,x23,x2      
      mulhu x14,x6,x13      
      li x22,0x0001000E1      
      and x22,x22,x8
      li x4,0x28074229F450CF44      
      sb x4,0x00000002B(x22)      
      sub x11,x26,x9      
      srai x17,x17,0x0E      
      li x22,0x000102027      
      and x22,x22,x8
      li x4,0xD805A110F182CF4E      
      dcache.civa x22      
      sltiu x31,x3,0x000000096      
      or x20,x29,x7      
      li x22,0x0001041DB      
      and x22,x22,x8
      li x4,0x863CA0772EA77AB1      
      dcache.cva x22      
      ori x29,x31,0x000000728      
      slli x26,x14,0x06      
      lui x26,0x1444D      
      li x22,0x000107A4F      
      and x22,x22,x8
      li x4,0x495283F286965865      
      lh x5,0x00000000A(x22)      
      ori x11,x3,0xFFFFFFFFFFFFFACE      
      div x7,x17,x23      
      addi x14,x20,0x0000000D1      
      li x22,0x000107782      
      and x22,x22,x8
      li x4,0xFC2FD91C2BC93288      
      lbu x5,0x000000048(x22)      
      sltu x25,x20,x4      
      remu x24,x24,x10      
      li x22,0x000107C63      
      and x22,x22,x8
      li x4,0x2526D862298EE75F      
      lh x5,0x00000000E(x22)      
      divu x10,x22,x27      
      divu x26,x12,x6      
      div x23,x20,x22      
      li x22,0x000107770      
      and x22,x22,x8
      li x4,0x9DED3CE71619F7F3      
      sd x4,0x000000028(x22)      
      ori x24,x13,0xFFFFFFFFFFFFF9BA      
      xor x17,x23,x26      
      mulhsu x16,x29,x13      
      li x22,0x000107C8F      
      and x22,x22,x8
      li x4,0x1FFDDF526C117486      
      dcache.cva x22      
      mulh x25,x7,x20      
      div x26,x25,x26      
      sltu x10,x17,x0      
      li x22,0x0001021B9      
      and x22,x22,x8
      li x4,0xA048D6887724AF48      
      dcache.civa x22      
      sra x12,x30,x15      
      mulhsu x16,x7,x3      
      ori x23,x5,0xFFFFFFFFFFFFFB06      
      li x22,0x000102062      
      and x22,x22,x8
      li x4,0x2D65D02B1B41444F      
      dcache.civa x22      
      sll x12,x20,x7      
      srli x25,x31,0x01F      
      ori x23,x8,0x0000004FE      
      li x22,0x00010214C      
      and x22,x22,x8
      li x4,0x95CB25A42BC36FDD      
      dcache.civa x22      
      rem x24,x5,x2      
      remu x14,x22,x25      
      sltu x27,x10,x9      
      li x22,0x000100174      
      and x22,x22,x8
      li x4,0xCB65E1792D3DC079      
      lbu x5,0x000000047(x22)      
      mulhsu x23,x25,x20      
      auipc x6,0x41310      
      li x22,0x0001079BE      
      and x22,x22,x8
      li x4,0x2C2030192A756C7A      
      lh x5,0x00000000A(x22)      
      xor x28,x24,x31      
      and x23,x4,x5      
      li x22,0x000107BC7      
      and x22,x22,x8
      li x4,0x19C1E9A3E3FFCF3F      
      dcache.civa x22      
      srai x17,x16,0x01B      
      auipc x26,0x997AA      
      rem x12,x13,x3      
      li x22,0x000107836      
      and x22,x22,x8
      li x4,0x54489083F4917B5A      
      lh x5,0x000000022(x22)      
      ori x12,x9,0x000000766      
      andi x19,x7,0xFFFFFFFFFFFFFC0C      
      xor x19,x17,x30      
      li x22,0x0001077E1      
      and x22,x22,x8
      li x4,0xE4A7ED246EAAD966      
      dcache.cva x22      
      sll x24,x10,x7      
      addi x23,x4,0x0000006E8      
      sltiu x28,x16,0x000000691      
      li x22,0x0001000C5      
      and x22,x22,x8
      li x4,0x3EFFA396C08C965C      
      dcache.civa x22      
      divu x13,x10,x26      
      slti x23,x27,0xFFFFFFFFFFFFFF3C      
      slt x15,x26,x14      
      li x22,0x000100050      
      and x22,x22,x8
      li x4,0x652BD9043E3C32BD      
      dcache.cva x22      
      slti x27,x21,0xFFFFFFFFFFFFF8F3      
      lui x30,0x971EF      
      xor x16,x12,x13      
      li x22,0x000104003      
      and x22,x22,x8
      li x4,0xE029C24A1026E666      
      sw x4,0x000000020(x22)      
      mul x29,x13,x6      
      sltu x26,x3,x26      
      li x22,0x00010219E      
      and x22,x22,x8
      li x4,0x15B69D3FFA669667      
      sb x4,0x000000046(x22)      
      lui x27,0xFF636      
      srli x10,x9,0x01F      
      li x22,0x000100063      
      and x22,x22,x8
      li x4,0x846983A3D26849F3      
      lwu x5,0x00000000C(x22)      
      add x30,x19,x26      
      mulh x20,x25,x2      
      slli x29,x8,0x05      
      li x22,0x0001077A6      
      and x22,x22,x8
      li x4,0xC38371171F7CB763      
      lw x5,0x000000010(x22)      
      sll x18,x28,x7      
      sra x13,x1,x27      
      sub x24,x13,x29      
      li x22,0x0001079D0      
      and x22,x22,x8
      li x4,0xA38D5432B89EBB67      
      lbu x5,0x00000000C(x22)      
      remu x28,x29,x14      
      slt x6,x22,x5      
      li x22,0x00010797B      
      and x22,x22,x8
      li x4,0xE42327686A0EAC51      
      sb x4,0x00000001A(x22)      
      slli x12,x18,0x02      
      lui x10,0x830D2      
      li x22,0x0001021E4      
      and x22,x22,x8
      li x4,0x8A736558EE74B029      
      lbu x5,0x000000014(x22)      
      srli x20,x1,0x01      
      slt x9,x3,x18      
      slt x14,x16,x17      
      li x22,0x000104188      
      and x22,x22,x8
      li x4,0x3B7AE75A73BC307E      
      dcache.cva x22      
      divu x27,x23,x11      
      sll x12,x29,x13      
      li x22,0x0001021F7      
      and x22,x22,x8
      li x4,0xC832756A6844D82F      
      sw x4,0x000000034(x22)      
      mulhu x10,x31,x12      
      mulhu x10,x7,x11      
      sll x9,x22,x14      
      li x22,0x00010406B      
      and x22,x22,x8
      li x4,0xC4F3F1F3A749F4B2      
      sd x4,0x000000030(x22)      
      mul x23,x14,x2      
      srai x23,x15,0x016      
      mulh x18,x7,x25      
      li x22,0x0001000E7      
      and x22,x22,x8
      li x4,0x9DE991B56F6B85FB      
      sb x4,0x00000000D(x22)      
      divu x28,x27,x14      
      srl x16,x25,x22      
      li x22,0x0001000A8      
      and x22,x22,x8
      li x4,0xB3F5D9C72DA4D9E6      
      dcache.civa x22      
      sltiu x30,x0,0xFFFFFFFFFFFFFCCF      
      mulhu x6,x28,x26      
      lui x17,0x4BF35      
      li x22,0x000107ABA      
      and x22,x22,x8
      li x4,0x17E549001015D5DB      
      lbu x5,0x000000026(x22)      
      sub x20,x28,x11      
      slli x12,x18,0x016      
      li x22,0x000107C7A      
      and x22,x22,x8
      li x4,0xB500F18B2D0D5671      
      sb x4,0x000000023(x22)      
      addi x6,x27,0xFFFFFFFFFFFFF94A      
      sll x12,x21,x5      
      li x22,0x00010210A      
      and x22,x22,x8
      li x4,0xCFDCAD1892AE8CA2      
      lhu x5,0x000000016(x22)      
      lui x25,0xF2CFF      
      srai x29,x7,0x07      
      remu x30,x14,x12      
      li x22,0x000100163      
      and x22,x22,x8
      li x4,0x74A7670B6916CB70      
      dcache.civa x22      
      xor x16,x10,x16      
      xor x11,x7,x7      
      li x22,0x000107C1C      
      and x22,x22,x8
      li x4,0xB95F03CE50B05BB6      
      dcache.civa x22      
      mul x12,x11,x19      
      remu x10,x14,x6      
      sll x27,x4,x13      
      li x22,0x000107ACC      
      and x22,x22,x8
      li x4,0x8F0B3EAF53D6703E      
      lwu x5,0x000000010(x22)      
      sra x15,x17,x6      
      sltu x7,x3,x5      
      slli x9,x29,0x04      
      li x22,0x000107A10      
      and x22,x22,x8
      li x4,0x16B8741FDD292F02      
      sb x4,0x00000003A(x22)      
      mulh x30,x22,x9      
      andi x21,x29,0x00000030B      
      addi x13,x24,0xFFFFFFFFFFFFFAD4      
      li x22,0x000100182      
      and x22,x22,x8
      li x4,0x2C6ACA57FBF0F192      
      lh x5,0x000000032(x22)      
      mulh x9,x28,x1      
      auipc x27,0x18CBE      
      li x22,0x000107826      
      and x22,x22,x8
      li x4,0x9F6555414F6E0E11      
      lbu x5,0x00000003C(x22)      
      slli x29,x22,0x06      
      srli x12,x10,0x01B      
      andi x10,x23,0x000000093      
      li x22,0x000104076      
      and x22,x22,x8
      li x4,0xAD27B7E6216480AE      
      sw x4,0x000000050(x22)      
      sltiu x9,x12,0xFFFFFFFFFFFFFEF7      
      remu x7,x13,x14      
      li x22,0x0001020C7      
      and x22,x22,x8
      li x4,0x8D573CB5F624F474      
      lw x5,0x000000004(x22)      
      divu x26,x14,x7      
      sltiu x12,x11,0x0000000B5      
      slli x20,x5,0x06      
      li x22,0x000104184      
      and x22,x22,x8
      li x4,0xCD8B18286FB094B      
      dcache.cva x22      
      slti x28,x9,0xFFFFFFFFFFFFFA7A      
      slti x11,x8,0xFFFFFFFFFFFFFD57      
      srl x9,x10,x18      
      li x22,0x000107795      
      and x22,x22,x8
      li x4,0x796B574EC9053A38      
      dcache.cva x22      
      sll x14,x15,x8      
      xori x13,x25,0x00000029D      
      xori x23,x9,0x000000751      
      li x22,0x000102172      
      and x22,x22,x8
      li x4,0xC9D49A48C49DCE0B      
      lw x5,0x000000018(x22)      
      slli x13,x4,0x0F      
      slli x25,x26,0x014      
      xor x19,x17,x10      
      li x22,0x000107A37      
      and x22,x22,x8
      li x4,0x8ED821D1DFF3C0B4      
      dcache.cva x22      
      slt x14,x8,x9      
      mul x20,x5,x3      
      li x22,0x0001079B4      
      and x22,x22,x8
      li x4,0xA32B1B2824876374      
      dcache.cva x22      
      rem x23,x2,x10      
      sll x9,x6,x26      
      li x22,0x000104134      
      and x22,x22,x8
      li x4,0x6DB478604D5759CE      
      lh x5,0x000000028(x22)      
      divu x27,x19,x20      
      sll x10,x0,x31      
      li x22,0x000107CB4      
      and x22,x22,x8
      li x4,0x96D5E805BFE110F1      
      lb x5,0x000000017(x22)      
      ori x27,x24,0xFFFFFFFFFFFFFECA      
      mulh x7,x18,x22      
      li x22,0x000107C9F      
      and x22,x22,x8
      li x4,0xB03D40EDB819D7F4      
      sh x4,0x000000016(x22)      
      andi x18,x11,0x00000037E      
      sltu x20,x28,x5      
      slti x20,x15,0xFFFFFFFFFFFFFE7C      
      li x22,0x000102048      
      and x22,x22,x8
      li x4,0x6473D014649D5BD1      
      dcache.civa x22      
      srai x31,x8,0x010      
      div x23,x5,x10      
      li x22,0x000107C32      
      and x22,x22,x8
      li x4,0xC2923F9733111792      
      dcache.cva x22      
      rem x14,x13,x1      
      add x21,x31,x6      
      li x22,0x0001077F2      
      and x22,x22,x8
      li x4,0x9F725D3D8D5270DF      
      lhu x5,0x00000003E(x22)      
      add x20,x27,x12      
      auipc x6,0x9569E      
      add x20,x2,x4      
      li x22,0x000104101      
      and x22,x22,x8
      li x4,0x35457660C0AD007A      
      lw x5,0x00000003C(x22)      
      mul x24,x7,x15      
      srl x21,x18,x7      
      div x11,x3,x4      
      li x22,0x000100154      
      and x22,x22,x8
      li x4,0x520A1AE32212F23A      
      sd x4,0x000000038(x22)      
      slti x21,x14,0x00000076F      
      andi x23,x19,0x000000435      
      and x26,x26,x24      
      li x22,0x00010416C      
      and x22,x22,x8
      li x4,0xE62A4F2CC57798AA      
      dcache.civa x22      
      mulhsu x14,x21,x15      
      mulhu x9,x1,x11      
      slt x17,x30,x6      
      li x22,0x00010009C      
      and x22,x22,x8
      li x4,0xBB89862FB9EC9561      
      sw x4,0x000000004(x22)      
      sll x12,x20,x15      
      sra x17,x10,x20      
      li x22,0x000104074      
      and x22,x22,x8
      li x4,0xE14A8AA069613B8B      
      dcache.cva x22      
      div x6,x3,x23      
      remu x11,x30,x24      
      li x22,0x000107B80      
      and x22,x22,x8
      li x4,0xB346AB510FFD9E68      
      sb x4,0x000000016(x22)      
      andi x28,x18,0x000000134      
      sub x11,x12,x31      
      li x22,0x000107946      
      and x22,x22,x8
      li x4,0xB9EA643E31E389B6      
      dcache.civa x22      
      mulhu x13,x18,x13      
      mul x30,x5,x26      
      li x22,0x000107981      
      and x22,x22,x8
      li x4,0x7264957D70A0A1D3      
      lhu x5,0x000000048(x22)      
      srli x11,x11,0x01F      
      mulhu x26,x28,x13      
      li x22,0x000100193      
      and x22,x22,x8
      li x4,0x5E6E6E5EDA33A45B      
      dcache.civa x22      
      mulhsu x20,x23,x19      
      and x14,x28,x5      
      li x22,0x000100015      
      and x22,x22,x8
      li x4,0xD3FEA6F04F219BDB      
      sw x4,0x000000040(x22)      
      remu x15,x23,x20      
      remu x15,x1,x11      
      add x7,x9,x18      
      li x22,0x000107812      
      and x22,x22,x8
      li x4,0xE7F70E6357B12971      
      dcache.cva x22      
      xori x11,x11,0x00000070C      
      auipc x21,0x5DEF5      
      auipc x23,0xC812A      
      li x22,0x00010214C      
      and x22,x22,x8
      li x4,0x2BA7B160B9019B58      
      sw x4,0x000000048(x22)      
      srli x26,x30,0x0C      
      add x6,x9,x17      
      li x22,0x0001040D6      
      and x22,x22,x8
      li x4,0x39437B2F4538464F      
      sd x4,0x000000008(x22)      
      ori x27,x30,0xFFFFFFFFFFFFFF13      
      andi x29,x25,0xFFFFFFFFFFFFF9D5      
      slli x11,x31,0x01A      
      li x22,0x000104118      
      and x22,x22,x8
      li x4,0x7698730912A19B26      
      dcache.civa x22      
      srai x18,x2,0x0F      
      or x25,x5,x12      
      li x22,0x00010781A      
      and x22,x22,x8
      li x4,0xD57FA8378AA4379E      
      sh x4,0x00000004C(x22)      
      slli x15,x24,0x013      
      and x10,x5,x30      
      addi x11,x19,0x0000003B2      
      li x22,0x000104178      
      and x22,x22,x8
      li x4,0xE520E904F851AC8A      
      dcache.civa x22      
      xori x29,x1,0x000000230      
      ori x15,x0,0x000000356      
      and x18,x16,x15      
      li x22,0x0001040DA      
      and x22,x22,x8
      li x4,0xC8EE1A4D8E1D513F      
      sd x4,0x000000040(x22)      
      sra x16,x24,x2      
      sltiu x21,x18,0xFFFFFFFFFFFFF933      
      ori x13,x19,0xFFFFFFFFFFFFFDCA      
      li x22,0x000102057      
      and x22,x22,x8
      li x4,0x201733E7C846D9F1      
      ld x5,0x000000020(x22)      
      mulhsu x12,x24,x12      
      remu x19,x31,x9      
      sll x31,x20,x7      
      li x22,0x0001077DA      
      and x22,x22,x8
      li x4,0xAC8CD3E444C69078      
      dcache.civa x22      
      sll x18,x3,x31      
      mul x10,x24,x1      
      li x22,0x000107CDB      
      and x22,x22,x8
      li x4,0x58AAB4B647D38A35      
      ld x5,0x000000010(x22)      
      sltu x7,x22,x11      
      addi x14,x10,0xFFFFFFFFFFFFFF51      
      slli x11,x28,0x0C      
      li x22,0x000107BE0      
      and x22,x22,x8
      li x4,0xD76A718BEEF54296      
      dcache.civa x22      
      div x6,x7,x19      
      xor x27,x6,x30      
      li x22,0x000100123      
      and x22,x22,x8
      li x4,0x4A6704A1859CB45B      
      sw x4,0x000000034(x22)      
      add x25,x15,x1      
      mulhsu x6,x17,x9      
      li x22,0x000107AC0      
      and x22,x22,x8
      li x4,0x84BE176C6648EB83      
      dcache.cva x22      
      slt x12,x16,x21      
      sltiu x6,x12,0xFFFFFFFFFFFFFBC9      
      li x22,0x000107766      
      and x22,x22,x8
      li x4,0xEF4322A850571319      
      dcache.cva x22      
      auipc x12,0x699F8      
      or x11,x17,x23      
      divu x20,x0,x19      
      li x22,0x0001079E6      
      and x22,x22,x8
      li x4,0x381DB073AC58A9C5      
      dcache.cva x22      
      srli x29,x16,0x01C      
      sub x29,x19,x28      
      auipc x28,0xEC6C      
      li x22,0x0001000CB      
      and x22,x22,x8
      li x4,0xA3B814F10F943920      
      lhu x5,0x000000018(x22)      
      sra x23,x20,x9      
      div x10,x25,x27      
      add x6,x12,x17      
      li x22,0x000107B7C      
      and x22,x22,x8
      li x4,0x646E05D52EE37EA2      
      dcache.civa x22      
      remu x7,x14,x10      
      remu x14,x14,x7      
      li x22,0x00010780B      
      and x22,x22,x8
      li x4,0xFE2A85AC6EDDFE3D      
      sd x4,0x000000000(x22)      
      andi x14,x19,0xFFFFFFFFFFFFF994      
      mulhsu x7,x28,x28      
      li x22,0x000104004      
      and x22,x22,x8
      li x4,0x64989996BBE87E0B      
      dcache.civa x22      
      slli x17,x7,0x01E      
      sll x23,x21,x19      
      or x24,x25,x24      
      li x22,0x000107CFE      
      and x22,x22,x8
      li x4,0x74FE977368D49C84      
      lhu x5,0x000000040(x22)      
      slli x12,x1,0x016      
      mul x20,x27,x30      
      li x22,0x00010782B      
      and x22,x22,x8
      li x4,0x1BDC3BB4204F5A96      
      dcache.cva x22      
      and x9,x24,x15      
      slti x28,x11,0x0000001F1      
      mul x18,x19,x21      
      li x22,0x000107824      
      and x22,x22,x8
      li x4,0x1627A216B2C628F7      
      dcache.cva x22      
      lui x27,0xF8B4D      
      sltiu x17,x31,0xFFFFFFFFFFFFFFA2      
      li x22,0x00010782C      
      and x22,x22,x8
      li x4,0xE1BE142D2772CA4E      
      lbu x5,0x000000017(x22)      
      slli x26,x17,0x07      
      and x27,x14,x29      
      xori x26,x17,0x000000658      
      li x22,0x00010208C      
      and x22,x22,x8
      li x4,0xDF586D0391B2EB29      
      dcache.cva x22      
      addi x14,x10,0x00000079C      
      slt x31,x11,x7      
      div x29,x24,x25      
      li x22,0x000102149      
      and x22,x22,x8
      li x4,0x6CFBB956C3068D60      
      lhu x5,0x00000003C(x22)      
      rem x15,x17,x24      
      srai x10,x3,0x02      
      li x22,0x000107721      
      and x22,x22,x8
      li x4,0xC0B4DCDE019ECB40      
      lw x5,0x00000001C(x22)      
      addi x17,x18,0xFFFFFFFFFFFFFF89      
      sll x12,x17,x24      
      li x22,0x00010211D      
      and x22,x22,x8
      li x4,0xA71E310DCE119ED4      
      dcache.cva x22      
      and x6,x2,x14      
      sub x6,x4,x2      
      slti x31,x26,0x0000004E8      
      li x22,0x000107B17      
      and x22,x22,x8
      li x4,0x96BBB5943B80DD50      
      dcache.civa x22      
      sltu x30,x12,x15      
      xori x9,x11,0x0000001E8      
      sra x17,x13,x0      
      li x22,0x00010214D      
      and x22,x22,x8
      li x4,0x3F0ED0D82B031221      
      sb x4,0x00000002A(x22)      
      xor x12,x3,x11      
      srl x12,x17,x12      
      auipc x23,0xBF7F1      
      li x22,0x0001079D7      
      and x22,x22,x8
      li x4,0xAD1C63789565EB36      
      dcache.cva x22      
      sll x28,x7,x31      
      sltu x17,x5,x5      
      srl x19,x14,x0      
      li x22,0x000107B2A      
      and x22,x22,x8
      li x4,0x80EA50C80845E7DB      
      ld x5,0x000000038(x22)      
      srai x12,x7,0x0E      
      srl x9,x1,x29      
      ori x19,x24,0xFFFFFFFFFFFFFF1F      
      li x22,0x0001040FB      
      and x22,x22,x8
      li x4,0x5D9FA8065EA6C306      
      lb x5,0x000000047(x22)      
      addi x18,x18,0xFFFFFFFFFFFFFD21      
      ori x29,x17,0x0000001CD      
      li x22,0x000107726      
      and x22,x22,x8
      li x4,0xC6C618908FE66FE9      
      sw x4,0x000000044(x22)      
      div x30,x24,x31      
      lui x30,0xB2DBB      
      sub x28,x15,x15      
      li x22,0x00010413E      
      and x22,x22,x8
      li x4,0x33F5FEADAF3A1F1B      
      lbu x5,0x000000015(x22)      
      or x14,x30,x1      
      slli x9,x4,0x011      
      xori x24,x27,0x00000056B      
      li x22,0x000107955      
      and x22,x22,x8
      li x4,0x82F18F86251A1FBB      
      dcache.civa x22      
      and x25,x14,x20      
      divu x11,x13,x19      
      mulh x23,x5,x10      
      li x22,0x0001001C6      
      and x22,x22,x8
      li x4,0x611493BBDA6713A5      
      lw x5,0x000000048(x22)      
      mulhu x31,x10,x15      
      slti x26,x15,0x000000475      
      slli x18,x2,0x07      
      li x22,0x00010408B      
      and x22,x22,x8
      li x4,0xDCDC4B5B0F2E12A4      
      sd x4,0x000000038(x22)      
      sltu x28,x21,x20      
      mulh x17,x10,x9      
      li x22,0x000104041      
      and x22,x22,x8
      li x4,0xCDEC7A9A6C7C3F0C      
      dcache.cva x22      
      mul x30,x5,x22      
      addi x6,x5,0x0000007C7      
      lui x15,0xBB116      
      li x22,0x000107890      
      and x22,x22,x8
      li x4,0x27C508E8FB7C6929      
      dcache.civa x22      
      lui x10,0x74F2B      
      sll x26,x17,x5      
      li x22,0x000107B12      
      and x22,x22,x8
      li x4,0xD1F2112B167CF58E      
      dcache.civa x22      
      andi x13,x28,0xFFFFFFFFFFFFFD69      
      slt x13,x4,x6      
      li x22,0x00010788C      
      and x22,x22,x8
      li x4,0xB8705B52D8150D6      
      lh x5,0x00000002E(x22)      
      divu x31,x27,x7      
      xori x17,x28,0x0000005A3      
      li x22,0x000107A3C      
      and x22,x22,x8
      li x4,0x172FAEDC406D30E1      
      dcache.cva x22      
      mulhu x13,x7,x8      
      srl x7,x16,x12      
      li x22,0x0001021FD      
      and x22,x22,x8
      li x4,0x48337468ABF617FF      
      lw x5,0x000000044(x22)      
      sub x27,x15,x26      
      div x29,x6,x4      
      sra x9,x27,x23      
      li x22,0x000107B89      
      and x22,x22,x8
      li x4,0x43EDB31D023737F9      
      dcache.cva x22      
      xor x15,x30,x18      
      div x13,x1,x2      
      divu x26,x5,x20      
      li x22,0x000107CEA      
      and x22,x22,x8
      li x4,0xFD6D53CF79B25055      
      dcache.civa x22      
      addi x23,x4,0xFFFFFFFFFFFFF960      
      addi x24,x0,0xFFFFFFFFFFFFFCE2      
      or x19,x11,x6      
      li x22,0x0001020CA      
      and x22,x22,x8
      li x4,0xA06B8DFFF5218BC2      
      lwu x5,0x000000034(x22)      
      lui x30,0xFE6A0      
      sltiu x10,x8,0xFFFFFFFFFFFFF82C      
      add x13,x12,x5      
      li x22,0x000102045      
      and x22,x22,x8
      li x4,0xBA216D205DF7D615      
      lhu x5,0x000000026(x22)      
      div x16,x21,x17      
      sra x17,x20,x1      
      li x22,0x000107755      
      and x22,x22,x8
      li x4,0x2D481B04F31856E2      
      dcache.cva x22      
      slli x11,x11,0x01      
      and x19,x0,x6      
      li x22,0x0001078AD      
      and x22,x22,x8
      li x4,0xDA76E38E96781643      
      sb x4,0x000000032(x22)      
      sltiu x6,x19,0xFFFFFFFFFFFFFADF      
      sra x18,x14,x20      
      li x22,0x000107CDD      
      and x22,x22,x8
      li x4,0xBC4F74A80F077F0D      
      dcache.civa x22      
      andi x24,x31,0xFFFFFFFFFFFFFE92      
      sra x31,x12,x8      
      li x22,0x000102164      
      and x22,x22,x8
      li x4,0xF8CD272B06A0CD67      
      dcache.civa x22      
      divu x10,x30,x31      
      srli x15,x14,0x01      
      li x22,0x0001021C6      
      and x22,x22,x8
      li x4,0xCA41B932CAC77251      
      sb x4,0x00000000A(x22)      
      sub x9,x3,x11      
      divu x7,x19,x26      
      li x22,0x0001077EA      
      and x22,x22,x8
      li x4,0x958AA5E1CA01CD79      
      lhu x5,0x000000024(x22)      
      srl x13,x14,x19      
      addi x16,x0,0x000000247      
      mul x7,x6,x23      
      li x22,0x00010780A      
      and x22,x22,x8
      li x4,0x15CF89767708BDCE      
      sh x4,0x000000030(x22)      
      addi x15,x6,0x0000007DE      
      or x24,x11,x4      
      li x22,0x000107A77      
      and x22,x22,x8
      li x4,0x58443FFCEAA16326      
      dcache.civa x22      
      srl x16,x8,x22      
      sltu x17,x0,x22      
      li x22,0x000100131      
      and x22,x22,x8
      li x4,0xC10099570DDE2C83      
      dcache.cva x22      
      sra x7,x23,x18      
      slti x23,x29,0x0000002F8      
      li x22,0x00010010B      
      and x22,x22,x8
      li x4,0xAEB3802A3F1299D9      
      sd x4,0x000000038(x22)      
      mulhsu x28,x6,x19      
      mulhsu x26,x1,x24      
      li x22,0x000107C5F      
      and x22,x22,x8
      li x4,0xBE792159B956663E      
      sd x4,0x000000000(x22)      
      xori x7,x4,0x000000372      
      xor x14,x10,x0      
      mulhu x10,x12,x23      
      li x22,0x0001040D6      
      and x22,x22,x8
      li x4,0x53D705EA7C06FFC5      
      sb x4,0x000000036(x22)      
      xori x30,x31,0x00000040A      
      xori x13,x30,0x0000000C0      
      li x22,0x0001000A3      
      and x22,x22,x8
      li x4,0x635F9E7E1266B8D2      
      sh x4,0x000000008(x22)      
      andi x31,x1,0xFFFFFFFFFFFFFC3E      
      sll x30,x22,x26      
      li x22,0x000107A3A      
      and x22,x22,x8
      li x4,0xCBCD09750FFC7663      
      dcache.civa x22      
      xori x7,x22,0xFFFFFFFFFFFFFD51      
      sll x6,x15,x22      
      li x22,0x000107C01      
      and x22,x22,x8
      li x4,0x426374F4EF2961F5      
      dcache.civa x22      
      srai x15,x23,0x0D      
      or x17,x24,x0      
      andi x24,x4,0xFFFFFFFFFFFFF8B9      
      li x22,0x000107901      
      and x22,x22,x8
      li x4,0xF9CEC82592679F63      
      lh x5,0x000000016(x22)      
      andi x26,x8,0xFFFFFFFFFFFFFBD4      
      xori x23,x3,0xFFFFFFFFFFFFFC9B      
      li x22,0x000107706      
      and x22,x22,x8
      li x4,0xB9CE1F10D8245270      
      dcache.civa x22      
      divu x30,x27,x17      
      mul x14,x23,x12      
      li x22,0x0001040C3      
      and x22,x22,x8
      li x4,0xF0D1E21B2C5E9779      
      lbu x5,0x000000034(x22)      
      addi x28,x13,0xFFFFFFFFFFFFF993      
      sub x11,x18,x4      
      li x22,0x000104177      
      and x22,x22,x8
      li x4,0x12747D4713DF99E6      
      dcache.cva x22      
      sltiu x11,x26,0xFFFFFFFFFFFFFE54      
      add x31,x24,x24      
      sltiu x15,x27,0xFFFFFFFFFFFFFC5C      
      li x22,0x0001020C6      
      and x22,x22,x8
      li x4,0x12846021A79DC841      
      sh x4,0x000000008(x22)      
      mulh x31,x22,x19      
      slli x14,x12,0x01B      
      li x22,0x0001000BC      
      and x22,x22,x8
      li x4,0x301131BA43A7A06E      
      sh x4,0x000000024(x22)      
      div x14,x20,x9      
      xori x14,x2,0xFFFFFFFFFFFFFCDC      
      li x22,0x000107B30      
      and x22,x22,x8
      li x4,0x705A219070FD5FA      
      ld x5,0x000000048(x22)      
      sra x20,x0,x16      
      div x6,x12,x25      
      li x22,0x000100192      
      and x22,x22,x8
      li x4,0x64B11D553DC2EE57      
      lh x5,0x00000004C(x22)      
      divu x26,x12,x4      
      sltiu x31,x8,0xFFFFFFFFFFFFFDA2      
      mulhu x23,x12,x21      
      li x22,0x000100016      
      and x22,x22,x8
      li x4,0x5EB2F10BAC9D4995      
      dcache.cva x22      
      auipc x30,0x1C43D      
      divu x21,x2,x3      
      li x22,0x000107B40      
      and x22,x22,x8
      li x4,0x7343302BECDB876D      
      lb x5,0x00000004D(x22)      
      and x31,x29,x16      
      sltiu x11,x26,0xFFFFFFFFFFFFFAE3      
      sra x25,x27,x12      
      li x22,0x000104135      
      and x22,x22,x8
      li x4,0xD8546D90A68DF229      
      ld x5,0x000000038(x22)      
      ori x10,x8,0xFFFFFFFFFFFFFCD5      
      mulhu x21,x14,x2      
      or x29,x25,x7      
      li x22,0x000107B84      
      and x22,x22,x8
      li x4,0xCDDB185B6A4D8A24      
      sw x4,0x000000004(x22)      
      xor x26,x25,x7      
      xori x9,x2,0xFFFFFFFFFFFFFE48      
      li x22,0x000100080      
      and x22,x22,x8
      li x4,0xF0FC20617C9E69A2      
      sd x4,0x000000010(x22)      
      or x27,x14,x4      
      sll x6,x2,x15      
      lui x13,0x6E68B      
      li x22,0x0001021ED      
      and x22,x22,x8
      li x4,0x84C0C81E7765BA4      
      dcache.civa x22      
      rem x25,x20,x14      
      ori x17,x27,0x00000002B      
      li x22,0x000107ADF      
      and x22,x22,x8
      li x4,0x6C82E60C8756BF1      
      sb x4,0x00000003E(x22)      
      ori x6,x23,0x0000002DB      
      sra x27,x9,x3      
      li x22,0x0001040BF      
      and x22,x22,x8
      li x4,0xEA4AB402CA5FAF36      
      lbu x5,0x000000038(x22)      
      srl x11,x12,x31      
      div x9,x12,x31      
      li x22,0x0001000E3      
      and x22,x22,x8
      li x4,0xF54A9A26F59FCA1F      
      dcache.civa x22      
      divu x31,x29,x4      
      addi x27,x23,0xFFFFFFFFFFFFFFF1      
      and x14,x13,x24      
      li x22,0x000100086      
      and x22,x22,x8
      li x4,0xC40B9EB9384D0CA2      
      dcache.cva x22      
      slt x12,x13,x5      
      mul x26,x22,x27      
      mulhu x7,x5,x17      
      li x22,0x00010006A      
      and x22,x22,x8
      li x4,0x88C153E0D23AC19B      
      sw x4,0x00000002C(x22)      
      sltiu x27,x11,0x000000342      
      div x27,x31,x11      
      li x22,0x000104128      
      and x22,x22,x8
      li x4,0xE4DB2087AEAD3798      
      ld x5,0x000000040(x22)      
      sltiu x28,x17,0xFFFFFFFFFFFFFFEA      
      srli x16,x4,0x010      
      andi x9,x13,0xFFFFFFFFFFFFFACF      
      li x22,0x0001040A6      
      and x22,x22,x8
      li x4,0xB7287F6B01DF099      
      sw x4,0x000000004(x22)      
      xor x7,x25,x20      
      sub x25,x5,x5      
      li x22,0x000100127      
      and x22,x22,x8
      li x4,0x49586545AF410958      
      dcache.civa x22      
      mulhu x13,x2,x4      
      sub x11,x10,x29      
      auipc x23,0x5058B      
      li x22,0x0001021C8      
      and x22,x22,x8
      li x4,0x777C9444F4047E73      
      sw x4,0x000000014(x22)      
      auipc x31,0xE8A6E      
      sll x14,x14,x8      
      li x22,0x000107ACA      
      and x22,x22,x8
      li x4,0xC06E30B431201310      
      sw x4,0x000000020(x22)      
      addi x24,x22,0xFFFFFFFFFFFFF874      
      sltiu x12,x13,0x000000726      
      li x22,0x0001021B7      
      and x22,x22,x8
      li x4,0xC8A6F4B6C2FAC7BB      
      lhu x5,0x000000008(x22)      
      srl x24,x28,x31      
      mulh x31,x11,x23      
      li x22,0x00010404E      
      and x22,x22,x8
      li x4,0x4D354ED09E1A9979      
      sw x4,0x000000040(x22)      
      slti x20,x21,0xFFFFFFFFFFFFFA0A      
      mulhsu x30,x7,x16      
      or x15,x9,x7      
      li x22,0x000107A9B      
      and x22,x22,x8
      li x4,0x7EB83D41C948969B      
      dcache.cva x22      
      srli x21,x2,0x06      
      mul x28,x4,x3      
      srai x19,x9,0x02      
      li x22,0x000104010      
      and x22,x22,x8
      li x4,0x6A6B0C3EB963F4F7      
      sh x4,0x000000000(x22)      
      sra x25,x25,x0      
      xor x24,x0,x20      
      li x22,0x000102016      
      and x22,x22,x8
      li x4,0x96E83B342B52D145      
      sb x4,0x000000019(x22)      
      add x29,x6,x29      
      div x25,x1,x18      
      li x22,0x0001020C8      
      and x22,x22,x8
      li x4,0x45242151CD970C9C      
      lw x5,0x000000020(x22)      
      sub x21,x26,x22      
      slti x7,x2,0x000000057      
      andi x11,x13,0x00000015E      
      li x22,0x0001077FD      
      and x22,x22,x8
      li x4,0x41B2C81B5D63824D      
      lb x5,0x00000001A(x22)      
      add x28,x3,x23      
      divu x9,x30,x15      
      li x22,0x00010799A      
      and x22,x22,x8
      li x4,0xF557845548CF4F88      
      dcache.cva x22      
      sltiu x9,x22,0x0000001EA      
      mul x29,x27,x11      
      ori x20,x15,0x00000021B      
      li x22,0x000107A6D      
      and x22,x22,x8
      li x4,0x43926FACE2335C74      
      lh x5,0x00000000E(x22)      
      divu x23,x10,x9      
      xor x30,x17,x29      
      wfi
#******this region is added by generator******

